#ifndef CRYSTALSCENEVIEW_H
#define CRYSTALSCENEVIEW_H

#include "shaderprogram.h"
#include "KeyboardMouseHandler.h"
#include "camera.h"
#include "cubeobject.h"

#include <QOpenGLWidget>
#include <QOpenGLFunctions>

class CrystalSceneView : public QOpenGLWidget, protected QOpenGLFunctions
{
public:
    CrystalSceneView(QWidget *parent = nullptr);
    ~CrystalSceneView();

protected:
    void initializeGL() override;
    void paintGL() override;
    void resizeGL(int w, int h) override;

    // Functions to handle key press and mouse press events
    void keyPressEvent(QKeyEvent *event) override;
    void keyReleaseEvent(QKeyEvent *event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;
    void wheelEvent(QWheelEvent *event) override;

private:
    CubeObject cubes;
    QVector<QVector3D> cubePositions;

    ShaderProgram shader;

    Camera camera;
    QMatrix4x4 projection;

    //deltaTime can't work with paintGL and was non used!
    float deltaTime = 0.0f;

    KeyboardMouseHandler     m_keyboardMouseHandler;
    bool                     m_inputEventReceived;

    void checkInput();
    void processInput();

};

#endif // CRYSTALSCENEVIEW_H
