#ifndef CRYSTALSTRUCTUREWIDGET_H
#define CRYSTALSTRUCTUREWIDGET_H

#include "checkboxheader.h"
#include "crystaldata.h"

#include <QWidget>

class QStandardItem;
class QStandardItemModel;
class QSortFilterProxyModel;

typedef struct {
    QString lab;
} scatter_type;

namespace Ui {
class CrystalStructureWidget;
}

class CrystalStructureWidget : public QWidget
{
    Q_OBJECT

public:
    explicit CrystalStructureWidget(QWidget *parent = nullptr);
    ~CrystalStructureWidget();
    void setSymmetry(int phase, double cell[], const QString &spg);
    void setAtoms(const QVector<crydata::AtomProperties> &atoms);
    void getSymmetry(double cell[], bool &cellref) const;    
    bool hasCellChanged();
    bool hasParameterToRefine();    
    void setConstraints(const QVector<crydata::ConstraintProperties> &newConstr);
    void setRefineEnabled(bool enable);

private slots:
    void headerCheckBoxClicked(bool state, int column);
    void checkBoxClicked(const QModelIndex &index, bool state);
    void atomChanged(QStandardItem *item);
    void markSelectedAtoms();
    void restraintsPushButtonClicked();
    void constraintsPushButtonClicked();
    void showModelPushButtonClicked();

private:
    Ui::CrystalStructureWidget *ui;
    QStandardItemModel *atomsModel;
    QSortFilterProxyModel *atomsProxyModel;
    QStandardItemModel *bondRestraintsModel;
    QSortFilterProxyModel *bondRestraintsProxyModel;
    CheckBoxHeader *cbHeader;
    bool tableChangedEnabled;
    int phaseId;
    QVector<crydata::ConstraintProperties> constr;
    enum {ID_COL, LABEL_COL, X_COL, Y_COL, Z_COL, XYZREF_COL, B_COL, BREF_COL, OCC_COL, OCCREF_COL};

    void createAtomsModel();
    void createRestraintsModel();
    void updateAtomsModel(const QVector<crydata::AtomProperties> &atoms);
    void setAtomsTableView();
    int rowCheckBoxClickedCount(int col) const;
    void put_restraints(const QVector<crydata::RestraintProperties> &restr, const QVector<double> &wei) const;
    bool isParInConstraint(int atom, int par);
    int parInConstraint(int atom, int par);
    void selectParInConstraint(int atom, int par, int state);
    void setItemColor(const QVector<crydata::AtomProperties> &atoms);
    void setRefineAtomsEnabled(bool enable);
};

#endif // CRYSTALSTRUCTUREWIDGET_H
