#include "directmethodsdialog.h"
#include "ui_directmethodsdialog.h"

#include <QPushButton>
#include <QDebug>

extern "C" void finish_dm(const char *cell_cont, int len_cont, const char *space, int len_space, double cell[], int alltrials);
extern "C" void get_symmetry_info(int phaseid, int *spgIndex, char *spaceg, double cell[6]);
extern "C" int get_cell_content_len();
extern "C" void get_cell_content(char *c_content);

DirectMethodsDialog::DirectMethodsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DirectMethodsDialog)
{
    ui->setupUi(this);

    ui->buttonBox->button(QDialogButtonBox::Ok)->setText("Run");
    ui->buttonBox->button(QDialogButtonBox::Ok)->setIcon(QIcon(":/pixmaps/pixmaps/go.png"));

    connect(ui->symmetryWidget, &SymmetryWidget::spaceWidgetChanged, this, [=]() {
        ui->contentWidget->updateInfo(ui->symmetryWidget->cellVolume());
        enableOkButton();
    });

    connect(ui->symmetryWidget, &SymmetryWidget::cellWidgetChanged, this, [=](){
        ui->contentWidget->updateInfo(ui->symmetryWidget->cellVolume());
        enableOkButton();
    });

    connect(ui->contentWidget, &FormulaWidget::formulaEdited, this, &DirectMethodsDialog::enableOkButton);
}

DirectMethodsDialog::~DirectMethodsDialog()
{
    delete ui;
}

void DirectMethodsDialog::setDialog()
{
    int spgIndex;
    char *spaceg = new char[40];
    double cell[6];
    get_symmetry_info(1, &spgIndex, spaceg, cell);

    ui->symmetryWidget->setData(cell, spgIndex, QString::fromUtf8(spaceg));

    delete [] spaceg;
    int lenc = get_cell_content_len();
    if (lenc > 0) {
        char *content = new char[lenc+1];
        get_cell_content(content);        
        ui->contentWidget->updateFormula(QString::fromUtf8(content), ui->symmetryWidget->cellVolume());
        delete [] content;
    }

    enableOkButton();

    //To prevent that enter key close the dialog. Call this after the show. See also accept()
    ui->buttonBox->button(QDialogButtonBox::Ok)->setDefault(false);
    ui->buttonBox->button(QDialogButtonBox::Ok)->setAutoDefault(false);
}

void DirectMethodsDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Ok) {
        if(!ui->buttonBox->button(QDialogButtonBox::Ok)->hasFocus()) return;
        QString spg;
        (void) ui->symmetryWidget->getSpaceGroup(spg);
        double cell[6];
        ui->symmetryWidget->getCellParameters(cell);
        this->hide();        
        finish_dm(ui->contentWidget->formula().toStdString().c_str(),
                  ui->contentWidget->formula().length(),
                  spg.toStdString().c_str(),                  
                  spg.length(),
                  cell,
                  ui->allTrialsCheckBox->isChecked()?1:0);
    } else if (stdButton == QDialogButtonBox::Cancel) {

    }
}

void DirectMethodsDialog::enableOkButton()
{
    bool enable = ui->contentWidget->isValidFormula() &&
            ui->symmetryWidget->isValidSpaceGroup() &&
            ui->symmetryWidget->isValidCell();

    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(enable);
}

void DirectMethodsDialog::accept()
{    
    //'ok' get focus if you hide the dialog. This avoid that enter key close the dialog
    if (!ui->buttonBox->button(QDialogButtonBox::Ok)->hasFocus()) return;

    QDialog::accept();
}
