#include "extrabackgrounddialog.h"
#include "ui_extrabackgrounddialog.h"
#include "customplotzoom.h"

extern "C" int get_intervals_number();
extern "C" void get_back_ncoeff(int ncoeff[]);
extern "C" void get_intervals_limit(float tmin[], float tmax[]);
extern "C" void change_polynomial(int interval, int ncoeff);

ExtraBackgroundDialog::ExtraBackgroundDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ExtraBackgroundDialog)
{
    ui->setupUi(this);

    connect(ui->buttonBox, &QDialogButtonBox::clicked, this, &ExtraBackgroundDialog::buttonBoxClicked);
    connect(ui->intervalsComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &ExtraBackgroundDialog::intervalSelected);
    connect(ui->coeffSpinBox, QOverload<int>::of(&QSpinBox::valueChanged), this, &ExtraBackgroundDialog::coeffSpinBoxChanged);
}

void ExtraBackgroundDialog::setBackground(CustomPlotZoom *plot)
{
    mPlot = plot;
    int nIntervals = get_intervals_number() -1; //-1 remove single peak interval

    auto nc = new int[nIntervals];
    get_back_ncoeff(nc);
    ncoeff.resize(nIntervals);
    for (int i = 0; i < nIntervals; i++) ncoeff[i] = nc[i];
    delete [] nc;
    ncoeffSav = ncoeff;

    auto tmin = new float[nIntervals+1];
    auto tmax = new float[nIntervals+1];
    get_intervals_limit(tmin, tmax);
    maxVal.resize(nIntervals);
    minVal.resize(nIntervals);
    for (int i = 0; i < nIntervals; i++) {
        minVal[i] = tmin[i+1];
        maxVal[i] = tmax[i+1];
    }
    delete [] tmin;
    delete [] tmax;

    ui->intervalsComboBox->clear();
    for (int i = 0; i < nIntervals; i++) {
        ui->intervalsComboBox->addItem(QString("Interval n. %1").arg(i+1));
    }
    ui->intervalsComboBox->setCurrentIndex(0);    
}

ExtraBackgroundDialog::~ExtraBackgroundDialog()
{
    delete ui;
}

void ExtraBackgroundDialog::intervalSelected(int index)
{
    if (index < 0) return;

    ui->coeffSpinBox->setValue(ncoeff.at(index));
    mPlot->xAxis->setRange(minVal.at(index),maxVal.at(index));
    mPlot->replot();
}

void ExtraBackgroundDialog::buttonBoxClicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Ok) {
        for (int i = 0; i < ncoeff.size(); i++) {
            if (ncoeff.at(i) != ncoeffSav.at(i)) {
                change_polynomial(i+1, ncoeff.at(i));
            }
        }

    }
    mPlot->xAxis->rescale();
    mPlot->replot();
}

void ExtraBackgroundDialog::coeffSpinBoxChanged(int value)
{
    int id = ui->intervalsComboBox->currentIndex();
    ncoeff[id] = value;    
}
