#include "fourierdialog.h"
#include "ui_fourierdialog.h"

#include <QPushButton>
#include <QDebug>

extern "C" void prefourier(int *natsC, int *nadd);
extern "C" void startfourier(FourierConditions fcond, int *ier);

FourierDialog::FourierDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::FourierDialog),
    fourCond{2, 5, 0, 0, 100., 0},
    fourCondSav(fourCond)
{
    ui->setupUi(this);

    ui->buttonBox->button(QDialogButtonBox::Ok)->setText("Execute");
    ui->buttonBox->button(QDialogButtonBox::Ok)->setIcon(QIcon(":/pixmaps/pixmaps/go.png"));
}

FourierDialog::~FourierDialog()
{
    delete ui;
}

void FourierDialog::setDialog()
{
    int nats;
    prefourier(&nats, &fourCond.npeaks);

    fourCondSav = fourCond;
    setConditions();
    ui->infoLabel->setText(QString("%1 atoms in the current model").arg(nats));
}

void FourierDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Ok) {
        getConditions();
        this->hide();
        int ier;
        startfourier(fourCond, &ier);
    } else if (stdButton == QDialogButtonBox::Cancel) {
        fourCond = fourCondSav;
    }
}

void FourierDialog::setConditions()
{
    if (fourCond.fourType == 1) {
        ui->ObservedRadioButton->setChecked(true);
    } else if (fourCond.fourType == 2) {
        ui->Fo2_FcRadioButton->setChecked(true);
    } else if (fourCond.fourType == 3) {
        ui->Fo_FcRadioButton->setChecked(true);
    }
    ui->peaksToAddSpinBox->setValue(fourCond.npeaks);
    ui->weightCheckBox->setChecked(fourCond.weiType);
    ui->mapCheckBox->setChecked(fourCond.showFourier);
}

void FourierDialog::getConditions()
{
    if (ui->ObservedRadioButton->isChecked()) {
        fourCond.fourType = 1;
    } else if (ui->Fo2_FcRadioButton->isChecked()) {
        fourCond.fourType = 2;
    } else if (ui->Fo_FcRadioButton->isChecked()) {
        fourCond.fourType = 3;
    }
    fourCond.npeaks = ui->peaksToAddSpinBox->value();
    fourCond.weiType = ui->weightCheckBox->isChecked()?1:0;
    fourCond.showFourier = ui->mapCheckBox->isChecked()?1:0;
}
