#ifndef INFODIALOG_H
#define INFODIALOG_H

#include <QDialog>
#include <QVector>
#include <QTextStream>
#include <QStringList>
#include <QStandardItem>
#include <QStandardItemModel>
#include <QItemSelectionModel>
#include <QModelIndex>
#include <QTableView>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QToolTip>
#include "models.h"
#include "myproxymodel.h"

#if defined(JAV_WIDGETS)
class CrystalViewWidget
#else
class JavWindow;
#endif

namespace Ui {
class InfoDialog;
}

class BC : public QGraphicsScene
{
private:
    QVector<QGraphicsItem *> Bars;
    QStringList Tips;

public:
    BC(QObject *parent = 0);
    void populateBarChart(QVector<bar_legend> barLegend, QVector<bar_info> bars);

protected:
    void mouseMoveEvent(QGraphicsSceneMouseEvent * mouseEvent) override;
};

class InfoDialog : public QDialog
{
    Q_OBJECT

private:
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav;
#else
    JavWindow *jav;
#endif
public:
    explicit InfoDialog(QWidget *parent = nullptr);
    ~InfoDialog();
    void FillInfo(int page);
    void FillPages();
    Model *GetModel(int which);
    int  CurrentPage();
    void UpdateInfoSelections();

private:
    Ui::InfoDialog *ui;
    QString &InfoStructure(bool im=true);
    void FillInfoStructure();
    void FillInfoAtoms();
    void FillInfoChains();
    void FillInfoAtoms_PDB();
    void FillInfoBonds();
    void FillInfoAngles();
    void FillInfoTorsions();
    void FillInfoBarChart();
    void FillInfoContacts();
    void UpdateAtomsSelection(QItemSelectionModel *selectionModel);
    void UpdateBondsSelection(QItemSelectionModel *selectionModel);
    void populateAtomsModel(bool ifEdit, bool ifB);
    void populateChainsModel();
    void populateBondsModel(bool ifEdit);
    void populateAnglesModel();
    void populateTorsionsModel();
    void populateContactsModel();
    void updateInfoAtoms(int index=-1);
    QStandardItemModel *CreaLista(QStringList headers);
    void setStdProperties(QTableView *table, bool sorting=true);
    int m_kmodel;
    QStandardItemModel *modelAtoms=nullptr;
    QStandardItemModel *modelChains=nullptr;
    QStandardItemModel *modelBonds=nullptr;
    QStandardItemModel *modelAngles=nullptr;
    QStandardItemModel *modelTorsions=nullptr;
    QStandardItemModel *modelContacts=nullptr;
    BC                 m_Scene;
    MyProxyModel       *modelProxyAtoms=nullptr;
    //void numRow(QStandardItemModel *model, int col, std::vector<int> &iord);
    void numRow(QAbstractItemModel *model, int col, std::vector<int> &iord);
    Model *md;
    bool pgSel = false;
    int selectedResidue = -1;
    int selectedChain = -1;
    int h0;

private slots:
    void on_button_text_clicked();
    void on_button_sequence_clicked();
    void on_button_secondary_clicked();
    void on_exportButton_clicked();
    void on_closeButton_clicked();
    void on_comboModels_currentIndexChanged(int index);
    void reject();
    void atomsSelectionChanged(const QItemSelection &, const QItemSelection &);
    void chainsSelectionChanged(const QItemSelection &, const QItemSelection &);
    void bondsSelectionChanged(const QItemSelection &, const QItemSelection &);
    void anglesSelectionChanged(const QItemSelection &, const QItemSelection &);
    void torsionsSelectionChanged(const QItemSelection &, const QItemSelection &);
    void contactsSelectionChanged(const QItemSelection &, const QItemSelection &);

protected:
    virtual void showEvent(QShowEvent *event);
};

#endif // INFODIALOG_H
