#include "infotablewidget.h"
#include "QHeaderView"

#include <QDebug>

InfoTableWidget::InfoTableWidget(QWidget *parent) : QTableWidget(parent)
{
    setRowCount(8);
    setColumnCount(2);
    verticalHeader()->setDefaultSectionSize(22);
    verticalHeader()->setVisible(false);
    horizontalHeader()->setVisible(false);
    setShowGrid(true);
    setSpan(0,0,1,2);
    setAlternatingRowColors(true);
    setSelectionMode(QAbstractItemView::NoSelection);
    setSelectionBehavior(QAbstractItemView::SelectRows);
    setEditTriggers(QAbstractItemView::NoEditTriggers);
    horizontalHeader()->setSectionResizeMode(1,QHeaderView::Stretch);
    horizontalHeader()->setDefaultSectionSize(150);
    setFocusPolicy(Qt::NoFocus);
}

void InfoTableWidget::writeMessage(const QString &string, int row)
{
    //qInfo() << "MSG: " << row << rowCount() << string;
    if (string.isEmpty()) return;

    if (row == 1) {
        QTableWidgetItem *newItem = new QTableWidgetItem(string);
        newItem->setTextAlignment(Qt::AlignHCenter);
        newItem->setForeground(QColor(Qt::red));
        setItem(row-1,0,newItem);
        return;
    }

    if (row > rowCount()) {
        //if (string.isEmpty()) return;
        setRowCount(row);
    }

    int pos = string.indexOf(":");    
    if (pos < 0) {
        QTableWidgetItem *newItem = new QTableWidgetItem(string);
        newItem->setForeground(QColor(Qt::blue));
        setItem(row-1,0,newItem);
        //setItem(row-1,1,new QTableWidgetItem());
        setSpan(row-1,0,1,2);
    } else {
        QTableWidgetItem *newItem1 = new QTableWidgetItem(string.left(pos+1));
        setItem(row-1,0,newItem1);
        newItem1->setForeground(QColor(Qt::blue));
        if (pos+1 != string.length()) {
            QTableWidgetItem *newItem2 = new QTableWidgetItem(string.mid(pos+1));
            setItem(row-1,1,newItem2);
        }
    }
    resizeColumnToContents(0);
}

void InfoTableWidget::setRowCountMessage(int nrow)
{
//qInfo() << "SET ROW:" << nrow;
    if (nrow == rowCount()) return;    
    setRowCount(nrow);
    clearSpans();
    if (nrow > 0) setSpan(0,0,1,2);
}

int InfoTableWidget::rowFindMessage(const QString &string)
{
    QList<QTableWidgetItem *> items = findItems(string, Qt::MatchExactly);
    if (items.size() > 0) return items.at(0)->row();

    return -1;
}

void InfoTableWidget::updateValue(const QString &string, int row)
{
    QTableWidgetItem *newItem = new QTableWidgetItem(string);
    setItem(row-1,1,newItem);
}

void InfoTableWidget::clearRow(int row)
{
    //qInfo() << "CLEAR ROW:" << row;
    if (row >= rowCount()) return;
    removeRow(row);
}
