#ifndef JAVWINDOW_H
#define JAVWINDOW_H

#include <QMainWindow>
#include <QCloseEvent>
#include <QLabel>
#include <QList>
#include "libmath.h"
#include "libcomune.h"
#include "models.h"
#include "maps.h"
#include "polyhedradialog.h"
#include "coddialog.h"
#include "optionsdialog.h"
#include "hbondsoptionsdialog.h"
#include "infodialog.h"
#include "bondsettingsdialog.h"
#include "myglwidget.h"
#include "powderpattern.h"
#include "pairdist.h"
#include "renumberdialog.h"
#include "geocalc.h"
#include "spheredialog.h"
#include "overlayinfo.h"
#include "centroidsdlg.h"
#include "transformdialog.h"
#include "propertiesdialog.h"
#include "normhydrodlg.h"
#include "bondvalencedialog.h"
#include "mopacdialog.h"
#if !defined(JAV_WIDGET)
enum codici {IFCELL, IFSYMM, PLOTSTYLE, IFLABELS, SYMLEVEL, IFPOLYHEDRA,
             IFGROW, IFHBONDS, IFSHORTC, IFDELHANG};
#endif

namespace Ui {
class JavWindow;
}

class JavWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit JavWindow(int maxSamples=0, QWidget *parent = nullptr);
    ~JavWindow();
    int IsValidModel();
    int GetInfo(int code, int whichModel);
    ColorWidget::ColorType GetBgType();
    void SetUndoRedo(int but, bool stato, char *str=nullptr);
    void readSettings(int which);
    void PassaModello (atom_type *atml, f_bond_type *legm,
                       int *nats, int *nats_au, int *nlegs, int *nlegs_au,
                       element_type *celem, int *nelem,
                       int *which,
                       int *nmodels, int *ifPdb, char *name, bool *ifNewModel,
                       int ifResize=1);
    void PassaMappa(const char *name, int nmaps, int which, int np[],
                int nrefl, float resm, int ifd, float denmin,
                float denmax, float avden, float stden);
    void map_set_visible(int which, int code);
    void UpdateInfoAtoms();
    void updateModify(int active);
    void update_items_modify(bool status, bool updateList = true);
    void aggiornaProperties(int whichModel=0, bool ifSave=false);
    bool IsPreviewMode() {return PreviewMode;}
    void Draw();
    void DrawModels();
    void DrawCentroids();
    void JavDrawText(vet3 pos, QString str, QColor c);
    void setLabelOnOff(bool status);
    void IsAtom(int x, int y);
    void DrawForSelect(bool only_atoms);
    void loadFile(const QString &fileName, const QString &action);
    int selectObject(int x, int y, bool sel_flag = true, bool only_atoms = false);
    int selectObjects(int x, int y, double wp, double hp);
    QString infoAtom(int whichModel, size_t jat);
    QString infoBond(int whichModel, size_t jat, int wtype);
    mat4x4 GetRotm();
    bool jav_modify(int *kptra,int *nptra,int *kptrb,int *nptrb, int *cod,
                    float *threshold = nullptr, int wm = -1, int *spec=nullptr,
                    const char *newname=nullptr, size_t idum=0);
    QStringList FillSpecies(int kadd);
    QStringList FillAtoms();
    QString  ModName(int k);
    Model *GetModel(int k);
    //QColor GetColor(int k) {return Colori[k];}
    void SetPacking(bool ifClick=true);
    void SetLabels();
    void SetCell(bool ifClick=true);
    void ChangeOrtho();
    void ResetLights();
    void SyncToggles();
    void updateCenter();
    bool IsMapVisible(int which);
    void RunMCAll();
    void DrawMaps();
    void RiNormalizza(bool fResize=false);
    void do_actionExpand_to_Molecule();
    void do_move_atoms();
    bool doPopup(int p[], QMenu *menu);
    bool SelectAtoms(int x, int y);
    void SelectAtom(int x, int y);
    void setUnselect();
    void addResult(int p[][2], float result_op, int toclick, QString &strRes);
    void Set_Hint(QString mesg);
    bool DistanceFromUI(QString title, double *val);
    void setMotionType(int whichCalc);
    void updateSphereList(int kfound, int vat[], float vdist[], op_type vsym[]);
    void updateMenuDeleteModel();
    void FillCentroidsDlg();
    void ClearCentroids(int whichModel);
    void AddCentroids(float xyz[3], int natoms, int at[]);
    QString getCurrentFilePath() const;
    void setStatusBarText(const QString &Msg);
    void clearStatusBarText();
    void waitProcess(const QString &Msg);
    void endProcess();
    double getStep();
    double getRotStep();
    void selectAtoms(const QVector<int> &selected, int whichModel=1);
    void selectAtomsAndLegs(const QVector<int> &selected, int whichModel=1);
    void CApplicaSimmetria(int *which);
    int GetNz(int atomic_num);
    void move_center(vet3 posi, int whichModel, int whichAtom, bool ifDrawCenter);
    void move_center(float posi[], int whichModel, int whichAtom, bool ifDrawCenter);
    void saveSettings();
    int importFileCOD(const QString &fileName, int tipo);
    int importFile(const QString &fileName, int tipo);

private slots:
    void closeEvent(QCloseEvent *event);
    bool event(QEvent *event);
    void on_actionImport_Structure_triggered();
    void on_actionExport_Structure_triggered();
    void on_actionExport_Scene_triggered();
    void on_actionScreenshot_triggered();
    void on_actionTetrahedron_triggered();
    void on_actionOctahedron_triggered();
    void on_actionSquare_Plane_triggered();
    void on_actionCube_triggered();
    void on_actionTrigonal_Plane_triggered();
    void on_actionAntiprism_Tetragonal_triggered();
    void on_actionPrism_Trigonal_triggered();
    void on_actionIcosahedron_triggered();
    void on_actionIsolated_Atoms_triggered();
    void on_actionFrom_Smiles_triggered();
    void on_actionFrom_COD_triggered();
    void on_actionUndo_triggered();
    void on_actionRedo_triggered();
    void on_actionDelete_Selected_Atoms_or_Bonds_triggered();
    void on_actionDelete_Bonds_triggered();
    void on_actionDelete_Duplicate_Atoms_triggered();
    void on_actionRename_triggered();
    void on_actionRenumber_triggered();
    void on_actionSort_by_Species_triggered();
    void on_actionAdd_Bonds_triggered();
    void on_actionAtoms_or_Bonds_triggered();
    void on_actionRectangular_triggered();
    void on_actionMolecule_triggered();
    void on_actionZoom_Mode_triggered();
    void on_actionManipulation_Mode_triggered();
    void on_actionSelect_All_triggered();
    void on_actionSelect_None_triggered();
    void on_actionInvert_Selection_triggered();
    //void on_actionSelect_Type_triggered();
    void on_actionExpand_to_Molecule_triggered();
    void on_actionPreferences_triggered();
    void on_actionPolyhedra_triggered();
    void on_actionPolyhedra_Properties_triggered();
    void on_actionShow_Multiple_Bonds_triggered();
    void on_actionBuild_Bonds_triggered();
    void on_actionBond_Settings_triggered();
    void on_actionComplete_triggered();
    void on_actionRegularize_triggered();
    void on_actionEnantiomorph_triggered();
    void on_actionTransform_triggered();
    void on_actionOptimize_Crystal_Structure_by_DFT_D_triggered();
    void on_actionOverlay_Structures_triggered();
    void on_actionOverlay_Info_triggered();
    //void on_actionDelete_Model_triggered();
    void onDeleteModel(int kmodel);
    void on_actionCentroids_triggered();
    void on_actionPowder_Diffraction_Pattern_triggered();
    void on_actionPair_Distribution_Function_PDF_triggered();
    void on_actionBond_Valence_Sum_triggered();
    void on_actionStructure_triggered();
    void on_actionList_of_Atoms_triggered();
    void on_actionList_of_Bonds_triggered();
    void on_actionList_of_Angles_triggered();
    void on_actionList_of_Torsion_Angles_triggered();
    void on_actionBar_Chart_triggered();
    void on_actionList_of_Contacts_triggered();
    void on_actionGeometry_triggered();
    void on_actionSphere_triggered();
    void ModifyProperties(int which);
    void on_actionAtom_Properties_triggered();
    void on_actionBond_Properties_triggered();
    void on_actionAngle_Properties_triggered();
    void on_actionTorsion_Properties_triggered();
    void on_actionFrom_File_triggered();
    void on_actionFrom_Smiles_2_triggered();
    void on_actionFrom_COD_2_triggered();
    void on_actionTetrahedron_2_triggered();
    void on_actionOctahedron_2_triggered();
    void on_actionSquare_Plane_2_triggered();
    void on_actionCube_2_triggered();
    void on_actionTrigonal_Plane_2_triggered();
    void on_actionAntiprism_Tetragonal_2_triggered();
    void on_actionPrism_Trigonal_2_triggered();
    void on_actionIcosahedron_2_triggered();
    void on_actionIsolated_Atoms_2_triggered();
    void on_actionDuplicate_All_triggered();
    void on_actionDuplicate_Selected_Atoms_triggered();
    void on_actionPacking_triggered();
    void on_actionExpand_triggered();
    void on_actionReduce_triggered();
    void on_actionView_Cell_Edges_triggered();
    void on_actionGrow_Fragments_triggered();
    void on_actionAssemble_triggered();
    void on_actionMove_Atoms_triggered();
    void on_actionAround_X_triggered();
    void on_actionAround_X_2_triggered();
    void on_actionAround_Y_triggered();
    void on_actionAround_Y_2_triggered();
    void on_actionAround_Z_triggered();
    void on_actionAround_Z_2_triggered();
    void on_actionStep_triggered();
    void on_actionAll_Atoms_triggered();
    void on_actionDelete_All_Labels_triggered();
    void on_actionexcept_Hydrogens_triggered();
    void on_actionHeavy_atoms_triggered();
    void on_actionselected_atoms_triggered();
    void on_actionAtom_Name_triggered();
    void on_actionAtom_Specie_triggered();
    void on_actionAtom_Number_triggered();
    void on_actionShow_All_triggered();
    void on_actionHide_Selected_triggered();
    void on_actionHide_Unselected_triggered();
    void on_actionHide_Hydrogens_triggered();
    void on_actiona_triggered();
    void on_actionb_triggered();
    void on_actionc_triggered();
    void on_actiona_2_triggered();
    void on_actionb_2_triggered();
    void on_actionc_2_triggered();
    void on_actionLeast_Square_Plane_triggered();
    void on_actionSelected_Plane_triggered();
    void on_actionHydrogen_Bonds_triggered();
    void on_actionShort_Contacts_triggered();
    void on_actionDelete_Hanging_Contacts_triggered();
    void on_actionEdit_H_Bonds_triggered();
    void on_actionShow_Fourier_Map_triggered();
    void on_actionCompute_Fo_triggered();
    void on_actionCompute_2Fo_Fc_triggered();
    void on_actionCompute_Fo_Fc_triggered();
    void on_actionCompute_Fc_triggered();
    void on_actionAutomatic_triggered();
    void on_actionMethine_C_H_triggered();
    void on_actionMethylene_Group_triggered();
    void on_actionMethyl_Group_triggered();
    void on_actionAromatic_C_H_triggered();
    void on_actionHydroxyl_O_H_triggered();
    void on_actionTerminal_X_CH2_triggered();
    void on_actionAcetylenic_C_H_triggered();
    void on_actionNormalise_Hydrogens_triggered();
    void on_actionTerminal_B_H_triggered();
    void on_actionOptimize_geometry_triggered();
    void on_actionOptimize_Selected_Atoms_triggered();
    void on_actionMOPAC_triggered();
    void on_actionView_MOPAC_Output_triggered();
    void on_actionLabels_On_Off_triggered();
    void on_actionZoom_In_triggered();
    void on_actionZoom_Out_triggered();
    void on_actionReset_Scale_triggered();
    void on_actionReset_View_triggered();
    void on_actionClear_Measurements_triggered();
    void on_actionSelection_Mode_triggered();
    void on_actionDistances_triggered(bool checked);
    void on_actionAngles_triggered(bool checked);
    void on_actionTorsion_Angles_triggered(bool checked);
    void on_actionExit_triggered();
    void on_action_About_triggered();
    void on_actionrxplus_triggered();
    void on_actionrxminus_triggered();
    void on_actionryplus_triggered();
    void on_actionryminus_triggered();
    void on_actionrzplus_triggered();
    void on_actionrzminus_triggered();
    void on_actionxcplus_triggered();
    void on_actionxcminus_triggered();
    void on_actionycplus_triggered();
    void on_actionycminus_triggered();
    void on_actionzcplus_triggered();
    void on_actionzcminus_triggered();
    //void stepRot_changed();
    //void stepTra_changed();
    void on_DeleteResidues(int which);
    void on_DeleteSideChain(int which);
    void on_DeleteAromatic(int which);
    void writeSettings();
    void readSettings();
    void set_label_status();
    void GesLabels(bool stato, bool force);
    void GesLabelsSpec();
    void UpdateMenus();
    void enableMenu(QMenu *menu, bool enab);
    QAction *getAction(QString label);
    bool selezionaSpecie(int z);
    void onChangeSpecie(int z=0);
    void onContactAtom(int which);
    void onContactMol(int which);
    void onLabel_by_spec(int z=0);
    void onDelete_by_spec(int z=0);
    void onSelect_by_spec(int z=0);
    void onAddAtom(int z=0);
    void onChangeMult(int which = 0);
    void set_symm(bool stato);
    void aggiornaMenuSym();
    void changePacking(float quanto);
    void AddHydro(int icod);
    void endModify(int whichModel, bool exitSelect = false);
    bool IsModifyEnabled();
    void openRecentFile();
    void on_Rejected();
    void on_trasformDlgClosed();
    void on_SelectedCentre(QString lab);
    void on_SelectedRadius(float val);
    void on_SelectedRow(QVector<int> v);
    void on_DistChanged();
    void on_OverlaySelectionChanged(QVector<int> sel);
    void on_OverlayModelSelected();
    void on_centroidChanged();
    void on_propertiesDialogClosed();
    void on_selectedAtom(int which);
    void on_selectedBond(int which);
    void on_selectedAngle(int which);
    void on_selectedTorsion(int which);
    void on_normHydroDlgClosed(int code, int nelems);

    void on_actionStandardization_of_Crystal_Structure_triggered();

private:
    void SetSphereRadius(float r, bool ifRedraw=true);
    void SetSphereCentre(int wm, int wa, bool ifRedraw=true);
    void on_OptimizeGeometry(bool onlySelected=false);
    //void loadFile(const QString &fileName);
    void setCurrentFile(const QString &filePath);
    void createRecentActions();
    //void updateRecentFileActions();
    void FillOverlayInfo(int km=0, float dist=0);

    Ui::JavWindow *ui;
    PolyhedraDialog *mPolyhedraDialog;
    CODDialog *mCODDialog;
    OptionsDialog *mOptionsDialog;
    HBondsOptionsDialog *mHBOptionsDialog;
    InfoDialog *mInfoDialog;
    BondSettingsDialog *mBondSettingsDialog;
    PowderPattern *mPdPDialog;
    PairDist *mPDFDialog;
    GeoCalc *mGeoCalcDlg;
    SphereDialog *mSphereDlg;
    OverlayInfo *mOverlay;
    CentroidsDlg *mCentDlg;
    TransformDialog *mTransformDlg;
    PropertiesDialog *mPropDlg;
    NormHydroDlg *mNormHydroDlg;
    BondValenceDialog *mBondValenceDlg;
    MopacDialog *mMopacDlg;

    QLabel *statusLabel;
    QLineEdit *stepRot;
    QLineEdit *stepTra;
    bool m_statusUndo;
    bool m_statusRedo;
    bool PreviewMode=false;
    MyGLWidget *mOpenGLWindow;
    int m_whichObject;
    RenumberDialog *mRenumDlg;
    QString LastModel;
    int defaultBondMult;
    Model *ModelInfo;
    Mappa *MapInfo;

    const int maxFileNr;
    compare_info_type *compare_info;
    QList<QAction*> recentFileActionList;
    QString currentFilePath;
    void importFromSmiles(int ifAdd);
    enum FourierType {FOUR_FO=1, FOUR_2FO_FC, FOUR_FO_FC, FOUR_FC};

protected:
    virtual void showEvent(QShowEvent *event);
    static const int stepDecimals = 2;
};

#if !defined(JAV_WIDGET)
JavWindow *getJav();
#endif
#endif // JAVWINDOW_H
