#ifndef MODELS_H
#define MODELS_H
#include <list>
#include <string>
#include <vector>
#include <fstream>
#include <QtGui/QOpenGLFunctions>
#include <QTextStream>
#include <QList>
#include <QVector>
#include <QStandardItem>
#include "stlutils.h"
#include "glutils.h"
#include "libcomune.h"
#include "maps.h"
using namespace std;
#define MAX_SYMLEVEL 3
#define NKNOTS 9   //17

#define OTHERS     NUM_COLORS - 2
#define MONOCOL    NUM_COLORS - 1
enum structures {COIL, HELIX, STRAND, TURN, OTHER};
enum labeltypes {LABEL_ATOM, LABEL_SPECIES, LABEL_NUMBER, N_LABELTYPES};
enum hdistances {CH3, CH2, CH, C_ALCHINO, C_Ar, X_CH2, O_H, N_HP, N_HS,
                 S_H, B_H, C_DEF, N_DEF,  NHDistances};
enum speciallabs {NONH=1, HEAVY=2, SELECTED=4};

typedef struct {
                int color;          // numero seq. di specie dell'atomo
                int Z;              // numero atomico
                int ptab;           // numero d'ordine nel file xen
                op_type  op;        // symmetry operator
                int seq ;           // numero (sequenziale) del residuo
                int numResiduo;     // numero di residuo (da 0 a 19)
                int ResNum;         // numero (dal file) del residuo
                int catena;         // numero di catena
                int catseq;         // numero sequenziale di catena
                int resseq;         // numero seq. di residuo (nella catena)
                float xyzc[3];      // coordinate frazionarie
                float xyz[3];       // coordinate ortonormalizzate
                float xyzN[3];      // coordinate normalizzate per la grafica
                float vn[3];        // 
                std::string lab;    // label
                std::string prot_info; // info su catena, residuo ecc. per proteine
                int   disegnato;    // flag per grafica
                float radN[4];      // raggio atomico per grafica
                int drawLabel;      // disegna label si'/no
                float ocry;         // occupanza
                float och;          // occupanza
                float B;            // fattore termico
                float Bij[6];       // fattori termici anisotropi
                float mat[4][4];    // 
                float scala[4];     // 
                float semin;        // 
                float seminD;       //
                float seminN;       // 
                float H;            // altezza del picco
                bool  deleted;      // cancellato si/no
                bool  isW;          // acqua si/no
                bool  selected;     // selezionato si/no
                bool  hidden;       // nascosto si/no
                int   heta;         // heteroatom? 1:0
                int   numleg;       // numero di atomi legati
                int   leg[100];     // atomi legati
                float dist[100];    // distanze atomi legati
               } jav_atom_type;

typedef struct {
               int   n1, n2;   // atomi legati
               float dist;     // distanza corrente
               float sigma;    // sigma
               bool  selected; // selezionato si/no
               int   ordleg;   // ordine del legame
               } jav_bond_type;

typedef struct {
               int   n1, n2, n3;  // atomi legati
               float fv;          // angolo tra i legami
               float sigma;       // sigma
               } jav_anglebond_type;

typedef struct {
               int   n1, n2, n3, n4; // atomi legati
               float fv;             // angolo tra i legami
               } jav_torsion_type;

typedef struct {
               int atom;
               int o;
               int o2;
               int c;
               int n;
               int h;
               float Hxyz[3];
               int chain;
               int residue;
               string ResLab; 
               bool breakF;
               bool selected;
               bool isAmino;
               float xyz[3];
               } C_Alpha;

typedef struct {
               int  num;
               string PDBnum;
               string name;
               char secondary;
               vector<int> atoms;
               int  ca, o, c, n, h;
               int  code;
               bool breakF;
               } RESIDUE;

typedef struct {
                string id;
                int nResidues;
                int nAmino;
                bool isDNA;
                bool isRNA;
                vector<RESIDUE> Residues;
                vector<RESIDUE> AminoRes;
               } CHAIN;

#define NMAX_FACE_POINTS 4
#define NMAX_FACES 8
typedef struct {
                 int fp[NMAX_FACE_POINTS];
                 int n;
               } face_type;

typedef struct {
                 int nvert;                  // numero di vertici del poliedro
                 int xc;                     // centro del poliedro
                 int vert[6];                // vertices of polyhedron
                 face_type face[NMAX_FACES]; // facce del poliedro
                 float d_index;              // distorsion index
                 float vol;                  // volume of polyhedron
               } polyhedron_type;

typedef struct {
                 bool  visible;
                 int   transparency;
                 bool  ifBorder;
                 QColor colore;
                 QColor Bcolore;
                 float line_width;
                 int   vert;
               } polyhedron_style;

typedef struct {
                 float   value;
                 QColor  color;
                 QString tip;
               } bar_info;

typedef struct {
                 float   val;
                 QString label;
                 QColor  color;
               } bar_legend;

typedef struct {
  float        dist;            // bond length
  int          at1,at2;         // atom pointer to asu
  float        xat1[3],xat2[3]; // coordinates
  op_type      op1, op2;        // symmetry operators
  int          dupl;            // pointer to similar bond for duplicate
  float        bv;              // bond valence
               } bond_info_t;

typedef struct {
  int    z1, z2 ;     // atomic numbers
  int    val1,val2;   // valence
  float  ro;          // bond valence parameter Ro
  float  b;           // bond valence parameter B
  int    info;
  float  rmin,rmax;   // bond window
               } bond_valence_t;

#if defined(JAV_WIDGET)
   class CrystalViewWidget;
#else
   class JavWindow;
#endif

   class Model {
   private:
#if defined(JAV_WIDGET)
   CrystalViewWidget *jav;
#else
       JavWindow *jav;
#endif
   public:
      Model();
      Model* Clone() const;  
      void SetModel(int status[], float fstatus[], statusSav vec);
      void InitColors();
      void FillSecondaryAndHet();
      //void RipassaModello(int k);
      void setValues(int status[], float fstatus[]);
      void freeModel();
      void freeVectors(); //bool tofree);
      void freeStructs(); //bool tofree);
      void PreparaTabelle();
      int  PreparaTabelle(string &msg);
      void DeleteResidue(int which);
      void DeleteSideChain(int which);
      void DeleteRing(int which);
      const char *NomeResiduo(int which);
      int  FindChain(string &curChain);
      int  FindChainId(string &curChain);
      int  FindResidue(int ch, string &curResidue, RESIDUE *rr, int atom);
      int  FindConn(int at1, int at2, int nlegs=-1);
      void FillPhList();
      //void FillListaAtomi(GtkWidget *dlg, int nodi[]);
      //void FillLists(GtkWidget *dlg);
      void UpdateSelectedAtom(size_t whichAtom, bool to_add=false);
      void UpdateSelectedBond(size_t whichBond, int whichType);
      void UpdateSelectedResidue(size_t whichAlpha, bool to_add=false);
      //vector<char *> ListaAtomi(vector<int> atoms, GtkWidget *dlg);
      void UpdateInfoSelections();
      //void FillStructureInfo(GtkWidget *dlg);
      //void FillStructureInfo(GLWindow *gl, std::stringstream &os, bool im=false);
      QList<QStandardItem *> FillListChains();
      QList<QStandardItem *> *FillListResidues();
      QList<QStandardItem *> *FillListAtomsPDB(bool ifB, int ch, int res, int *nelems);
      QList<QStandardItem *> *FillListAtoms(bool is_edit=false, bool ifB=true, int *nelems=nullptr);
      QList<QStandardItem *> *FillListBonds(bool is_edit=false);
      QList<QStandardItem *> *FillListAngles(bool is_edit=false);
      QList<QStandardItem *> *FillListTorsions(bool is_edit=false);
      QList<QStandardItem *> *FillListContacts(int *ncontacts);
      QVector<bar_info> FillBarChart(QVector<bar_legend> &barLegend);
      QList<QStandardItem *> *FillListSphere(int nrows, int atomi[],
                                             float vdist[], op_type vsym[]);
      QList<QStandardItem *> *FillBondValence(float *bvs, int *cn);
      QList<QStandardItem *> *FillBondValenceInfo(int ninfo, bond_info_t *dinfo);
      QList<QStandardItem *> *FillBondValenceParameters(int n, bond_valence_t *bvec) const;
      //void FillBarChart(GtkWidget *dlg);
      //void FillListSphere(GtkWidget *dlg, int nrows, int atomi[],
                          //float vdist[], op_type vsym[]);
      //void FillComboSpec(GtkWidget *dlg);
      //void FillBondValence(GtkWidget *dlg, float *bvs, int *cn);
      //void FillBondLengths(GtkWidget *but, int wm, int kat);
      void GetConns();
      void GetEquivs();
      void CalcAngles();
      void CalcTors();
      float Distance(int atom1, int atom2);
      float Ang(int atom1, int atom2, int atom3);
      float Tors(int atom1, int atom2, int atom3, int atom4);
      int  SetColor(int whichAtom, float color1[]);
      QColor SetRungColor(int whichAtom);
      void DrawModel(QOpenGLFunctions *f);
      void DrawSBonds(QOpenGLFunctions *f, int nlegs, jav_bond_type *legs, int tipo);
      void DrawAtoms(int j);
      void DrawBonds(QOpenGLFunctions *f, int j0);
      void DrawBonds2(QOpenGLFunctions *f, int j0, bool ifLigands=false);
      void DrawIsolated(QOpenGLFunctions *f);
      void DrawSticks(QOpenGLFunctions *f);
      void DrawSticks(QOpenGLFunctions *f, int i0, int i1, jav_bond_type *legs,
                      bool iflig=false, float *colore=nullptr);
      void Draw2Cylinder (QOpenGLFunctions *f, int i0, int i1, jav_bond_type *legs);
      void DrawScaletta (QOpenGLFunctions *f);
      void DrawRibbons (QOpenGLFunctions *f, bool selected=false, int kmodel=-1);
      void DrawRibbon(QOpenGLFunctions *f, int ca,  Knot knot[], int npoints, bool selected=false);
      void DrawTubes(QOpenGLFunctions *f, int npoints, Knot knot[], float col[], double factor=1);
      void DrawFlatRibbon(QOpenGLFunctions *f, Knot src, Knot dst, float color[]);
      void DrawSolidRibbon(Knot src, Knot dst, float color[]);
      void DrawStrands(int npoints, Knot knot[], float color[]);
      void DrawCircles(int ind=0);
      void DrawBalls(QOpenGLFunctions *f, int ind, bool ifEll=false);
      void DrawSelected(QOpenGLFunctions *f);
      void DrawSelectedBalls();
      void DrawSelectedCylinder(QOpenGLFunctions *f, int i0, int i1, jav_bond_type *legs);
      void DrawCyls (QOpenGLFunctions *f, bool selected=false, bool ifLigands=false);
      void DrawPeaks (QOpenGLFunctions *context);
      void DrawLabels(QOpenGLFunctions *context, bool ifSingle=false);
      void DrawLigandsLabels(float model[], float proj[], GLint view[]);
      void DrawAtomLabel(jav_atom_type at, int i, GLdouble x, GLdouble y,
               GLdouble z, const char *lab=nullptr);
      void MoveCenter(float posi[], int katom, bool ifDrawCenter);
      //PangoRectangle *GetLabelRect(jav_atom_type at, int i,
                                    //PangoFontDescription *fdesc);

      //void Normalizza(GLWindow *gl);
      bool IsHydrogen(int which);
      bool IsHeavy(int which);
      bool IsPeak(int which);
      bool IsDna(int atom);
      bool IsHelix(int atom);
      bool IsBetaSheet(int atom);
      bool IsTurn(int atom);
      bool IsLigand(int atom);
      bool IsHetero(int atom);
      bool IsType(int atom, string type);
      int  IsTypeBB(int atom, const char *types[], int n);
      bool IsAromatic(int atom);
      bool IsSideChain(int atom);
      bool IsCAlpha(int atom) {return IsType(atom, "CA");}
      bool IsOxygen(int atom) {return IsType(atom, "O");}
      bool IsCarbon(int atom) {return IsType(atom, "C");}
      bool IsNitrogen(int atom) {return IsType(atom, "N");}
      bool IsProtein() {return (nResAmino > 0);}
      int  FindAtom(string type, RESIDUE rr, int tipo=0);
      int  GetNumResiduo(int ch, int r);
      string GetResName(int res1);
      QString GetSpecies(int whichNo);
      const char *GetSpeciesName(int whichNo);
      bool CalcolaBaricentro(int i0, int i1, float bc[3]);
      void AllCAlpha();
      float VolumePerAtom(float Volume, int nsymt);
      int FindSpec(QString &spec);
      atom_type *FromJavatomToAtom();
      atom_type *FromListToAtom(QList<QStandardItem *>  *lista, int *natoms);
      f_bond_type *FromJavbondToLegm();
      f_bond_type *FromListToLegm(QList<QStandardItem *>  *lista, int *nbonds);
      jav_anglebond_type *FromListToAngs(QList<QStandardItem *>  *lista, int *nangs);
      jav_torsion_type *FromListToTors(QList<QStandardItem *>  *lista, int *ntors);
      QString GetType(int na, ...);
      //void FromLegmToList(GtkWidget *list, f_bond_type *legm);
      //void FromAngsToList(GtkWidget *list, jav_anglebond_type *angs);
      void FillAngs(int nangles, jav_anglebond_type *angs);
      //void FromTorsToList(GtkWidget *list, jav_torsion_type *angs);
      void FillTors(int nangles, jav_torsion_type *angs);
      void CopyAtoms(jav_atom_type *atml, int natoms);
      void CopyAtomsAU(jav_atom_type *atml, int natoms);
      void CopySpecies(QString species[], int nspecies);
      void DrawPolyhedra(QOpenGLFunctions *f);
      void CalcolaNormali();
      void FillPolyhedra(int np, polyhedron_type *p);
      void FillPolyhedraStyles();
      int  FindStyle(int whichPol);
      void NorEll(int na, float el[3][3], float Ell[3][3], float CD[3][3],
                  float semax[3] , int *ifIso);
      void SetEllVal(int na, float Ell[3][3], float CD[3][3], float semax[3]);
      void AddProb();
      void ToggleLabels(bool flag);
      void SetLabelsSelected(bool stato);
      void CreaLegsBB();
      void CreaLegsCA();
      void CreaLegsLig();
      //void PopulateModel (GtkTreeView *list, int ifClear);
      void CreaMappa(int quale, float pRadius=1.2f, float step=0.3f);
      void FillAtoms(atom_type *atml);
      void FillAtoms(atom_type *atml,  structure_info_type StructInfo, bool au);
      void FillLegs(int *nlegs, int *nlegs_au, f_bond_type *legm, bool ed=false);
      void exportScene(std::vector<vet3> &xyz);
      int IsAtom(QOpenGLFunctions *f, int x, int y, int ind=0);
      void FillPreview(QTextStream &strm);
      void FillPreview(QTextStream &strm, float I_coor[][3], int conn[][3]);
      void Normalizza();
      void PreparaEllissoidi();
      void DrawRibbonsSTL(vector<C_Alpha> &c_alphab);
      void DrawRibbonSTL(int kc,  Knot knot[], int npoints);
      void PrintPoliedriSTL();
      RESIDUE *GetResiduo(int whichChain, int whichResidue)
                   {return &Chains[whichChain].Residues[whichResidue] ;}
      void SelezionaAngolo(int which);
      void SelezionaTorsion(int which);
      void SettaSurface(bool ifView);
      void DrawSurface(QOpenGLFunctions *f);
      void RicalcolaSurface();
      void ClearAtomsSelection();
      void SelezionaAtomo(int which);
      void ClearBondsSelection();
      void SelezionaBond(int which, int nn1=-1, int nn2=-1);
      void FillSite(QVector<QVector<int>>& site);
      float distanzaCart(int at1, int at2) {return Distanza(atm[at1].xyz, atm[at2].xyz) ;}
      float distanzaCris(int at1, int at2) {return Distanza(atm[at1].xyzc, atm[at2].xyzc) ;}

      int   nknots;
      float H[4][NKNOTS-2];
      int nModello;                 // numero del modello
      bool isPDB;                   // se proviene da un file PDB
      float Bmin, Bmax;             // min/max value of thermal factor
      float Bmin_ca, Bmax_ca;       // min/max value of thermal factor(alpha carbons)
      float maxRad[4];              // raggio massimo degli atomi presenti
      float radMax;                                                        
      float baricentro[3];          // coordinate del baricentro
      int  nats;                    // numero di atomi del modello
      int  nats_au;                 // numero di atomi dell'unita' asimmetrica
      int  nlegs_au;                // legami           "    "      "      "
      int  nangbonds;
      int  ntorsions;
      int  nconn;                   // = nconn_prot
      int  nconn_prot;              // numero di legami della proteina
      int  nconn_bb;                // numero di legami del backbone
      int  nconn_ca;                // numero di legami tra i carboni alfa
      int  nconn_lig;               // numero di legami dei ligandi
      int  nconn_mix;               // numero di legami tra ligandi e proteina
      int  nconn_nuc;               // numero di legami dei nucleotidi
      float radii[MAX_SPECIES][2];  // raggio covalente per ogni specie
      int  numSpec;                 // numero di specie presenti
      int  numT;                    // dimensioni della Bond Table;
      int  Species[MAX_SPECIES];    // n. di specie atomica ( = no del fortran)
      int  Content[MAX_SPECIES];    // n. di atomi per specie ( = nw del fortran)
      QString LabSpec[MAX_SPECIES]; // label specie
      int  ptab[MAX_SPECIES];       // ptab per specie
      bool created;                 // flag per grafica
      bool ifVis;                   //  "   "   "
      bool ifLabels;                //  "   "   "
      int  specialLab;              //  "   "   " (1 =tutti tranne idrogeni)
                                    //            (2 =solo atomi pesanti)
                                    //            (3 =solo atomi selezionati)
                                    //            (-1=solo atomi di una det.
                                    // specie, indicata da labelS)
      int  LabelS[N_ELEMENTS];      //  "   "   "
      bool ifCentro;                //  struttura centrosimmetrica si'/no
      int  plotStyle;               // plot style corrente
      int  drawLigand;              // flag per grafica
      bool ifSymmetry;              // symmetry si'/no
      float klevel[3];              // packing range x simmetria (da 0 a 10)
      int  FitOpt;                  // fit option x simmetria (da 1 a 4)
      bool ifChangedSymmetry;       //  "   "   modificata
      bool ifDrawH;                 // draw hydrogens si'/no
      bool ifDrawW;                 // draw watre molecules si'/no
      bool ifResize;                // resize su tutti gli atomi o solo a.u.
      int  colorMode;               // tipo di colore
      int  labelType;               // tipo di label (0 = nome, 1 = specie)
      bool ifSurface;               // richiesta superficie di VanDerWaals
      bool Surface_c;               // creata superficie di VanDerWaals
      bool ifPoly;                  // richiesti poliedri si'/no
      int   nChains;                // numero di catene
      int   nResTot;                // numero di residui
      int   nResAmino;              // numero di residui veri (amminoacidi)
      bool  hasSecondary;           // struttura secondaria determinata si'/no
      float RF;                     // R-factor
      int nlegsH;
      int nlegsW;
      int   nucleic;                // 0 inorganic or protein; 1 dna; 2 rna
      QColor light_chain;         // luci e colori per grafica
      QColor labelcolor;          // luci e colori per grafica
      QColor light_diffuse[NUM_COLORS];  // luci e colori per grafica
      int   numbStructs[4];         // numero di coils, eliche, di strands, e di turns
      float distHn[NHDistances];    // distanze X-H normalizzate
      int nPoly;                    // numero di Poliedri
      int nPolyStyles;              // numero di Poliedri distinti
      string modname;               // nome del modello
      QString fileName;             // nome del file
      jav_atom_type *atm;           // info sugli atomi 
      jav_atom_type *atm_au;        // info sugli atomi (a.u.)
      int  *iEqui;                  // equivalenti per simmetria
      jav_bond_type *bonds;         // legami 
      jav_bond_type *bonds_bb;      // legami backbone
      jav_bond_type *bonds_ca;      // legami c_alpha and/or phospho...
      jav_bond_type *bonds_lig;     // legami ligandi
      jav_bond_type *bonds_mix;     // legami ligandi/proteina
      jav_bond_type *bonds_nuc;     // legami nucleotidi
      jav_bond_type *legsH;         // legami idrogeno
      jav_bond_type *legsW;         // legami Wandervaals
      char  *Secondary;             // info sulla struttura secondaria (+)
      jav_anglebond_type *anglebonds;// legami + angoli
      jav_torsion_type *torsions;   // angoli di torsione
      polyhedron_type *Polyhedra;   // poliedri
      polyhedron_style *PolyStyle;  //stile poliedri
      int *NConns;                  // per ogni atomo numero degli atomi connessi
      int *NEquivs;                 // per ogni atomo numero degli atomi equivalenti
      vector<CHAIN> Chains;         // catene;
      vector<C_Alpha> c_alpha;      //carboni alfa 
      vector<C_Alpha> c_alphab;     //carboni alfa 
      vector<int> atoms_sel;        //atomi selezionati
      vector<int> bonds_sel;        //legami selezionati
      vector<int> resid_sel;        //residui selezionati
      vector<int> phlist   ;        //
      vector<string> atom_types;    //
      vector<string> AtomTypes;     //
      vector<vector<int> > Conns   ;// per ogni atomo lista degli atomi connessi
      vector<vector<int> > Equivs  ;// per ogni atomo lista degli atomi equivalenti
      Mappa VWSurface;
// (+)   struttura secondaria
//       'E' 'e'   --> beta sheet
//       'H' 'h'   --> elica
//       'G' 'g'   --> elica
//       'P' 'p'   --> elica
   };

//void add_model(GtkWidget *window, int i, GtkWidget *father, GtkWidget *parent);
//void model_settings(GtkWidget *window, GtkWidget *dlg, GtkWidget *vbox);
//void CApplicaSimmetria(int *which);
bool DetectSecondary(Model *md, std::string &msg);
void FillSecondary(Model *md);
void PrintSecondary(Model *md, QString &os);
#endif
