//#include <QCoreApplication>
#include <QDir>
#include "networkdownload.h"

// constructor
NetworkDownload::NetworkDownload(QObject *parent)
    : QObject(parent), downloadFolder(QDir::currentPath())
{
    // signal finish(), calls downloadFinished()
    connect(&manager, SIGNAL(finished(QNetworkReply*)),
            this, SLOT(downloadFinished(QNetworkReply*)));
}

void NetworkDownload::execute(const QStringList &args)
{
//    // get the argument including program's name
//    QStringList args = QCoreApplication::instance()->arguments();

//    // skip the first argument, which is the program's name
//    args.takeFirst();

//    if (args.isEmpty()) {
//        printf("Qt Download example - downloads all URLs in parallel\n"
//               "Usage: download url1 [url2... urlN]\n"
//               "\n"
//               "Downloads the URLs passed in the command-line to the local directory\n"
//               "If the target file already exists, a .0, .1, .2, etc. is appended to\n"
//               "differentiate.\n");
//        QCoreApplication::instance()->quit();
//        return;
//    }

    // process each url starting from the 2nd one
    foreach (QString arg, args) {

        // QString::toLocal8Bit()
        //  - local 8-bit representation of the string as a QByteArray
        // Qurl::fromEncoded(QByteArray)
        //  - Parses input and returns the corresponding QUrl.
        //    input is assumed to be in encoded form,
        //    containing only ASCII characters.

        QUrl url = QUrl::fromEncoded(arg.toLocal8Bit());

        // makes a request
        doDownload(url);
    }
}

// Constructs a QList of QNetworkReply
void NetworkDownload::doDownload(const QUrl &url)
{
    QNetworkRequest request(url);
    QNetworkReply *reply = manager.get(request);

#ifndef QT_NO_SSL
    connect(reply, SIGNAL(sslErrors(QList<QSslError>)), SLOT(sslErrors(QList<QSslError>)));
#endif

    // List of reply
    currentDownloads.append(reply);
}

QString NetworkDownload::saveFileName(const QUrl &url)
{
    QString path = url.path();
    QString basename = QFileInfo(path).fileName();

    if (basename.isEmpty())
        basename = "download";

    QString filename = downloadFolder + QDir::separator() + basename;

    if (QFile::exists(filename)) {
        // already exists, don't overwrite
        int i = 0;
        filename += '.';
        while (QFile::exists(filename + QString::number(i)))
            ++i;

        filename += QString::number(i);
    }

    return filename;
}

void NetworkDownload::downloadFinished(QNetworkReply *reply)
{
    QUrl url = reply->url();
    //QString filename = downloadFolder + QDir::separator() + saveFileName(url);
    QString filename = saveFileName(url);
    bool success = false;
    if (reply->error()) {
//        fprintf(stderr, "Download of %s failed: %s\n",
//                url.toEncoded().constData(),
//                qPrintable(reply->errorString()));

    } else {
        if (saveToDisk(filename, reply)) {
//            printf("Download of %s succeeded (saved to %s)\n",
//                   url.toEncoded().constData(), qPrintable(filename));
            success = true;
        }
    }

    currentDownloads.removeAll(reply);
    reply->deleteLater();

    emit downloadCompleted(success, filename);

//    if (currentDownloads.isEmpty())
//        // all downloads finished
//        QCoreApplication::instance()->quit();
}

bool NetworkDownload::saveToDisk(const QString &filename, QIODevice *reply)
{
    QFile file(filename);
    if (!file.open(QIODevice::WriteOnly)) {
//        fprintf(stderr, "Could not open %s for writing: %s\n",
//                qPrintable(filename),
//                qPrintable(file.errorString()));
        return false;
    }

    file.write(reply->readAll());
    file.close();

    return true;
}

void NetworkDownload::sslErrors(const QList<QSslError> &sslErrors)
{
#ifndef QT_NO_SSL
    foreach (const QSslError &error, sslErrors)
        fprintf(stderr, "SSL error: %s\n", qPrintable(error.errorString()));
#else
    Q_UNUSED(sslErrors);
#endif
}

void NetworkDownload::setDownloadFolder(const QString &value)
{
    downloadFolder = value;
}
