#include "ui_optionsdialog.h"
#include "javkeysettings.h"
#include "optionsdialog.h"
//#include "mainwindow.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif
#include "qt_utils.h"
#include "colorbutton.h"
#include "myslider.h"

#include <QDebug>
#include <QColorDialog>
#include <QStringListModel>
#include <QSettings>
#include <QList>
#include <QFontDialog>
#include <QMessageBox>

static const QStringList styleList = {"Wires","Simple", "Bonds", "Rods", "Stick and Balls", "Thermal Ellipsoids","Van der Waals"};
static const QStringList styleListProt = {"Backbone", "Chain Trace", "Ribbons", "Strands", "Cartoons"};
static const QStringList colorList  = {"by atom species", "by temperature (B fact.)",
              "by s.o.f.", "by symmetry operator", "monochromatic"};
static const QStringList colorListProt  = {"by residue", "by chain", "by secondary structure", "by position along chain"};

static const QStringList glasses = {"RB", "RG", "RC", "BR", "GR", "CR"};


OptionsDialog::OptionsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::OptionsDialog)
{
#if defined(JAV_WIDGET)
    jav = qobject_cast<CrystalViewWidget*>(parent);
#else
    jav = qobject_cast<JavWindow*>(parent);
#endif
    ui->setupUi(this);
    connect(ui->backgroundCombo, &ColorWidget::colorWidgetChanged, this, &OptionsDialog::backgroundChanged);
}

OptionsDialog::~OptionsDialog()
{
    delete ui;
}


void OptionsDialog::SetPage(int n)
{
    ui->listWidget->setCurrentRow(n);
}

//void OptionsDialog::on_listWidget_clicked(const QModelIndex &index)
//{
//    ui->stackedWidget->setCurrentIndex(index.row());
//}

void OptionsDialog::reDraw()
{
     if(ui->stackedWidget->currentIndex() == PAGE_LIGHT)
        jav->ResetLights();
     jav->Draw();
}

void OptionsDialog::on_listWidget_currentRowChanged(int currentRow)
{
    ui->stackedWidget->setCurrentIndex(currentRow);
}

Model *OptionsDialog::GetModel(int k)
{
    return jav->GetModel(k+1);
}


void OptionsDialog::saveInfo(int kmod)
{
// Models and Polyhedra
    int k1=0, k2=MAX_NUM_MODELS;
    if(kmod >= 0)
    {
       k1=kmod;
       k2=kmod+1;
    }
    for(int k=k1; k<k2; k++)
    {
        ModelSav[k].ifVis = GetModel(k)->ifVis;
        ModelSav[k].ifLabels = GetModel(k)->ifLabels;
        ModelSav[k].plotStyle = GetModel(k)->plotStyle;
        ModelSav[k].drawLigand = GetModel(k)->drawLigand;
        ModelSav[k].ifSymmetry = GetModel(k)->ifSymmetry;
        ModelSav[k].FitOpt = GetModel(k)->FitOpt;
        ModelSav[k].ifDrawH = GetModel(k)->ifDrawH;
        ModelSav[k].ifDrawW = GetModel(k)->ifDrawW;
        ModelSav[k].colorMode = GetModel(k)->colorMode;
        ModelSav[k].labelcolor = GetModel(k)->labelcolor;
        ModelSav[k].nPolyStyles = GetModel(k)->nPolyStyles;
        ModelSav[k].ifSurface = GetModel(k)->ifSurface;
        ModelSav[k].wireframe = GetModel(k)->VWSurface.wireframe;
        ModelSav[k].color_mappa = GetModel(k)->VWSurface.color_mappa;
        ModelSav[k].radius = GetModel(k)->VWSurface.radius;
        ModelSav[k].step = GetModel(k)->VWSurface.step;
        ModelSav[k].PolyStyle = new polyhedron_style [ModelSav[k].nPolyStyles]();
	for(int j=0; j<ModelSav[k].nPolyStyles; j++)
            ModelSav[k].PolyStyle[j] =  GetModel(k)->PolyStyle[j];
    memcpy(ModelSav[k].klevel, GetModel(k)->klevel, 3*sizeof(float));
    }
// Map(s)
//
    for (int i=0; i<3; i++)
        MapSav.centro[i] = Properties::xyzC.v[i];
    MapSav.raggio = Properties::Radius;
// Atoms
//
    for (int i = 0; i < N_ELEMENTS; i++)
    {
	 AtomsSav.Colori[i] = Properties::GetColor(i);
	 AtomsSav.scale_atoms[i] = Properties::scale_atoms[i];
    }
    AtomsSav.sc_atoms = Properties::sc_atoms;
    AtomsSav.RadType = Properties::RadType;
    AtomsSav.ell_tr = Properties::ell_tr;
    AtomsSav.ell_prob = Properties::ell_prob;
    AtomsSav.ell_princ = Properties::ell_princ;
    AtomsSav.ell_color = Properties::ell_color;
    AtomsSav.ell_hydro = Properties::ell_hydro;
//Bonds
//
    BondsSav.sc_bonds = Properties::sc_bonds;    
    BondsSav.ColorHB = Properties::ColorHB;    
    BondsSav.ColorSC = Properties::ColorSC;    
    BondsSav.ShowMultBonds = Properties::ShowMultBonds;    

//Draw Options
//
    DrawSav.BgType = Properties::BgType;
    DrawSav.BgColor = Properties::BgColor;
    DrawSav.BgColor1 = Properties::BgColor1;
    DrawSav.BgColor2 = Properties::BgColor2;
    DrawSav.ifCell = Properties::ifCell;
    DrawSav.CellColor = Properties::CellColor;
    DrawSav.ifAxes = Properties::ifAxes;
    DrawSav.ifDrawAxes = Properties::ifDrawAxes;
    DrawSav.Clip = Properties::Clip;
    DrawSav.ClipFactor = Properties::ClipFactor;
    DrawSav.Fog = Properties::Fog;
    DrawSav.FogDensity = Properties::FogDensity;
    DrawSav.DistColor = Properties::DistColor;
    DrawSav.xoffset = Properties::xoffset;
    DrawSav.yoffset = Properties::yoffset;
    DrawSav.fontname = Properties::fontname;
    
//Disp Options
    DispSav.MSEnabled = {Properties::IsMSEnabled() > 0};
    DispSav.MSBuffers = Properties::MSBuffers();
    DispSav.ifGrid = Properties::ifGrid;
    DispSav.nDivisions = Properties::nDivisions;
    DispSav.GridPattern = Properties::GridPattern;
    DispSav.Stereo = Properties::Stereo;
    DispSav.glasses_type = Properties::glasses_type;
    DispSav.ifOrtho = Properties::ifOrtho;

//Light Options
    for(int j=0; j<3; j++)
        LightSav[j] = Properties::lightInfo[j];
}

void OptionsDialog::fillAtomsDefault()
{
    for (int i = 0; i < N_ELEMENTS; i++)
    {
         AtomsDefault.Colori[i] = get_elem_color(i);
         AtomsDefault.scale_atoms[i] = 1;
    }
    AtomsDefault.sc_atoms = 1;
    AtomsDefault.RadType = 0;
    AtomsDefault.ell_tr = 0;
    AtomsDefault.ell_prob = 50;
    AtomsDefault.ell_princ = true;
    AtomsDefault.ell_color = ToQColor(0.1f, 0.1f, 0.1f);
    AtomsDefault.ell_hydro = false;
}

void OptionsDialog::restoreModelInfo(_model_info ModelCopy, int k, bool ifpol)
{
    bool changedSymm=false;
    {
        GetModel(k)->ifVis = ModelCopy.ifVis;
        if((k == m_kmodel) && (GetModel(k)->ifLabels != ModelCopy.ifLabels))
           jav->SetLabels();
        GetModel(k)->ifLabels = ModelCopy.ifLabels;
        GetModel(k)->plotStyle = ModelCopy.plotStyle ;
        GetModel(k)->drawLigand = ModelCopy.drawLigand ;
        if(GetModel(k)->ifSymmetry != ModelCopy.ifSymmetry)
           changedSymm = true;
        GetModel(k)->ifSymmetry = ModelCopy.ifSymmetry ;
        if((k == m_kmodel) && (GetModel(k)->FitOpt != ModelCopy.FitOpt))
            changedSymm = true;
        GetModel(k)->FitOpt = ModelCopy.FitOpt ;
        GetModel(k)->ifDrawH = ModelCopy.ifDrawH ;
        GetModel(k)->ifDrawW = ModelCopy.ifDrawW ;
        GetModel(k)->colorMode = ModelCopy.colorMode ;
        GetModel(k)->labelcolor = ModelCopy.labelcolor ;
        GetModel(k)->ifSurface = ModelCopy.ifSurface;
        GetModel(k)->VWSurface.wireframe = ModelCopy.wireframe;
        GetModel(k)->VWSurface.color_mappa = ModelCopy.color_mappa;
        GetModel(k)->VWSurface.radius = ModelCopy.radius;
        GetModel(k)->VWSurface.step = ModelCopy.step;
        if(ifpol)   // solo sul Cancel e non sui default
        {
           GetModel(k)->nPolyStyles = ModelCopy.nPolyStyles ;
           GetModel(k)->PolyStyle = new polyhedron_style [ModelCopy.nPolyStyles];
           for(int j=0; j<ModelCopy.nPolyStyles; j++)
                GetModel(k)->PolyStyle[j] = ModelCopy.PolyStyle[j];
        }
        if((k == m_kmodel) &&
           ((GetModel(k)->klevel[0] != ModelCopy.klevel[0]) ||
           (GetModel(k)->klevel[1] != ModelCopy.klevel[1]) ||
           (GetModel(k)->klevel[2] != ModelCopy.klevel[2])))
             changedSymm = true;
        memcpy(ModelCopy.klevel, GetModel(k)->klevel, 3*sizeof(float));
        if(changedSymm)
           jav->SetPacking();
    }
}

void OptionsDialog::restoreInfo(bool ifDefault)
{
    if(ifDefault)
    {
	switch(ui->stackedWidget->currentIndex())
	{
	    case PAGE_MODELS:
	         restoreModelInfo(ifDefault);
             fillOptionsModel(m_kmodel + 1);
		 break;
	    case PAGE_ATOMS:
	         restoreAtomInfo(ifDefault);
             fillPageAtoms();
		 break;
	    case PAGE_BONDS:
	         restoreBondInfo(ifDefault);
             fillPageBonds();
		 break;
	    case PAGE_MAPS:
	         restoreMapInfo(ifDefault);
             fillPageMaps();
		 break;
	    case PAGE_POLY:
	         restorePolyInfo();
             fillPagePoly();
		 break;
	    case PAGE_DRAW:
	         restoreDrawInfo(ifDefault);
             fillPageDraw();
		 break;
	    case PAGE_DISP:
	         restoreDispInfo(ifDefault);
             fillPageDisp();
		 break;
	    case PAGE_LIGHT:
	         restoreLightInfo(ifDefault);
             fillPageLight();
		 break;
            default:
		 break;
	}
	reDraw();
    }
    else
    {
	restoreModelInfo(ifDefault);
	restoreAtomInfo(ifDefault);
	restoreBondInfo(ifDefault);
	restoreMapInfo(ifDefault);
	restoreDrawInfo(ifDefault);
	restoreDispInfo(ifDefault);
	restoreLightInfo(ifDefault);
        jav->ResetLights();
    }
}

void OptionsDialog::restoreModelInfo(bool ifDefault)
{
    for(int k=0; k<MAX_NUM_MODELS; k++)
    {
	if(GetModel(k)->IsProtein())
	{
	   ModelDefault.plotStyle = CARTOONS;
	   ModelDefault.colorMode = BY_STRUCT;
	}
	else
	{
	   ModelDefault.plotStyle = STICK_AND_BALLS;
	   ModelDefault.colorMode = BY_SPECIES;
	}
	if(ifDefault)
	   restoreModelInfo(ModelDefault, k, false);
	else
	   restoreModelInfo(ModelSav[k], k);
    }
}

void OptionsDialog::restoreAtomInfo(bool ifDefault)
{
    _atoms_info *atoms_info;
    if(ifDefault)
       atoms_info = &AtomsDefault;
    else
       atoms_info = &AtomsSav;
    for (int i = 0; i < N_ELEMENTS; i++)
    {
         Properties::SetColor(i, atoms_info->Colori[i]);
         Properties::scale_atoms[i] = atoms_info->scale_atoms[i];
    }
    Properties::sc_atoms = atoms_info->sc_atoms;
    Properties::RadType = atoms_info->RadType;
    Properties::ell_tr = atoms_info->ell_tr;
    Properties::ell_prob = atoms_info->ell_prob;
    Properties::ell_princ = atoms_info->ell_princ;
    Properties::ell_color = atoms_info->ell_color;
    Properties::ell_hydro = atoms_info->ell_hydro;
    for (int i = 0; i < Properties::nModels; i++)
     jav->GetModel(i+1)->InitColors();
}

void OptionsDialog::restoreBondInfo(bool ifDefault)
{
    _bonds_info *bonds_info;
    if(ifDefault)
       bonds_info = &BondsDefault;
    else
       bonds_info = &BondsSav;
    Properties::ColorHB = bonds_info->ColorHB;
    Properties::ColorSC = bonds_info->ColorSC;
    Properties::sc_bonds = bonds_info->sc_bonds;
    Properties::ShowMultBonds = bonds_info->ShowMultBonds;
}

void OptionsDialog::restoreMapInfo(bool ifDefault)
{
    _map_info *map_info;
    if(ifDefault)
       map_info = &MapDefault;
    else
       map_info = &MapSav;
    Properties::Radius = map_info->raggio;
    for(int i=0; i<3; i++)
        Properties::xyzC.v[i] = map_info->centro[i];
}

void OptionsDialog::restoreDrawInfo(bool ifDefault)
{
    _draw_info *draw_info;
    if(ifDefault)
       draw_info = &DrawDefault;
    else
       draw_info = &DrawSav;

    bool changedCell = (Properties::ifCell != draw_info->ifCell);
    Properties::BgType = draw_info->BgType;
    Properties::BgColor = draw_info->BgColor;
    Properties::BgColor1 = draw_info->BgColor1;
    Properties::BgColor2 = draw_info->BgColor2;
    Properties::ifCell = draw_info->ifCell;
    Properties::CellColor = draw_info->CellColor;
    Properties::ifAxes = draw_info->ifAxes;
    Properties::ifDrawAxes = draw_info->ifDrawAxes;
    Properties::Clip = draw_info->Clip;
    Properties::ClipFactor = draw_info->ClipFactor;
    Properties::Fog = draw_info->Fog;
    Properties::FogDensity = draw_info->FogDensity;
    Properties::DistColor = draw_info->DistColor;
    Properties::xoffset = draw_info->xoffset;
    Properties::yoffset = draw_info->yoffset;
    Properties::fontname = draw_info->fontname;
    if(changedCell)
       jav->SetCell();
}

void OptionsDialog::restoreDispInfo(bool ifDefault)
{
    _disp_info *disp_info;
    if(ifDefault)
       disp_info = &DispDefault;
    else
       disp_info = &DispSav;

    bool changed_ortho = false;
    Properties::Set_AA(param_ENABLE, disp_info->MSEnabled);
    Properties::Set_AA(param_BUFFERS, disp_info->MSBuffers);
    Properties::ifGrid = disp_info->ifGrid;
    Properties::nDivisions = disp_info->nDivisions;
    Properties::GridPattern = disp_info->GridPattern;
    Properties::Stereo = disp_info->Stereo;
    Properties::glasses_type = disp_info->glasses_type;
    if(Properties::ifOrtho != disp_info->ifOrtho)
       changed_ortho = true;
    Properties::ifOrtho = disp_info->ifOrtho;
    if(changed_ortho)
       jav->ChangeOrtho();
}

void OptionsDialog::restoreLightInfo(bool ifDefault)
{
    if(ifDefault)
       Properties::DefaultLights();
    else
    {
       for(int j=0; j<3; j++)
           Properties::lightInfo[j] = LightSav[j];
    }
}

void OptionsDialog::restorePolyInfo()
{
    for(int k=0; k<MAX_NUM_MODELS; k++)
    {
	GetModel(k)->nPolyStyles = 0;
	GetModel(k)->FillPolyhedraStyles();
    }
}

void OptionsDialog::setOptions()
{
    int nmodels = Properties::nModels;
    if(ModelSav != nullptr)
        delete [] ModelSav;
    ModelSav = new _model_info[MAX_NUM_MODELS]();
    saveInfo();
    m_kmodel = Properties::GetActiveModel()-1;
    if(m_kmodel == -1)
        m_kmodel = 0;
    Properties::SetActiveModel(m_kmodel+1);
    //Set number of models and name for combo
    ui->labelModels->setVisible(nmodels > 1);
    ui->comboBoxModels->setVisible(nmodels > 1);
    ui->comboBoxModels->blockSignals(true);
    ui->comboBoxModels->clear();
    if (nmodels > 1)
    {
        for (int i=0; i<nmodels; i++)
            ui->comboBoxModels->addItem(jav->ModName(i+1));
        ui->comboBoxModels->setCurrentIndex(m_kmodel);
    }
    ui->comboBoxModels->blockSignals(false);
    fillOptionsModel(m_kmodel + 1);

    fillAtomsDefault();
    fillPageAtoms();   
    fillPageBonds();   
    fillPageMaps();
    fillPageDraw();
    fillPagePoly();
    fillPageDisp();
    fillPageLight();
}

void OptionsDialog::colorspecies_changed(QColor newColor, int k)
{
    Properties::SetColor(k, newColor);
    for (int i = 0; i < Properties::nModels; i++)
     jav->GetModel(i+1)->light_diffuse[k] = newColor;
    reDraw();
}

void OptionsDialog::fillPageAtoms()
{
    QGridLayout *layout = ui->atomGridLayout;
    removeChildren(layout);

    QLabel *label = new QLabel(this);
    label->setText("Atom");
    layout->addWidget(label, 0, 0, Qt::AlignLeft);
    label = new QLabel(this);
    label->setText("Colour");
    layout->addWidget(label, 0, 1, Qt::AlignLeft);
    label = new QLabel(this);
    label->setText("Radius");
    layout->addWidget(label, 0, 2, Qt::AlignLeft);

    m_chainbutton = new ChainButton();
    layout->addWidget(m_chainbutton, 0, 3, Qt::AlignCenter);
    for (int i = 0; i < Properties::NSpecies; i++)
    {
           int z = Properties::ListaSpecies[i]-1;
           QString item_lab = get_nth_element_name(Properties::ListaSpecies[i]-1);
           label = new QLabel(this);
           label->setText(item_lab);
           layout->addWidget(label, i+1, 0, Qt::AlignLeft);

           QColor colore = Properties::GetColor(z);
           ColorButton *cbutton = new ColorButton(colore, z, this);
           layout->addWidget(cbutton, i+1, 1, Qt::AlignLeft);
           connect(cbutton, &ColorButton::colorChanged, this,
                   &OptionsDialog::colorspecies_changed);

           MySlider *slider = new MySlider(this, z, 0.2, 3,
			   Properties::scale_atoms[z], 0.01, 2);
           layout->addWidget(slider, i+1, 2, Qt::AlignLeft);
           connect(slider, &MySlider::sliderChanged, this,
                   &OptionsDialog::radiusspecies_changed);
    }
    ui->radioCovalent->setChecked(!Properties::RadType);
    ui->radioThermal->setChecked(Properties::RadType);
    ui->spinEllTransparency->setValue(Properties::ell_tr);
    ui->spinProb->setValue(Properties::ell_prob);
    ui->SliderProb->setValue(Properties::ell_prob);
    ui->checkEllPrinc->setChecked(Properties::ell_princ);
    ui->checkEllHydro->setChecked(Properties::ell_hydro);
    ui->ellPrincColor->SetColor(Properties::ell_color);
}

void OptionsDialog::fillPageBonds()
{
    ui->bondRadiusSpinBox->setValue(Properties::sc_bonds);
    ui->MultipleBondscheck->setChecked(Properties::ShowMultBonds);
    ui->SBondsColorButton->SetColor(Properties::ColorSC);
    ui->HBondsColorButton->SetColor(Properties::ColorHB);
}

void OptionsDialog::fillPageMaps()
{
    ui->spinBoxMaps->setValue(Properties::Radius);
    ui->spinBoxX->setValue(Properties::xyzC.x);
    ui->spinBoxY->setValue(Properties::xyzC.y);
    ui->spinBoxZ->setValue(Properties::xyzC.z);
}

void OptionsDialog::fillPageDraw()
{
    ui->backgroundCombo->setColorWidget(Properties::BgType, Properties::BgColor, Properties::BgColor1, Properties::BgColor2);
    ui->cellEdgesCheck->setChecked(Properties::ifCell);
    ui->cellEdgesColorButton->setEnabled(Properties::ifCell);
    ui->cellEdgesColorButton->SetColor(Properties::CellColor);
    ui->cellEdgesColorButton->setEnabled(Properties::ifCell);
    ui->cAxesVisibleCheck->setChecked(Properties::ifAxes);
    ui->displayInCellCheck->setEnabled(Properties::ifCell);
    ui->displayInCellCheck->setChecked(Properties::ifDrawAxes);
    ui->clipEnableCheck->setChecked(Properties::Clip);
    ui->clipSpinBox->setEnabled(Properties::Clip);
    ui->clipSpinBox->setValue(Properties::ClipFactor);
    ui->fogEnableCheck->setChecked(Properties::Fog);
    ui->fogSpinBox->setEnabled(Properties::Fog);
    ui->fogSpinBox->setValue(Properties::FogDensity);
    ui->distancesColorButton->SetColor(Properties::DistColor);
    QFont font;
    font.fromString(Properties::fontname);
    ui->labelFontButton->setText(font.family()+" "+QString::number(font.pointSize()));
    ui->labelFontButton->setFont(font);
    
    ui->xOffsetSpin->setValue(Properties::xoffset);
    ui->yOffsetSpin->setValue(Properties::yoffset);
}

void OptionsDialog::fillPageDisp()
{
    ui->comboBoxFilter->blockSignals(true);
    ui->comboBoxFilter->clear();
    int ns = 0;

    if(!Properties::samples().empty())
       ns = Properties::samples().size();
    ui->groupBox_MS->setEnabled(ns > 0);
    if(ns > 0)
    {
       for (int i=0; i<ns; i++) 
       {
	 ui->comboBoxFilter->addItem(QString::number(Properties::samples().at(i))+ "x");
         if(Properties::samples().at(i) == Properties::MSBuffers())
            ui->comboBoxFilter->setCurrentIndex(i);
       }
       ui->checkBoxMultiS->setChecked(Properties::IsMSEnabled());
    }
    ui->comboBoxFilter->blockSignals(false);
    ui->labelFilter->setEnabled(Properties::IsMSEnabled());
    ui->comboBoxFilter->setEnabled(Properties::IsMSEnabled());
    ui->boxViewGrid->setChecked(Properties::ifGrid);    
    ui->spinBoxGridDiv->setValue(Properties::nDivisions);
    ui->comboBoxGridStyle->setCurrentIndex(Properties::GridPattern);
    ui->groupBox3D->setChecked(Properties::Stereo);    
    QString nome = "radioButton" + glasses.at(Properties::glasses_type);
    QRadioButton *w = findChild<QRadioButton *> (nome);
    if(w != nullptr)
       w->setChecked(true);
    ui->radioOrtho->blockSignals(true);
    ui->radioPersp->blockSignals(true);
    if(Properties::ifOrtho)
       ui->radioOrtho->setChecked(true);
    else
       ui->radioPersp->setChecked(true);
    ui->radioOrtho->blockSignals(false);
    ui->radioPersp->blockSignals(false);
}

void OptionsDialog::on_comboBoxFilter_currentIndexChanged(int index)
{
    if(index < 0)
       return;
    if(!Properties::samples().empty() && (Properties::samples().size() > index)) 
    Properties::Set_AA(param_BUFFERS, Properties::samples().at(index)); 
    reDraw();
}

void OptionsDialog::on_checkBoxMultiS_stateChanged(int arg1)
{
    Properties::Set_AA(param_ENABLE, (arg1 == Qt::Checked)?1:0);
    ui->labelFilter->setEnabled(arg1 == Qt::Checked);
    ui->comboBoxFilter->setEnabled(arg1 == Qt::Checked);
    reDraw();
}


void OptionsDialog::fillPageLight()
{
    ui->boxLight1->setChecked(Properties::lightInfo[JAV_LIGHT0].enabled);
    ui->boxLight2->setChecked(Properties::lightInfo[JAV_LIGHT1].enabled);
    ui->boxLight3->setChecked(Properties::lightInfo[JAV_AMBIENT].enabled);
    ui->pushButtonL1D->SetAlpha(true);
    ui->pushButtonL1S->SetAlpha(true);
    ui->pushButtonL2D->SetAlpha(true);
    ui->pushButtonL2S->SetAlpha(true);
    ui->pushButtonL3D->SetAlpha(true);
    ui->pushButtonL1D->SetColor(Properties::lightInfo[JAV_LIGHT0].diffuse);
    ui->pushButtonL1S->SetColor(Properties::lightInfo[JAV_LIGHT0].specular);
    ui->pushButtonL2D->SetColor(Properties::lightInfo[JAV_LIGHT1].diffuse);
    ui->pushButtonL2S->SetColor(Properties::lightInfo[JAV_LIGHT1].specular);
    ui->pushButtonL3D->SetColor(Properties::lightInfo[JAV_AMBIENT].diffuse);
    ui->spinBoxL1x->setValue(Properties::lightInfo[JAV_LIGHT0].position[0]);
    ui->spinBoxL1y->setValue(Properties::lightInfo[JAV_LIGHT0].position[1]);
    ui->spinBoxL1z->setValue(Properties::lightInfo[JAV_LIGHT0].position[2]);
    ui->spinBoxL2x->setValue(Properties::lightInfo[JAV_LIGHT1].position[0]);
    ui->spinBoxL2y->setValue(Properties::lightInfo[JAV_LIGHT1].position[1]);
    ui->spinBoxL2z->setValue(Properties::lightInfo[JAV_LIGHT1].position[2]);
}

void OptionsDialog::setOptionsModel(int kmodel)
{
    md = jav->GetModel(kmodel);
}

void OptionsDialog::fillOptionsModel(int kmodel, bool ifSave)
{
    if(ifSave)
       saveInfo(kmodel-1);
    setOptionsModel(kmodel);
    //Set title and state of groupbox
    ui->groupBoxStyle->setTitle(QString("Visualize Model n.%1: %2").arg(kmodel).arg(QString(jav->ModName(kmodel))));
    ui->groupBoxStyle->setChecked(md->ifVis);

    //Fill combo with style list
    QStringListModel *model = new QStringListModel();
    QStringListModel *model1 = new QStringListModel();
    if (md->IsProtein()) {
        model->setStringList(styleList+styleListProt);
        model1->setStringList(colorList+colorListProt);
    } else {
        model->setStringList(styleList);
        model1->setStringList(colorList);
    }
    ui->plotStyleComboBox->blockSignals(true);
    ui->coloursSettingComboBox->blockSignals(true);
    ui->plotStyleComboBox->setModel(model);
    ui->coloursSettingComboBox->setModel(model1);

    //Set widgets for style and color
    ui->plotStyleComboBox->setCurrentIndex(md->plotStyle);
    ui->coloursSettingComboBox->setCurrentIndex(md->colorMode);
    ui->plotStyleComboBox->blockSignals(false);
    ui->coloursSettingComboBox->blockSignals(false);
    ui->coloursSettingButton->setVisible(ui->coloursSettingComboBox->currentText() == "monochromatic");
    ui->coloursSettingButton->SetColor(md->light_diffuse[MONOCOL]);
    ui->checkBoxLabel->setChecked(md->ifLabels);
    //set_color_pushbutton(ui->labelsColorButton, md->labelcolor);
    ui->labelsColorButton->SetColor(md->labelcolor);
    ui->checkBoxHydrogens->setChecked(md->ifDrawH);
    ui->ligandsComboBox->setVisible(md->IsProtein());
    ui->checkBoxLigands->setVisible(md->IsProtein());
    ui->checkBoxLigands->setChecked((md->drawLigand > 0));
    ui->checkBoxWater->setVisible(md->IsProtein());
    ui->checkBoxWater->setChecked(md->ifDrawW);
    ui->ligandsComboBox->setEnabled(ui->checkBoxLigands->isChecked());
    if(md->drawLigand > 0)
       ui->ligandsComboBox->setCurrentIndex(md->drawLigand-1);

    //Set widgets for packing
    ui->groupBoxPacking->setChecked(md->ifSymmetry);
    ui->comboBoxPacking->setCurrentIndex(md->FitOpt);
    ui->spinA->setValue(md->klevel[0]);
    ui->spinB->setValue(md->klevel[1]);
    ui->spinC->setValue(md->klevel[2]);

    //Set widgets for Van der Waals
    //xRosi: inserisci le variabili corrispondenti al modello kmodel
    //xCorrado: questa parte entrerà in funzione solo quando avrò aggiunto la gestione delle mappe (minimo 2 mesi)

    //on_groupBoxPacking_clicked();
    //ui->groupBoxVdW->hide();
    ui->groupBoxVdW->setChecked(md->ifSurface);
    ui->comboBoxVdW->setCurrentIndex(md->VWSurface.wireframe);
    ui->styleVdWColorButton->SetColor(md->VWSurface.color_mappa);
    ui->radiusSpinBox->setValue(md->VWSurface.radius);
    ui->gridSpinBox->setValue(md->VWSurface.step);
}

void OptionsDialog::writeSettings()
{
    QSettings settings;
    settings.beginWriteArray(MODEL_OPTIONS_KEYS);    

    Properties::WriteSettings();
    for (int i=0; i < Properties::nModels; i++) {
        settings.setArrayIndex(i);
        settings.setValue(PLOT_STYLE_KEY,GetModel(i)->plotStyle);
        settings.setValue(COLOUR_STYLE_KEY,GetModel(i)->colorMode);
        settings.setValue(LABEL_COLOUR_KEY,GetModel(i)->labelcolor);
        settings.setValue(COLOR_MONO_KEY,GetModel(i)->light_diffuse[MONOCOL]);
        settings.setValue(DRAW_LABELS_KEY,GetModel(i)->ifLabels);
        settings.setValue(DRAW_HYD_KEY,GetModel(i)->ifDrawH);
        settings.setValue(DRAW_WAT_KEY,GetModel(i)->ifDrawW);
        settings.setValue(DRAW_LIG_KEY,GetModel(i)->drawLigand);
        //settings.setValue(PACKING_KEY, GetModel(i)->ifSymmetry);
        settings.setValue(PACKLEV0_KEY, GetModel(i)->klevel[0]);
        settings.setValue(PACKLEV1_KEY, GetModel(i)->klevel[1]);
        settings.setValue(PACKLEV2_KEY, GetModel(i)->klevel[2]);
        settings.setValue(FITOPT_KEY, GetModel(i)->FitOpt);
    }
    settings.endArray();
}

void OptionsDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    //if ((QPushButton *)button == ui->buttonBox->button(QDialogButtonBox::Ok)){
    if (qobject_cast<QPushButton *>(button) == ui->buttonBox->button(QDialogButtonBox::Ok)){
        //qDebug() << "Button OK";
        writeSettings();
    //} else if ((QPushButton *)button == ui->buttonBox->button(QDialogButtonBox::Cancel)) {
    } else if (qobject_cast<QPushButton *>(button) == ui->buttonBox->button(QDialogButtonBox::Cancel)) {
        restoreInfo();
        reDraw();
        //qDebug() << "Button Cancel";
    } else {
	restoreInfo(true);
        reDraw();
        //qDebug() << "Restore defaults for page: " << ui->stackedWidget->currentIndex();
    }
}

void OptionsDialog::reject()
{
    //qDebug() << "Dialog closed from x button";
    restoreInfo();
    reDraw();
    QDialog::reject();
}

void OptionsDialog::on_comboBoxModels_currentIndexChanged(int index)
{
    m_kmodel = index;
    Properties::SetActiveModel(m_kmodel+1);
    fillOptionsModel(index+1);
}

void OptionsDialog::on_plotStyleComboBox_currentIndexChanged(int index)
{
    md->plotStyle = index;
    reDraw();
    //qDebug() << "Selected style " << ui->plotStyleComboBox->currentIndex();
}

//void OptionsDialog::on_checkBoxLabel_stateChanged(int arg1)
void OptionsDialog::on_checkBoxLabel_clicked()
{
    jav->SetLabels();
    //md->ifLabels = (arg1 == Qt::Checked);
    //reDraw();
}

void OptionsDialog::on_coloursSettingComboBox_activated(int index)
{
    ui->coloursSettingButton->setVisible(index == MONOC);
    md->colorMode = index;
    reDraw();
}

void OptionsDialog::set_color_pushbutton(QPushButton *button, const QColor &color)
{
    QPixmap pixmap(32,32);
    pixmap.fill(color);
    QIcon newIcon(pixmap);
    button->setIcon(newIcon);
}

void OptionsDialog::on_coloursSettingButton_colorChanged(QColor newcolor)
{
    md->light_diffuse[MONOCOL] = newcolor;
    reDraw();
}

void OptionsDialog::on_labelsColorButton_colorChanged(QColor newColor)
{
    md->labelcolor = newColor;
    reDraw();
}

void OptionsDialog::on_checkBoxHydrogens_stateChanged(int arg1)
{
    md->ifDrawH = (arg1 == Qt::Checked);
    reDraw();
}

void OptionsDialog::on_groupBoxPacking_clicked()
{
    if (ui->groupBoxPacking->isChecked())
        ui->groupBoxVdW->setChecked(false);
    ui->groupBoxVdW->setEnabled(!ui->groupBoxPacking->isChecked());
    md->ifSymmetry = ui->groupBoxPacking->isChecked();
    jav->SetPacking();
}

void OptionsDialog::setCellRange(double arg1, int spinNum)
{
    if (ui->pushButtonRange->isChecked()) {
        if (spinNum != 1) ui->spinA->setValue(arg1);
        if (spinNum != 2) ui->spinB->setValue(arg1);
        if (spinNum != 3) ui->spinC->setValue(arg1);
    }
    md->klevel[0] = ui->spinA->value();
    md->klevel[1] = ui->spinB->value();
    md->klevel[2] = ui->spinC->value();
    jav->SetPacking(false);
    reDraw();
}

void OptionsDialog::on_spinA_valueChanged(double arg1)
{    
    setCellRange(arg1,1);
}

void OptionsDialog::on_spinB_valueChanged(double arg1)
{
    setCellRange(arg1,2);
}

void OptionsDialog::on_spinC_valueChanged(double arg1)
{
    setCellRange(arg1,3);
}

void OptionsDialog::on_comboBoxPacking_activated(int index)
{
    md->FitOpt = index;
    jav->SetPacking(false);
    reDraw();
}

void OptionsDialog::on_ligandsComboBox_currentIndexChanged(int index)
{
    if(ui->checkBoxLigands->isEnabled())
       md->drawLigand = index + 1;
    reDraw();
}

void OptionsDialog::on_checkBoxLigands_stateChanged(int arg1)
{
    ui->ligandsComboBox->setEnabled((arg1 == Qt::Checked));
    if(arg1 == Qt::Checked)
       md->drawLigand = ui->ligandsComboBox->currentIndex()+1;
    else 
       md->drawLigand = 0;
    reDraw();
}

void OptionsDialog::on_groupBoxVdW_clicked(bool checked)
{
    md->ifSurface = checked;
    md->SettaSurface(checked);
}

void OptionsDialog::on_comboBoxVdW_activated(int index)
{
    md->VWSurface.wireframe = index;
    reDraw();
}

void OptionsDialog::on_styleVdWColorButton_colorChanged(QColor newcolor)
{
    md->VWSurface.color_mappa = newcolor;
    reDraw();
}

void OptionsDialog::on_radiusSpinBox_valueChanged(double arg1)
{
    if(arg1 != md->VWSurface.radius)
    {
       md->VWSurface.radius = arg1;
       md->RicalcolaSurface();
       reDraw();
    }
}

void OptionsDialog::on_gridSpinBox_valueChanged(double arg1)
{
    if(arg1 != md->VWSurface.step)
    {
       md->VWSurface.step = arg1;
       md->RicalcolaSurface();
       reDraw();
    }
}

void OptionsDialog::SliderToDoubleSpin(int value, const QSlider* slider, QDoubleSpinBox* doubleSpin)
{
    //Convert value to double for spinbox
    double coeff = (doubleSpin->maximum() - doubleSpin->minimum())/
            (slider->maximum() - slider->minimum());
    double spinValue =-coeff*slider->minimum()+doubleSpin->minimum()+coeff*value;

    //blockSignals is used to avoid signal valueChanged from spin
    doubleSpin->blockSignals(true);
    doubleSpin->setValue(spinValue);
    doubleSpin->blockSignals(false);
    Properties::sc_bonds = spinValue;
    reDraw();
    //qDebug() << "From Slider: " << spinValue;
}

void OptionsDialog::DoubleSpinToSlider(double value, const QDoubleSpinBox* doubleSpin, QSlider* slider)
{
    // Convert arg1 to int for slider
    double coeff = (slider->maximum() - slider->minimum())/
            (doubleSpin->maximum() - doubleSpin->minimum());
    int sliderValue = static_cast<int>(-coeff*doubleSpin->minimum()+slider->minimum()+coeff*value);

    //blockSignals is used to avoid signal valueChanged from slider
    slider->blockSignals(true);
    slider->setValue(sliderValue);
    slider->blockSignals(false);
    //qDebug() << "From Spin: " << static_cast<int>(sliderValue);
}

void OptionsDialog::on_radioCovalent_toggled(bool checked)
{
    Properties::RadType = !checked;
    if(checked)
       reDraw();
}

void OptionsDialog::on_radioThermal_toggled(bool checked)
{
    Properties::RadType = checked;
    if(checked)
       reDraw();
}

void OptionsDialog::on_spinEllTransparency_valueChanged(int value)
{
    Properties::ell_tr = value;
    ui->EllTransparency->blockSignals(true);
    ui->EllTransparency->setValue(value);
    ui->EllTransparency->blockSignals(false);
    reDraw();
}

void OptionsDialog::on_spinProb_valueChanged(int value)
{
    Properties::ell_prob = value;
    ui->SliderProb->blockSignals(true);
    ui->SliderProb->setValue(value);
    ui->SliderProb->blockSignals(false);
    md->AddProb();
    reDraw();
}

void OptionsDialog::on_checkEllPrinc_stateChanged(int arg1)
{
    Properties::ell_princ = (arg1 == Qt::Checked);
    reDraw();
}

void OptionsDialog::on_checkEllHydro_stateChanged(int arg1)
{
    Properties::ell_hydro = (arg1 == Qt::Checked);
    reDraw();
}

void OptionsDialog::on_ellPrincColor_colorChanged(QColor newcolor)
{
    Properties::ell_color = newcolor;
    reDraw();
}

void OptionsDialog::on_bondRadiusSlider_valueChanged(int value)
{
    SliderToDoubleSpin(value, ui->bondRadiusSlider, ui->bondRadiusSpinBox);
}

void OptionsDialog::on_bondRadiusSpinBox_valueChanged(double arg1)
{
    DoubleSpinToSlider(arg1, ui->bondRadiusSpinBox, ui->bondRadiusSlider);
    Properties::sc_bonds = arg1;
    reDraw();
}

void OptionsDialog::on_HBondsColorButton_colorChanged(QColor newcolor)
{
    Properties::ColorHB = newcolor;
    reDraw();
}

void OptionsDialog::on_SBondsColorButton_colorChanged(QColor newcolor)
{
    Properties::ColorSC = newcolor;
    reDraw();
}

void OptionsDialog::on_MultipleBondscheck_stateChanged(int arg1)
{
    Properties::ShowMultBonds = (arg1 == Qt::Checked);
    reDraw();
}

void OptionsDialog::backgroundChanged()
{
    ui->backgroundCombo->getColorWidget(Properties::BgType, Properties::BgColor, Properties::BgColor1, Properties::BgColor2);
    reDraw();
}

void OptionsDialog::on_cellEdgesCheck_clicked()
{
    jav->SetCell();
    ui->cellEdgesColorButton->setEnabled(Properties::ifCell);
    ui->displayInCellCheck->setEnabled(Properties::ifCell);
}

void OptionsDialog::on_cellEdgesColorButton_colorChanged(QColor newcolor)
{
    Properties::CellColor = newcolor;
    reDraw();
}

void OptionsDialog::on_cAxesVisibleCheck_stateChanged(int arg1)
{
    Properties::ifAxes = (arg1 == Qt::Checked);
    reDraw();
}

void OptionsDialog::on_displayInCellCheck_stateChanged(int arg1)
{
    Properties::ifDrawAxes = (arg1 == Qt::Checked);
    reDraw();
}

void OptionsDialog::on_clipEnableCheck_stateChanged(int arg1)
{
    ui->clipSpinBox->setEnabled((arg1 == Qt::Checked));
    Properties::Clip = (arg1 == Qt::Checked);
    reDraw();
}

void OptionsDialog::on_clipSpinBox_valueChanged(double arg1)
{
    Properties::ClipFactor = arg1;
    reDraw();
}

void OptionsDialog::on_fogEnableCheck_stateChanged(int arg1)
{
    ui->fogSpinBox->setEnabled((arg1 == Qt::Checked));
    Properties::Fog = (arg1 == Qt::Checked);
    reDraw();
}

void OptionsDialog::on_fogSpinBox_valueChanged(double arg1)
{
    Properties::FogDensity = arg1;
    reDraw();
}

void OptionsDialog::on_distancesColorButton_colorChanged(QColor newcolor)
{
    Properties::DistColor = newcolor;
    reDraw();
}

void OptionsDialog::on_labelFontButton_clicked()
{
    const int MAXPOINTS = 16;
    bool ok;
    QFont font;
    font.fromString(Properties::fontname);
    font = QFontDialog::getFont(&ok, font, this);
    if (ok) {
        // Set font and text for the button
        QFont buttonFont(font);
        if (buttonFont.pointSize() > MAXPOINTS) buttonFont.setPointSize(MAXPOINTS); // avoid large button!
        ui->labelFontButton->setFont(buttonFont);
        Properties::fontname = buttonFont.toString();
        ui->labelFontButton->setText(buttonFont.family()+" "+QString::number(font.pointSize()));
        reDraw();
    }
}

void OptionsDialog::on_xOffsetSpin_valueChanged(double arg1)
{
    Properties::xoffset = arg1;
    reDraw();
}

void OptionsDialog::on_yOffsetSpin_valueChanged(double arg1)
{
    Properties::yoffset = arg1;
    reDraw();
}

void OptionsDialog::radiusspecies_changed(double val, int z)
{
     if((z < 0) || (z>N_ELEMENTS))
	 return;

     Properties::scale_atoms[z] = val;
     bool joined = m_chainbutton->isChecked();
     if(joined)
     {
        MySlider::syncAll(this, val, z);
        for (int i = 0; i < Properties::NSpecies; i++)
        {
            int zz = Properties::ListaSpecies[i]-1;
            Properties::scale_atoms[zz] = val;
        }
     }
     Properties::sc_atoms = 1;
     for(int i=0; i<N_ELEMENTS; i++)
          Properties::sc_atoms = max(Properties::sc_atoms, Properties::scale_atoms[i]);
     reDraw();
}

void OptionsDialog::polyVis_changed(QCheckBox *check, int i)
{
    if(i >= 10000)
       md->PolyStyle[i-10000].ifBorder = check->isChecked();
    else
       md->PolyStyle[i].visible = check->isChecked();
    reDraw();
}

void OptionsDialog::colorpoly_changed(QColor newColor, int k)
{
    if(k > 10000)
       md->PolyStyle[k-1-10000].Bcolore = newColor;
    else
       md->PolyStyle[k-1].colore = newColor;
    reDraw();
}

void OptionsDialog::polyTransp_changed(int val, int i)
{
    md->PolyStyle[i].transparency = val;
    reDraw();
}

void OptionsDialog::polyLinewidth_changed(QDoubleSpinBox *spin, int i)
{
    md->PolyStyle[i].line_width = spin->value();
    reDraw();
}

void OptionsDialog::fillPagePoly()
{
   const QStringList titles = { "Centre", "Show", "Colour",
                           "Transparency", "Show",
                           "Colour", "Line width"};
	
    QGridLayout *layout = ui->polyGridLayout;
    removeChildren(layout);

    QLabel *label = new QLabel(this);
    label->setText("Planes");
    layout->addWidget(label, 0, 0, Qt::AlignCenter);
    label = new QLabel(this);
    label->setText("Edges");
    layout->addWidget(label, 0, 4, Qt::AlignCenter);
    
    for(int i=0; i<titles.size(); i++)
    {
	label = new QLabel(this);
	label->setText(titles.at(i));
        layout->addWidget(label, 1, i, Qt::AlignLeft);
    }
    for (int i = 0; i < md->nPolyStyles; i++)
    {
	 int v = md->PolyStyle[i].vert;
         label = new QLabel(this);
         label->setText(QString(md->atm_au[v].lab.c_str()));
         layout->addWidget(label, i+2, 0, Qt::AlignLeft);

	 QCheckBox *checkbox = new QCheckBox(this);
         layout->addWidget(checkbox, i+2, 1, Qt::AlignLeft);
	 checkbox->setChecked(md->PolyStyle[i].visible);
	 connect(checkbox, &QCheckBox::stateChanged, this,
	    [=] { polyVis_changed(checkbox, i); });

         QColor colore = md->PolyStyle[i].colore;
         ColorButton *cbutton = new ColorButton(colore, i+1, this);
	 cbutton->SetSquare();
         layout->addWidget(cbutton, i+2, 2, Qt::AlignLeft);
         connect(cbutton, &ColorButton::colorChanged, this,
                   &OptionsDialog::colorpoly_changed);

         MySlider *slider = new MySlider(this, i, 0, 100,
                        md->PolyStyle[i].transparency, 1);
         layout->addWidget(slider, i+2, 3, Qt::AlignLeft);
         connect(slider, &MySlider::intSliderChanged, this,
                   &OptionsDialog::polyTransp_changed);
	 
	 checkbox = new QCheckBox(this);
         layout->addWidget(checkbox, i+2, 4, Qt::AlignLeft);
	 checkbox->setChecked(md->PolyStyle[i].ifBorder);
	 connect(checkbox, &QCheckBox::stateChanged, this,
	    [=] { polyVis_changed(checkbox, i+10000); });

         colore = md->PolyStyle[i].colore;
         cbutton = new ColorButton(colore, i+1+10000, this);
	 cbutton->SetSquare();
         layout->addWidget(cbutton, i+2, 5, Qt::AlignLeft);
         connect(cbutton, &ColorButton::colorChanged, this,
                   &OptionsDialog::colorpoly_changed);

	 QDoubleSpinBox *spinBox = new QDoubleSpinBox(this);
         spinBox->setRange(0.1, 10);
         spinBox->setSingleStep(0.1);
         spinBox->setValue(md->PolyStyle[i].line_width);
         layout->addWidget(spinBox, i+2, 6, Qt::AlignLeft);
#if (QT_VERSION >= QT_VERSION_CHECK(5, 7, 0))
         connect(spinBox, QOverload<double>::of(&QDoubleSpinBox::valueChanged),
	     this, [=] {polyLinewidth_changed(spinBox, i); });
#else
         connect(spinBox, static_cast<void(QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
         this, [=] {polyLinewidth_changed(spinBox, i); });
#endif
    }
}

void OptionsDialog::on_checkBoxFillUC_clicked(bool checked)
{
    ui->labelMap->setDisabled(checked);
    ui->spinBoxMaps->setDisabled(checked);
    ui->labelCentre->setDisabled(checked);
    ui->pushButtonCentre->setDisabled(checked);
    ui->spinBoxX->setDisabled(checked);
    ui->spinBoxY->setDisabled(checked);
    ui->spinBoxZ->setDisabled(checked);
    Properties::ifWhole = checked;
    if(Properties::ifWhole)
    {
        Properties::xyzC.x = Properties::xyzC.y = Properties::xyzC.z =0.5;
        jav->RiNormalizza();
        reDraw ();
    }

}

void OptionsDialog::on_pushButtonCentre_clicked()
{
    bool visibleModel = false;
    for(int i=0; i<Properties::nModels; i++)
          if(jav->GetModel(i+1)->ifVis)
             visibleModel = true;
    if(!visibleModel)
    {
        QMessageBox::warning(this, tr("Error"),
                  tr("Sorry. No visible atoms."),QMessageBox::Cancel);
         return;
    }
    jav->setMotionType(SELECT_CENTRE);
}

void OptionsDialog::on_comboBoxGridStyle_currentIndexChanged(int index)
{
    Properties::GridPattern = index;
    reDraw();
}

void OptionsDialog::on_boxViewGrid_clicked()
{
    Properties::ifGrid = ui->boxViewGrid->isChecked();
    reDraw();
}

void OptionsDialog::on_spinBoxGridDiv_valueChanged(int arg1)
{
    Properties::nDivisions = arg1;
    reDraw();
}

void OptionsDialog::on_radioButtonRB_toggled(bool checked)
{
    if(!checked)
       return;
    Properties::glasses_type = 0;
    reDraw();
}

void OptionsDialog::on_radioButtonRG_toggled(bool checked)
{
    if(!checked)
       return;
    Properties::glasses_type = 1;
    reDraw();
}

void OptionsDialog::on_radioButtonRC_toggled(bool checked)
{
    if(!checked)
       return;
    Properties::glasses_type = 2;
    reDraw();
}

void OptionsDialog::on_radioButtonBR_toggled(bool checked)
{
    if(!checked)
       return;
    Properties::glasses_type = 3;
    reDraw();
}

void OptionsDialog::on_radioButtonGR_toggled(bool checked)
{
    if(!checked)
       return;
    Properties::glasses_type = 4;
    reDraw();
}

void OptionsDialog::on_radioButtonCR_toggled(bool checked)
{
    if(!checked)
       return;
    Properties::glasses_type = 5;
    reDraw();
}

void OptionsDialog::on_groupBox3D_clicked()
{
    Properties::Stereo = ui->groupBox3D->isChecked();
    ui->radioOrtho->setEnabled(!Properties::Stereo);
    ui->radioPersp->setEnabled(!Properties::Stereo);
    if(Properties::ifDesaturate())
    {
       Properties::oldOrtho = Properties::ifOrtho;
       Properties::ifOrtho = false;
    }
    else
       Properties::ifOrtho = Properties::oldOrtho;
    ui->radioOrtho->blockSignals(true);
    if(Properties::ifOrtho)
       ui->radioOrtho->setChecked(true);
    else
       ui->radioPersp->setChecked(true);
    ui->radioOrtho->blockSignals(false);
    jav->ChangeOrtho();
}

void OptionsDialog::on_radioOrtho_toggled(bool checked)
{
    Properties::ifOrtho = checked;
    jav->ChangeOrtho();
}

void OptionsDialog::on_boxLight1_toggled(bool checked)
{
    Properties::lightInfo[JAV_LIGHT0].enabled = checked;
    reDraw();
}

void OptionsDialog::on_boxLight2_toggled(bool checked)
{
    Properties::lightInfo[JAV_LIGHT1].enabled = checked;
    reDraw();
}

void OptionsDialog::on_boxLight3_toggled(bool checked)
{
    Properties::lightInfo[JAV_AMBIENT].enabled = checked;
    reDraw();
}

void OptionsDialog::on_spinBoxL1x_valueChanged(double arg1)
{
    Properties::lightInfo[JAV_LIGHT0].position[0] = arg1;
    reDraw();
}

void OptionsDialog::on_spinBoxL1y_valueChanged(double arg1)
{
    Properties::lightInfo[JAV_LIGHT0].position[1] = arg1;
    reDraw();
}

void OptionsDialog::on_spinBoxL1z_valueChanged(double arg1)
{
    Properties::lightInfo[JAV_LIGHT0].position[2] = arg1;
    reDraw();
}

void OptionsDialog::on_spinBoxL2x_valueChanged(double arg1)
{
    Properties::lightInfo[JAV_LIGHT1].position[0] = arg1;
    reDraw();
}

void OptionsDialog::on_spinBoxL2y_valueChanged(double arg1)
{
    Properties::lightInfo[JAV_LIGHT1].position[1] = arg1;
    reDraw();
}

void OptionsDialog::on_spinBoxL2z_valueChanged(double arg1)
{
    Properties::lightInfo[JAV_LIGHT1].position[2] = arg1;
    reDraw();
}

void OptionsDialog::on_pushButtonL1D_colorChanged(QColor newcolor)
{
    Properties::lightInfo[JAV_LIGHT0].diffuse = newcolor;
    reDraw();
}

void OptionsDialog::on_pushButtonL1S_colorChanged(QColor newcolor)
{
    Properties::lightInfo[JAV_LIGHT0].specular = newcolor;
    reDraw();
}

void OptionsDialog::on_pushButtonL2D_colorChanged(QColor newcolor)
{
    Properties::lightInfo[JAV_LIGHT1].diffuse = newcolor;
    reDraw();
}

void OptionsDialog::on_pushButtonL2S_colorChanged(QColor newcolor)
{
    Properties::lightInfo[JAV_LIGHT1].specular = newcolor;
    reDraw();
}

void OptionsDialog::on_pushButtonL3D_colorChanged(QColor newcolor)
{
    Properties::lightInfo[JAV_AMBIENT].diffuse = newcolor;
    reDraw();
}

void OptionsDialog::on_spinBoxMaps_valueChanged(double arg1)
{
    Properties::Radius = arg1;
    reDraw();
}

void OptionsDialog::on_spinBoxX_valueChanged(double arg1)
{
    Properties::xyzC.x = arg1;
    reDraw();
}
void OptionsDialog::on_spinBoxY_valueChanged(double arg1)
{
    Properties::xyzC.y = arg1;
    reDraw();
}
void OptionsDialog::on_spinBoxZ_valueChanged(double arg1)
{
    Properties::xyzC.z = arg1;
    reDraw();
}
