#include "overlayinfo.h"
#include "ui_overlayinfo.h"
#include <QStringListModel>

OverlayInfo::OverlayInfo(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::OverlayInfo),
    kmodel(1),
    ovmodel(2)
{
    ui->setupUi(this);
    QStringList titles = { "Atom", "Coordinates", "Distance",
                               "Atom", "Coordinates"};
    matchesList = new QStandardItemModel(this);
    matchesList->setHorizontalHeaderLabels(titles);
    ui->matches->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->matches->verticalHeader()->hide();

    ui->matches->setModel(matchesList);
    ui->matches->resizeColumnsToContents();
    ui->results->viewport()->setAutoFillBackground(false);
}

OverlayInfo::~OverlayInfo()
{
    delete ui;
}

void OverlayInfo::fillComboModels(QStringList list)
{
    QStringListModel *model = new QStringListModel();
    model->setStringList(list);
    ui->comboModels->blockSignals(true);
    ui->comboModels->setModel(model);
    ui->comboModels->setCurrentIndex(list.count()-1);
    ui->comboModels->blockSignals(false);
    ovmodel = ui->comboModels->currentIndex()+2;
}

float OverlayInfo::getDist()
{
    return ((float)ui->spinDist->value());
}

void OverlayInfo::fillResults(compare_info_type *info, QString nome1, int n1, int n2)
{
    int nrows = matchesList->rowCount();
    matchesList->removeRows(0, nrows);
    QString newheader = QString("x,y,z (%1)").arg(ui->comboModels->currentText());
    matchesList->setHeaderData(4, Qt::Horizontal,newheader, Qt::DisplayRole);
    for(int i=0; i<info->nass; i++)
    {
        QList<QStandardItem *> items;
        QStandardItem *item;
        int n1 = info->nv1[i] - 1;
        int n2 = info->nv2[i] - 1;
        item = new QStandardItem(QString(info->at1[n1].lab).trimmed());
        items.append(item);
        QString coord1 = QString::number(info->at1[n1].xc[0], 'f', 3);
        for(int j=1; j<3; j++)
        {
            coord1.append(" ");
            coord1.append(QString::number(info->at1[n1].xc[j], 'f', 3));
        }
        item = new QStandardItem(coord1);
        items.append(item);
        item = new QStandardItem(QString::number(info->vdist[i], 'f', 3));
        items.append(item);
        item = new QStandardItem(QString(info->at2[n2].lab).trimmed());
        items.append(item);
        coord1 = QString::number(info->at2[n2].xc[0], 'f', 3);
        for(int j=1; j<3; j++)
        {
            coord1.append(" ");
            coord1.append(QString::number(info->at2[n2].xc[j], 'f', 3));
        }
        item = new QStandardItem(coord1);
        items.append(item);
        matchesList->appendRow(items);
    }
    ui->matches->resizeColumnsToContents();
    QItemSelectionModel *selectionModel = ui->matches->selectionModel();
    connect(selectionModel, &QItemSelectionModel::selectionChanged,
           this, &OverlayInfo::rowSelectionChanged);

    QString results = "Atoms in ";
    results.append(nome1);
    results.append(": ");
    results.append(QString::number(n1));
    results.append("<br/>");
    results.append("Atoms in ");
    results.append(ui->comboModels->currentText());
    results.append(": ");
    results.append(QString::number(n2));
    results.append("<br/>");
    results.append("Matches found:     ");
    results.append(QString::number(info->nfound));
    results.append("<br/>");
    results.append("Mean Phase Error:  ");
    results.append(QString::number(info->errph, 'f', 3));
    results.append(" using ");
    results.append(QString::number(info->nref));
    results.append(" reflections<br/>");
    results.append("&lt;Dist&gt;:             ");
    results.append(QString::number(info->distm, 'f', 3));
    results.append("<br/>");
    results.append("RMSD:               ");
    results.append(QString::number(info->rmsd, 'f', 3));
    results.append("<br/>");
    if(info->rmsd_nonH >= 0)
    {
       results.append("RMSD (non H atoms): ");
       results.append(QString::number(info->rmsd_nonH));
       results.append("<br/>");
    }
    ui->results->setText(results);
}

void OverlayInfo::setDist(float val)
{
    ui->spinDist->blockSignals(true);
    ui->spinDist->setValue((double) val);
    ui->spinDist->blockSignals(false);
}

void OverlayInfo::on_closeButton_clicked()
{
    reject();
}

void OverlayInfo::on_spinDist_valueChanged(double )
{
    emit(distChanged());
}

void OverlayInfo::on_comboModels_currentIndexChanged(int index)
{
    ovmodel = index + 2;
    emit(overlayModelSelected());
}

void OverlayInfo::rowSelectionChanged(const QItemSelection &, const QItemSelection &)
{
     QVector<int> selectedRows;
     QModelIndexList selection = ui->matches->selectionModel()->selectedIndexes();
     for (const QModelIndex &index : selection) {
           if(index.column() == 0)
               selectedRows.push_back(index.row());
      }
      emit(overlaySelectionChanged(selectedRows));
}
