#include "profilefunctionwidget.h"
#include "ui_profilefunctionwidget.h"

#include <QDoubleValidator>
#include <QHashIterator>
#include <QDebug>

extern "C" void get_profile_functions(int data_id, int phase_id, int *typefun, c_profile_function cprof[]);
extern "C" void set_profile_functions(int data_id, int phase_id, int typefun, c_profile_function cprof[], int changed[]);

ProfileFunctionWidget::ProfileFunctionWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ProfileFunctionWidget),
    typeFunction(-1),
    functionTypeChanged(false)
{
    ui->setupUi(this);

    auto floatValidator = new QDoubleValidator();
    floatValidator->setDecimals(6);

    ui->scaleLineEdit->setValidator(floatValidator);
    ui->UPearsonLineEdit->setValidator(floatValidator);
    ui->VPearsonLineEdit->setValidator(floatValidator);
    ui->WPearsonLineEdit->setValidator(floatValidator);
    ui->beta0PearsonLineEdit->setValidator(floatValidator);
    ui->beta1PearsonLineEdit->setValidator(floatValidator);
    ui->beta2PearsonLineEdit->setValidator(floatValidator);
    ui->asym1PearsonLineEdit->setValidator(floatValidator);
    ui->asym2PearsonLineEdit->setValidator(floatValidator);
    ui->asym3PearsonLineEdit->setValidator(floatValidator);
    ui->asym4PearsonLineEdit->setValidator(floatValidator);
    ui->UPVoigtLineEdit->setValidator(floatValidator);
    ui->VPVoigtLineEdit->setValidator(floatValidator);
    ui->WPVoigtLineEdit->setValidator(floatValidator);
    ui->eta0PVoigtLineEdit->setValidator(floatValidator);
    ui->eta1PVoigtLineEdit->setValidator(floatValidator);
    ui->eta2PVoigtLineEdit->setValidator(floatValidator);
    ui->asym1PVoigtLineEdit->setValidator(floatValidator);
    ui->asym2PVoigtLineEdit->setValidator(floatValidator);
    ui->asym3PVoigtLineEdit->setValidator(floatValidator);
    ui->asym4PVoigtLineEdit->setValidator(floatValidator);
    ui->UTCHLineEdit->setValidator(floatValidator);
    ui->VTCHLineEdit->setValidator(floatValidator);
    ui->WTCHLineEdit->setValidator(floatValidator);
    ui->ZTCHLineEdit->setValidator(floatValidator);
    ui->XTCHLineEdit->setValidator(floatValidator);
    ui->YTCHLineEdit->setValidator(floatValidator);
    ui->asym1TCHLineEdit->setValidator(floatValidator);
    ui->asym2TCHLineEdit->setValidator(floatValidator);
    ui->asym3TCHLineEdit->setValidator(floatValidator);
    ui->asym4TCHLineEdit->setValidator(floatValidator);

//    connect(ui->functionTypeComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
//            ui->functionStackedWidget, &QStackedWidget::setCurrentIndex);

    connectLineEdit(ui->scaleLineEdit);
    connectLineEdit(ui->UPearsonLineEdit);
    connectLineEdit(ui->VPearsonLineEdit);
    connectLineEdit(ui->WPearsonLineEdit);
    connectLineEdit(ui->beta0PearsonLineEdit);
    connectLineEdit(ui->beta1PearsonLineEdit);
    connectLineEdit(ui->beta2PearsonLineEdit);
    connectLineEdit(ui->asym1PearsonLineEdit);
    connectLineEdit(ui->asym2PearsonLineEdit);
    connectLineEdit(ui->asym3PearsonLineEdit);
    connectLineEdit(ui->asym4PearsonLineEdit);
    connectLineEdit(ui->UPVoigtLineEdit);
    connectLineEdit(ui->VPVoigtLineEdit);
    connectLineEdit(ui->WPVoigtLineEdit);
    connectLineEdit(ui->eta0PVoigtLineEdit);
    connectLineEdit(ui->eta1PVoigtLineEdit);
    connectLineEdit(ui->eta2PVoigtLineEdit);
    connectLineEdit(ui->asym1PVoigtLineEdit);
    connectLineEdit(ui->asym2PVoigtLineEdit);
    connectLineEdit(ui->asym3PVoigtLineEdit);
    connectLineEdit(ui->asym4PVoigtLineEdit);
    connectLineEdit(ui->UTCHLineEdit);
    connectLineEdit(ui->VTCHLineEdit);
    connectLineEdit(ui->WTCHLineEdit);
    connectLineEdit(ui->ZTCHLineEdit);
    connectLineEdit(ui->XTCHLineEdit);
    connectLineEdit(ui->YTCHLineEdit);
    connectLineEdit(ui->asym1TCHLineEdit);
    connectLineEdit(ui->asym2TCHLineEdit);
    connectLineEdit(ui->asym3TCHLineEdit);
    connectLineEdit(ui->asym4TCHLineEdit);
}

ProfileFunctionWidget::~ProfileFunctionWidget()
{
    delete ui;
}

void ProfileFunctionWidget::setData(int phaseId, const crydata::crystal_t &cryst)
{
    //int typefun;
    auto prof = new c_profile_function[3];
    get_profile_functions(1, phaseId, &typeFunction, prof);

    setWidgetData(cryst, prof);

    delete [] prof;
}

void ProfileFunctionWidget::getData(int phaseId, crydata::crystal_t &cryst)
{
    //int typefun;
    auto prof = new c_profile_function[3];
    qInfo() << "Value Changed: " << hasFunctionValueChanged(0);
    getWidgetData(cryst, prof);

    int changed[3];
    changed[PEARSON7] = hasFunctionValueChanged(PEARSON7);
    changed[PVOIG] = hasFunctionValueChanged(PVOIG);
    changed[TCHZ] = hasFunctionValueChanged(TCHZ);
    set_profile_functions(1, phaseId, typeFunction, prof, changed);

    delete [] prof;
}

bool ProfileFunctionWidget::hasParameterToRefine()
{
    if (ui->scaleCheckBox->isChecked()) return true;

    if (ui->UPearsonCheckBox->isChecked()) return true;
    if (ui->VPearsonCheckBox->isChecked()) return true;
    if (ui->WPearsonCheckBox->isChecked()) return true;
    if (ui->beta0PearsonCheckBox->isChecked()) return true;
    if (ui->beta1PearsonCheckBox->isChecked()) return true;
    if (ui->beta2PearsonCheckBox->isChecked()) return true;
    if (ui->asym1PearsonCheckBox->isChecked()) return true;
    if (ui->asym2PearsonCheckBox->isChecked()) return true;
    if (ui->asym3PearsonCheckBox->isChecked()) return true;
    if (ui->asym4PearsonCheckBox->isChecked()) return true;

    if (ui->UPVoigtCheckBox->isChecked()) return true;
    if (ui->VPVoigtCheckBox->isChecked()) return true;
    if (ui->WPVoigtCheckBox->isChecked()) return true;
    if (ui->eta0PVoigtCheckBox->isChecked()) return true;
    if (ui->eta1PVoigtCheckBox->isChecked()) return true;
    if (ui->eta2PVoigtCheckBox->isChecked()) return true;
    if (ui->asym1PVoigtCheckBox->isChecked()) return true;
    if (ui->asym2PVoigtCheckBox->isChecked()) return true;
    if (ui->asym3PVoigtCheckBox->isChecked()) return true;
    if (ui->asym4PVoigtCheckBox->isChecked()) return true;

    if (ui->UTCHCheckBox->isChecked()) return true;
    if (ui->VTCHCheckBox->isChecked()) return true;
    if (ui->WTCHCheckBox->isChecked()) return true;
    if (ui->ZTCHCheckBox->isChecked()) return true;
    if (ui->XTCHCheckBox->isChecked()) return true;
    if (ui->YTCHCheckBox->isChecked()) return true;
    if (ui->asym1TCHCheckBox->isChecked()) return true;
    if (ui->asym2TCHCheckBox->isChecked()) return true;
    if (ui->asym3TCHCheckBox->isChecked()) return true;
    if (ui->asym4TCHCheckBox->isChecked()) return true;

    return false;
}

bool ProfileFunctionWidget::hasParameterChanged()
{
    if (functionTypeChanged) return true;
    int typefun = ui->functionTypeComboBox->currentIndex();    
    if (hasFunctionValueChanged(typefun)) return true;

    return false;
}

void ProfileFunctionWidget::setRefineEnabled(bool enable)
{
    ui->scaleCheckBox->setEnabled(enable);

    ui->UPearsonCheckBox->setEnabled(enable);
    ui->VPearsonCheckBox->setEnabled(enable);
    ui->WPearsonCheckBox->setEnabled(enable);
    ui->beta0PearsonCheckBox->setEnabled(enable);
    ui->beta1PearsonCheckBox->setEnabled(enable);
    ui->beta2PearsonCheckBox->setEnabled(enable);
    ui->asym1PearsonCheckBox->setEnabled(enable);
    ui->asym2PearsonCheckBox->setEnabled(enable);
    ui->asym3PearsonCheckBox->setEnabled(enable);
    ui->asym4PearsonCheckBox->setEnabled(enable);

    ui->UPVoigtCheckBox->setEnabled(enable);
    ui->VPVoigtCheckBox->setEnabled(enable);
    ui->WPVoigtCheckBox->setEnabled(enable);
    ui->eta0PVoigtCheckBox->setEnabled(enable);
    ui->eta1PVoigtCheckBox->setEnabled(enable);
    ui->eta2PVoigtCheckBox->setEnabled(enable);
    ui->asym1PVoigtCheckBox->setEnabled(enable);
    ui->asym2PVoigtCheckBox->setEnabled(enable);
    ui->asym3PVoigtCheckBox->setEnabled(enable);
    ui->asym4PVoigtCheckBox->setEnabled(enable);

    ui->UTCHCheckBox->setEnabled(enable);
    ui->VTCHCheckBox->setEnabled(enable);
    ui->WTCHCheckBox->setEnabled(enable);
    ui->ZTCHCheckBox->setEnabled(enable);
    ui->XTCHCheckBox->setEnabled(enable);
    ui->YTCHCheckBox->setEnabled(enable);
    ui->asym1TCHCheckBox->setEnabled(enable);
    ui->asym2TCHCheckBox->setEnabled(enable);
    ui->asym3TCHCheckBox->setEnabled(enable);
    ui->asym4TCHCheckBox->setEnabled(enable);
}

void ProfileFunctionWidget::setWidgetData(const crydata::crystal_t &cryst, c_profile_function prof[])
{
   //textChangedEnabled = false;
   functionTypeChanged = false;
   QHashIterator<QLineEdit *, bool> i(lineEditChanged);
   while (i.hasNext()) {
       i.next();
       lineEditChanged[i.key()] = false;
   }

   ui->scaleLineEdit->setText(QString::number(cryst.scal,'f',6));
   ui->scaleCheckBox->setChecked(cryst.scalcod);

   ui->functionTypeComboBox->setCurrentIndex(typeFunction);

   //Pearson VII
   ui->UPearsonLineEdit->setText(QString::number(prof[PEARSON7].par[0],'f',6));
   ui->UPearsonCheckBox->setChecked(prof[PEARSON7].rcod[0]);
   ui->VPearsonLineEdit->setText(QString::number(prof[PEARSON7].par[1],'f',6));
   ui->VPearsonCheckBox->setChecked(prof[PEARSON7].rcod[1]);
   ui->WPearsonLineEdit->setText(QString::number(prof[PEARSON7].par[2],'f',6));
   ui->WPearsonCheckBox->setChecked(prof[PEARSON7].rcod[2]);
   ui->beta0PearsonLineEdit->setText(QString::number(prof[PEARSON7].par[3],'f',6));
   ui->beta0PearsonCheckBox->setChecked(prof[PEARSON7].rcod[3]);
   ui->beta1PearsonLineEdit->setText(QString::number(prof[PEARSON7].par[4],'f',6));
   ui->beta1PearsonCheckBox->setChecked(prof[PEARSON7].rcod[4]);
   ui->beta2PearsonLineEdit->setText(QString::number(prof[PEARSON7].par[5],'f',6));
   ui->beta2PearsonCheckBox->setChecked(prof[PEARSON7].rcod[5]);
   ui->asym1PearsonLineEdit->setText(QString::number(prof[PEARSON7].par[6],'f',6));
   ui->asym1PearsonCheckBox->setChecked(prof[PEARSON7].rcod[6]);
   ui->asym2PearsonLineEdit->setText(QString::number(prof[PEARSON7].par[7],'f',6));
   ui->asym2PearsonCheckBox->setChecked(prof[PEARSON7].rcod[7]);
   ui->asym3PearsonLineEdit->setText(QString::number(prof[PEARSON7].par[8],'f',6));
   ui->asym3PearsonCheckBox->setChecked(prof[PEARSON7].rcod[8]);
   ui->asym4PearsonLineEdit->setText(QString::number(prof[PEARSON7].par[9],'f',6));
   ui->asym4PearsonCheckBox->setChecked(prof[PEARSON7].rcod[9]);

   //Pseudo-Voigt
   ui->UPVoigtLineEdit->setText(QString::number(prof[PVOIG].par[0],'f',6));
   ui->UPVoigtCheckBox->setChecked(prof[PVOIG].rcod[0]);
   ui->VPVoigtLineEdit->setText(QString::number(prof[PVOIG].par[1],'f',6));
   ui->VPVoigtCheckBox->setChecked(prof[PVOIG].rcod[1]);
   ui->WPVoigtLineEdit->setText(QString::number(prof[PVOIG].par[2],'f',6));
   ui->WPVoigtCheckBox->setChecked(prof[PVOIG].rcod[2]);
   ui->eta0PVoigtLineEdit->setText(QString::number(prof[PVOIG].par[3],'f',6));
   ui->eta0PVoigtCheckBox->setChecked(prof[PVOIG].rcod[3]);
   ui->eta1PVoigtLineEdit->setText(QString::number(prof[PVOIG].par[4],'f',6));
   ui->eta1PVoigtCheckBox->setChecked(prof[PVOIG].rcod[4]);
   ui->eta2PVoigtLineEdit->setText(QString::number(prof[PVOIG].par[5],'f',6));
   ui->eta2PVoigtCheckBox->setChecked(prof[PVOIG].rcod[5]);
   ui->asym1PVoigtLineEdit->setText(QString::number(prof[PVOIG].par[6],'f',6));
   ui->asym1PVoigtCheckBox->setChecked(prof[PVOIG].rcod[6]);
   ui->asym2PVoigtLineEdit->setText(QString::number(prof[PVOIG].par[7],'f',6));
   ui->asym2PVoigtCheckBox->setChecked(prof[PVOIG].rcod[7]);
   ui->asym3PVoigtLineEdit->setText(QString::number(prof[PVOIG].par[8],'f',6));
   ui->asym3PVoigtCheckBox->setChecked(prof[PVOIG].rcod[8]);
   ui->asym4PVoigtLineEdit->setText(QString::number(prof[PVOIG].par[9],'f',6));
   ui->asym4PVoigtCheckBox->setChecked(prof[PVOIG].rcod[9]);

   //TCH
   ui->UTCHLineEdit->setText(QString::number(prof[TCHZ].par[0],'f',6));
   ui->UTCHCheckBox->setChecked(prof[TCHZ].rcod[0]);
   ui->VTCHLineEdit->setText(QString::number(prof[TCHZ].par[1],'f',6));
   ui->VTCHCheckBox->setChecked(prof[TCHZ].rcod[1]);
   ui->WTCHLineEdit->setText(QString::number(prof[TCHZ].par[2],'f',6));
   ui->WTCHCheckBox->setChecked(prof[TCHZ].rcod[2]);
   ui->ZTCHLineEdit->setText(QString::number(prof[TCHZ].par[2],'f',6));
   ui->ZTCHCheckBox->setChecked(prof[TCHZ].rcod[2]);
   ui->XTCHLineEdit->setText(QString::number(prof[TCHZ].par[3],'f',6));
   ui->XTCHCheckBox->setChecked(prof[TCHZ].rcod[3]);
   ui->YTCHLineEdit->setText(QString::number(prof[TCHZ].par[4],'f',6));
   ui->YTCHCheckBox->setChecked(prof[TCHZ].rcod[4]);
   ui->asym1TCHLineEdit->setText(QString::number(prof[TCHZ].par[6],'f',6));
   ui->asym1TCHCheckBox->setChecked(prof[TCHZ].rcod[6]);
   ui->asym2TCHLineEdit->setText(QString::number(prof[TCHZ].par[7],'f',6));
   ui->asym2TCHCheckBox->setChecked(prof[TCHZ].rcod[7]);
   ui->asym3TCHLineEdit->setText(QString::number(prof[TCHZ].par[8],'f',6));
   ui->asym3TCHCheckBox->setChecked(prof[TCHZ].rcod[8]);
   ui->asym4TCHLineEdit->setText(QString::number(prof[TCHZ].par[9],'f',6));
   ui->asym4TCHCheckBox->setChecked(prof[TCHZ].rcod[9]);

   //textChangedEnabled = true;
}

void ProfileFunctionWidget::getWidgetData(crydata::crystal_t &cryst, c_profile_function prof[])
{
    cryst.scal = ui->scaleLineEdit->text().toFloat();
    cryst.scalcod = ui->scaleCheckBox->isChecked();

    typeFunction = ui->functionTypeComboBox->currentIndex();

    //Pearson VII
    prof[PEARSON7].par[0] = ui->UPearsonLineEdit->text().toFloat();
    prof[PEARSON7].rcod[0] = ui->UPearsonCheckBox->isChecked();
    prof[PEARSON7].par[1] = ui->VPearsonLineEdit->text().toFloat();
    prof[PEARSON7].rcod[1] = ui->VPearsonCheckBox->isChecked();
    prof[PEARSON7].par[2] = ui->WPearsonLineEdit->text().toFloat();
    prof[PEARSON7].rcod[2] = ui->WPearsonCheckBox->isChecked();
    prof[PEARSON7].par[3] = ui->beta0PearsonLineEdit->text().toFloat();
    prof[PEARSON7].rcod[3] = ui->beta0PearsonCheckBox->isChecked();
    prof[PEARSON7].par[4] = ui->beta1PearsonLineEdit->text().toFloat();
    prof[PEARSON7].rcod[4] = ui->beta1PearsonCheckBox->isChecked();
    prof[PEARSON7].par[5] = ui->beta2PearsonLineEdit->text().toFloat();
    prof[PEARSON7].rcod[5] = ui->beta2PearsonCheckBox->isChecked();
    prof[PEARSON7].par[6] = ui->asym1PearsonLineEdit->text().toFloat();
    prof[PEARSON7].rcod[6] = ui->asym1PearsonCheckBox->isChecked();
    prof[PEARSON7].par[7] = ui->asym2PearsonLineEdit->text().toFloat();
    prof[PEARSON7].rcod[7] = ui->asym2PearsonCheckBox->isChecked();
    prof[PEARSON7].par[8] = ui->asym3PearsonLineEdit->text().toFloat();
    prof[PEARSON7].rcod[8] = ui->asym3PearsonCheckBox->isChecked();
    prof[PEARSON7].par[9] = ui->asym4PearsonLineEdit->text().toFloat();
    prof[PEARSON7].rcod[9] = ui->asym4PearsonCheckBox->isChecked();

    //Pseudo-Voigt
    prof[PVOIG].par[0] = ui->UPVoigtLineEdit->text().toFloat();
    prof[PVOIG].rcod[0] = ui->UPVoigtCheckBox->isChecked();
    prof[PVOIG].par[1] = ui->VPVoigtLineEdit->text().toFloat();
    prof[PVOIG].rcod[1] = ui->VPVoigtCheckBox->isChecked();
    prof[PVOIG].par[2] = ui->WPVoigtLineEdit->text().toFloat();
    prof[PVOIG].rcod[2] = ui->WPVoigtCheckBox->isChecked();
    prof[PVOIG].par[3] = ui->eta0PVoigtLineEdit->text().toFloat();
    prof[PVOIG].rcod[3] = ui->eta0PVoigtCheckBox->isChecked();
    prof[PVOIG].par[4] = ui->eta1PVoigtLineEdit->text().toFloat();
    prof[PVOIG].rcod[4] = ui->eta1PVoigtCheckBox->isChecked();
    prof[PVOIG].par[5] = ui->eta2PVoigtLineEdit->text().toFloat();
    prof[PVOIG].rcod[5] = ui->eta2PVoigtCheckBox->isChecked();
    prof[PVOIG].par[6] = ui->asym1PVoigtLineEdit->text().toFloat();
    prof[PVOIG].rcod[6] = ui->asym1PVoigtCheckBox->isChecked();
    prof[PVOIG].par[7] = ui->asym2PVoigtLineEdit->text().toFloat();
    prof[PVOIG].rcod[7] = ui->asym2PVoigtCheckBox->isChecked();
    prof[PVOIG].par[8] = ui->asym3PVoigtLineEdit->text().toFloat();
    prof[PVOIG].rcod[8] = ui->asym3PVoigtCheckBox->isChecked();
    prof[PVOIG].par[9] = ui->asym4PVoigtLineEdit->text().toFloat();
    prof[PVOIG].rcod[9] = ui->asym4PVoigtCheckBox->isChecked();

    //TCH
    prof[TCHZ].par[0] = ui->UTCHLineEdit->text().toFloat();
    prof[TCHZ].rcod[0] = ui->UTCHCheckBox->isChecked();
    prof[TCHZ].par[1] = ui->VTCHLineEdit->text().toFloat();
    prof[TCHZ].rcod[1] = ui->VTCHCheckBox->isChecked();
    prof[TCHZ].par[2] = ui->WTCHLineEdit->text().toFloat();
    prof[TCHZ].rcod[2] = ui->WTCHCheckBox->isChecked();
    prof[TCHZ].par[2] = ui->ZTCHLineEdit->text().toFloat();
    prof[TCHZ].rcod[2] = ui->ZTCHCheckBox->isChecked();
    prof[TCHZ].par[3] = ui->XTCHLineEdit->text().toFloat();
    prof[TCHZ].rcod[3] = ui->XTCHCheckBox->isChecked();
    prof[TCHZ].par[4] = ui->YTCHLineEdit->text().toFloat();
    prof[TCHZ].rcod[4] = ui->YTCHCheckBox->isChecked();
    prof[TCHZ].par[6] = ui->asym1TCHLineEdit->text().toFloat();
    prof[TCHZ].rcod[6] = ui->asym1TCHCheckBox->isChecked();
    prof[TCHZ].par[7] = ui->asym2TCHLineEdit->text().toFloat();
    prof[TCHZ].rcod[7] = ui->asym2TCHCheckBox->isChecked();
    prof[TCHZ].par[8] = ui->asym3TCHLineEdit->text().toFloat();
    prof[TCHZ].rcod[8] = ui->asym3TCHCheckBox->isChecked();
    prof[TCHZ].par[9] = ui->asym4TCHLineEdit->text().toFloat();
    prof[TCHZ].rcod[9] = ui->asym4TCHCheckBox->isChecked();
}

void ProfileFunctionWidget::connectLineEdit(QLineEdit *lineEdit)
{
    lineEditChanged[lineEdit] = false;
    connect(lineEdit, &QLineEdit::textEdited, this, [this](const QString &text) {
        //if (!textChangedEnabled) return;
        QLineEdit *ledit = qobject_cast<QLineEdit *>(sender());
        lineEditChanged[ledit] = true;
        qInfo() << "Value changed: " << text << lineEditChanged[ledit] << lineEditChanged.count();
    });
}

bool ProfileFunctionWidget::hasFunctionValueChanged(int typefun)
{
    if (lineEditChanged[ui->scaleLineEdit]) return true;

    switch (typefun) {
    case PEARSON7:
        if (lineEditChanged[ui->UPearsonLineEdit]) return true;
        if (lineEditChanged[ui->VPearsonLineEdit]) return true;
        if (lineEditChanged[ui->WPearsonLineEdit]) return true;
        if (lineEditChanged[ui->beta0PearsonLineEdit]) return true;
        if (lineEditChanged[ui->beta1PearsonLineEdit]) return true;
        if (lineEditChanged[ui->beta2PearsonLineEdit]) return true;
        if (lineEditChanged[ui->asym1PearsonLineEdit]) return true;
        if (lineEditChanged[ui->asym2PearsonLineEdit]) return true;
        if (lineEditChanged[ui->asym3PearsonLineEdit]) return true;
        if (lineEditChanged[ui->asym4PearsonLineEdit]) return true;
        break;

    case PVOIG:
        if (lineEditChanged[ui->UPVoigtLineEdit]) return true;
        if (lineEditChanged[ui->VPVoigtLineEdit]) return true;
        if (lineEditChanged[ui->WPVoigtLineEdit]) return true;
        if (lineEditChanged[ui->eta0PVoigtLineEdit]) return true;
        if (lineEditChanged[ui->eta1PVoigtLineEdit]) return true;
        if (lineEditChanged[ui->eta2PVoigtLineEdit]) return true;
        if (lineEditChanged[ui->asym1PVoigtLineEdit]) return true;
        if (lineEditChanged[ui->asym2PVoigtLineEdit]) return true;
        if (lineEditChanged[ui->asym3PVoigtLineEdit]) return true;
        if (lineEditChanged[ui->asym4PVoigtLineEdit]) return true;
        break;

    case TCHZ:
        if(lineEditChanged[ui->UTCHLineEdit]) return true;
        if(lineEditChanged[ui->VTCHLineEdit]) return true;
        if(lineEditChanged[ui->WTCHLineEdit]) return true;
        if(lineEditChanged[ui->ZTCHLineEdit]) return true;
        if(lineEditChanged[ui->XTCHLineEdit]) return true;
        if(lineEditChanged[ui->YTCHLineEdit]) return true;
        if(lineEditChanged[ui->asym1TCHLineEdit]) return true;
        if(lineEditChanged[ui->asym2TCHLineEdit]) return true;
        if(lineEditChanged[ui->asym3TCHLineEdit]) return true;
        if(lineEditChanged[ui->asym4TCHLineEdit]) return true;
        break;
    }
    return false;
}

void ProfileFunctionWidget::on_functionTypeComboBox_currentIndexChanged(int index)
{
    ui->functionStackedWidget->setCurrentIndex(index);
    functionTypeChanged = typeFunction != index;
    qInfo() << "Function Changed: " << functionTypeChanged;
}
