#include "propertiesdialog.h"
#include "ui_propertiesdialog.h"
#include "properties.h"
#include "aligndelegate.h"
#include "itemdelegate.h"
#include "qt_utils.h"
#include <QStringList>

PropertiesDialog::PropertiesDialog(QWidget *parent) :
    QDialog(parent),
    ModelEdit(nullptr),
    ui(new Ui::PropertiesDialog),
    mModel(nullptr),
    kmodel(0),
    mAtom(-1),
    mBond(-1),
    mAngle(-1),
    mTors(-1),
    mAddDelete(false)
{
    ui->setupUi(this);
}

PropertiesDialog::~PropertiesDialog()
{
    delete ui;
}

void PropertiesDialog::showEvent(QShowEvent *event)
{
    ui->addButton->setVisible(mTipo == 0);
    ui->deleteButton->setVisible(mTipo == 0);
    ui->deleteButton->setEnabled(false);
    if(mModel)
    {
       mModel->clear();
       delete mModel;
    }
    mModel = createAndPopulateModel();
    int larg = 0;
    for(int i=0; i<mModel->columnCount();i++)
        if(!ui->tableView->isColumnHidden(i))
           larg += ui->tableView->columnWidth(i);
    int h = size().height();
    this->resize(larg+50, h);
    QItemSelectionModel *selectionModel = ui->tableView->selectionModel();
    connect(selectionModel, &QItemSelectionModel::currentRowChanged,
           this, &PropertiesDialog::rowSelectionChanged);
    ui->tableView->selectRow(0);
    QDialog::showEvent(event);
}

void PropertiesDialog::rowSelectionChanged(const QModelIndex &current, const QModelIndex &)
{
    QModelIndex index;
    if(mTipo == 0)
        index = proxyModel->mapToSource(proxyModel->index(current.row(), 0, QModelIndex()));
    else
        index= mModel->index(current.row(), 0, QModelIndex());
    int row = mModel->data(index, Qt::DisplayRole).toInt() - 1;
    if(row < 0)
        return;
    switch(mTipo)
    {
    case 0:
        ui->deleteButton->setEnabled(true);
        mAtom = row;
        emit selectedAtom(row);
        break;
    case 1:
        mBond = row;
        emit selectedBond(row);
        break;
    case 2:
        mAngle = row;
        emit selectedAngle(row);
        break;
    case 3:
        mTors = row;
        emit selectedTorsion(row);
        break;
    }
}

QStandardItemModel *PropertiesDialog::createAndPopulateModel()
{
    QStandardItemModel *model;
    QList<QStandardItem *> *lista;
    QStringList headers;
    int nAtoms = 0, nElems = 0;
    const char *tipi = "iasffffffau";

    switch(mTipo)
    {
    default:
    case 0:
    {
        proxyModel = new MyProxyModel(this);
        headers = QStringList({"Number", "Label", "Type", "H", "Xfrac", "Yfrac", "Zfrac", "B", "Occ.", "Symm.op.","z"});
        int n = headers.count();
        proxyModel->setColumnForSort(n-1);
        proxyModel->setColumnTypes(n, tipi);
        lista = ModelEdit->FillListAtoms(true, true, &nAtoms);
        nElems = nAtoms;
    }
        break;
    case 1:
        headers = QStringList({"Number", "Type", "Atom1", "Atom2", "Distance"});
        lista = ModelEdit->FillListBonds(true);
        nElems = ModelEdit->nconn_prot;
        break;
    case 2:
        headers = QStringList({"Number", "Type", "Start Atom", "Vertex", "End Atom", "Angle"});
        lista = ModelEdit->FillListAngles(true);
        nElems = ModelEdit->nangbonds;
        break;
    case 3:
        headers = QStringList({"Number", "Type", "Atom1", "Atom2", "Atom3", "Atom4", "Angle"});
        lista = ModelEdit->FillListTorsions(true);
        nElems = ModelEdit->ntorsions;
        break;
    }
    QStyledItemDelegate *defaultDelegate = new QStyledItemDelegate(ui->tableView);
    model = new QStandardItemModel(this);
    model->setHorizontalHeaderLabels(headers);
    for(int i=0; i<model->columnCount(); i++)
        ui->tableView->setItemDelegateForColumn(i, defaultDelegate);
    for(int i=0; i<nElems;i++)
        model->appendRow(lista[i]);
    if(mTipo == 0)
    {
        proxyModel->setSourceModel(model);
        ui->tableView->setModel(proxyModel);
        for(int i=1; i<headers.count(); i++)
        {
            if(tipi[i] == 's')
            {
                QStringList items;
                for (int k = 0; k < Properties::NSpecies; k++)
                {
                     QString elem = get_nth_element(Properties::ListaSpecies[k]-1);
                     items.append(elem);
                }
                ComboDelegate *cDelegate = new ComboDelegate(ui->tableView);
                cDelegate->setItems(items);
                ui->tableView->setItemDelegateForColumn(i, cDelegate);
            }
            else if(tipi[i] == 'f')
            {
                AlignFloatDelegate *delegate = new AlignFloatDelegate(ui->tableView);
                if(i == (headers.count()-3))    // occ.
                    delegate->setDecimals(2);
                else
                    delegate->setDecimals(5);
                if((i >=4) && (i <=6))
                    delegate->setRange(-1.f, 1.f);
                else
                    delegate->setRange(1e-5, 1000000.f);
                ui->tableView->setItemDelegateForColumn(i, delegate);
            }
        }
        ui->tableView->setColumnHidden(3, true);      // H
        ui->tableView->setColumnHidden(9, true);      // Symm.op
        ui->tableView->setColumnHidden(headers.count()-1, true); // z
    }
    else
    {
        ui->tableView->setModel(model);
        AlignFloatDelegate *delegate = new AlignFloatDelegate(ui->tableView);
        delegate->setDecimals(5);
        if(mTipo == 1)
           delegate->setRange(1e-5, 1000000.f);
        else
        {
           delegate->setDecimals(2);
           delegate->setRange(-1000000.f, 1000000.f);
        }
        ui->tableView->setItemDelegateForColumn(headers.count()-1, delegate);
    }
    AlignFloatDelegate *delegate0 = new AlignFloatDelegate(ui->tableView);
    delegate0->setDecimals(0);
    delegate0->setRange(1, 1000000);
    ui->tableView->setItemDelegateForColumn(0, delegate0);
    ui->tableView->setSortingEnabled(true);
    ui->tableView->sortByColumn(0, Qt::AscendingOrder);
    ui->tableView->horizontalHeader()->setStretchLastSection(false);
    ui->tableView->resizeColumnsToContents();
    ui->tableView->horizontalHeader()->setStretchLastSection(true);
    ui->tableView->setSortingEnabled(true);
    ui->tableView->sortByColumn(0, Qt::AscendingOrder);
    connect(model,SIGNAL(dataChanged(QModelIndex,QModelIndex)),
            this, SLOT(tableView_dataChanged(QModelIndex,QModelIndex)));
    return model;
}

void PropertiesDialog::tableView_dataChanged(const QModelIndex &,
                                             const QModelIndex &)
{
     if(!mAddDelete)
        applyEdit();
}

void PropertiesDialog::on_addButton_clicked()
{
    addRow();
    int lastRow = mModel->rowCount()-1;
    QModelIndex index = proxyModel->mapFromSource(mModel->index(lastRow, 1, QModelIndex()));
    ui->tableView->selectRow(index.row());
    ui->tableView->edit(index);
}

void PropertiesDialog::reorderItems(int rowDel)
{
    int nRows = mModel->rowCount();
    for(int i=0; i<nRows; i++)
    {
        QModelIndex index = mModel->index(i, 0, QModelIndex());
        int num = mModel->data(index, Qt::DisplayRole).toInt();
        if(num > rowDel)
        {
            num--;
            mModel->setData(index, num, Qt::DisplayRole);
        }
    }
}

void PropertiesDialog::on_deleteButton_clicked()
{
    if(mAtom != -1)
    {
        mAddDelete = true;
        int katom = -mAtom-1;
        int rowDel = mAtom;
        mModel->removeRow(rowDel);
        reorderItems(rowDel);
        applyEdit(&katom);
        mAddDelete = false;
    }
}

void PropertiesDialog::on_okButton_clicked()
{
    int code = 1;
    int km = kmodel;

    switch(mTipo)
    {
    case 0:
    {
        int natoms = 0;
        atom_type *atml = FromListToAtom(&natoms);
        jav_edit_structure_(&code, atml, &natoms,&km);
    }
        break;
    case 1:
    {
        f_bond_type *legm = FromListToLegm();
        int selbond = mBond+1;
        jav_edit_bond_properties_(&code, legm, &km, &selbond);
    }
        break;
    case 2:
    {
        int angolo = 1;
        jav_anglebond_type *angs = FromListToAngs();
        jav_edit_angle_properties_(&code, angs, &km, &angolo);
    }
        break;
    case 3:
    {
        jav_torsion_type angs = ModelEdit->torsions[0];
        float tval = 0;
        jav_edit_torsion_properties_(&code, &angs, &tval, &km);
    }
        break;
    }

    emit propertiesDialogClosed();
    accept();
}

void PropertiesDialog::applyEdit(int *kat)
{
    int code = 3;
    if(kat != nullptr)
        code = *kat;
    int km = kmodel;

    switch(mTipo)
    {
    case 0:
    {
        int natoms = 0;
        atom_type *atml = FromListToAtom(&natoms);
        jav_edit_structure_(&code, atml, &natoms,&km);
    }
        break;
    case 1:
    {
        f_bond_type *legm = FromListToLegm();
        int selbond = mBond+1;
        jav_edit_bond_properties_(&code, legm, &km, &selbond);
    }
        break;
    case 2:
    {
        jav_anglebond_type *angs = FromListToAngs();
        int selang = mAngle+1;
        jav_edit_angle_properties_(&code, angs, &km, &selang);
    }
        break;
    case 3:
    {
        int ntors = mModel->rowCount();
        QList<QStandardItem *>  *lista = new QList<QStandardItem *> [ntors]();
        for(int i=0; i<ntors; i++)
        {
            QStandardItem *item = mModel->item(i,0);
            lista[i].append(item);
            item = mModel->item(i,6);
            lista[i].append(item);
        }
        jav_torsion_type *angs = ModelEdit->FromListToTors(lista, &ntors);
        jav_torsion_type ang = angs[mTors];
        float tval = ang.fv;
        //FIX: Chiedere a Rosi se è possibile recuperare il valore iniziale di tval (ricalcolato in fortran). In caso contrario rimuovi tval
        jav_edit_torsion_properties_(&code, &ang, &tval, &km);
    }
        break;
    }
}

void PropertiesDialog::on_cancelButton_clicked()
{
    int code = 2;
    int km = kmodel;

    switch(mTipo)
    {
    case 0:
    {
        atom_type *atml = ModelEdit->FromJavatomToAtom();
        int natoms = ModelEdit->nats_au;
        jav_edit_structure_(&code, atml, &natoms,&km);
    }
        break;
    case 1:
    {
        f_bond_type *legm = ModelEdit->FromJavbondToLegm();
        int selbond = 1;
        jav_edit_bond_properties_(&code, legm, &km, &selbond);
    }
        break;
    case 2:
    {
        int angolo = 1;
        jav_anglebond_type *angs = ModelEdit->anglebonds;
        jav_edit_angle_properties_(&code, angs, &km, &angolo);
    }
        break;
    case 3:
    {
        jav_torsion_type angs = ModelEdit->torsions[0];
        float tval = 0;
        jav_edit_torsion_properties_(&code, &angs, &tval, &km);
    }
        break;
    }

    emit propertiesDialogClosed();
    reject();
}

jav_anglebond_type *PropertiesDialog::FromListToAngs()
{
    int nangs = mModel->rowCount();
    QList<QStandardItem *>  *lista = new QList<QStandardItem *> [nangs]();
    for(int i=0; i<nangs; i++)
    {
        QStandardItem *item = mModel->item(i,0);
        lista[i].append(item);
        item = mModel->item(i,5);
        lista[i].append(item);
    }
    jav_anglebond_type *angs = ModelEdit->FromListToAngs(lista, &nangs);
    return angs;
}

f_bond_type *PropertiesDialog::FromListToLegm()
{
    int nbonds = mModel->rowCount();
    QList<QStandardItem *>  *lista = new QList<QStandardItem *> [nbonds]();
    for(int i=0; i<nbonds; i++)
    {
        QStandardItem *item = mModel->item(i,0);
        lista[i].append(item);
        item = mModel->item(i,4);
        lista[i].append(item);
    }
    f_bond_type *legm = ModelEdit->FromListToLegm(lista, &nbonds);
    return legm;
}

atom_type *PropertiesDialog::FromListToAtom(int *natoms)
{
    *natoms = mModel->rowCount();
    QList<QStandardItem *>  *lista = new QList<QStandardItem *> [*natoms]();
    for(int i=0; i<*natoms; i++)
    {
        int ncol = mModel->columnCount();
        for (int k=0; k<ncol; k++)
        {
            QStandardItem *item = mModel->item(i, k);
            lista[i].append(item);
        }
    }
    atom_type *atml = ModelEdit->FromListToAtom(lista, natoms);
    return atml;
}

void PropertiesDialog::addRow()
{
    QList<QStandardItem *> lista;
    int ieq = mModel->rowCount()+1;
    QStandardItem *item = new QStandardItem();
    item->setData(ieq, Qt::DisplayRole);
    item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
    lista.append(item);
    item = new QStandardItem(" ");
    lista.append(item);
    item = new QStandardItem(" ");
    lista.append(item);
    item = new QStandardItem();
    item->setData(0,  Qt::DisplayRole);
    lista.append(item);
    for(int k=0; k<3;k++)
    {
        item = new QStandardItem();
        item->setData(0,  Qt::DisplayRole);
        lista.append(item);
    }
    item = new QStandardItem();
    item->setData(1,  Qt::DisplayRole);
    lista.append(item);
    item = new QStandardItem();
    item->setData(1,  Qt::DisplayRole);
    lista.append(item);
    item = new QStandardItem(" ");
    lista.append(item);
    item = new QStandardItem();
    item->setData(0,  Qt::DisplayRole);
    lista.append(item);
    mModel->appendRow(lista);
}
