#include "spacegdialog.h"
#include "ui_spacegdialog.h"

#include <QPushButton>

SpacegDialog::SpacegDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::SpacegDialog)
{
    ui->setupUi(this);       

    connect(ui->symmetryWidget, &SymmetryWidget::spaceWidgetChanged, this, [=]() {
        ui->contentWidget->updateInfo(ui->symmetryWidget->cellVolume());
        enableOkButton();
    });

    connect(ui->symmetryWidget, &SymmetryWidget::cellWidgetChanged, this, [=](){
        ui->contentWidget->updateInfo(ui->symmetryWidget->cellVolume());
        enableOkButton();
    });

    connect(ui->contentWidget, &FormulaWidget::formulaEdited, this, &SpacegDialog::enableOkButton);
}

SpacegDialog::~SpacegDialog()
{
    delete ui;
}

void SpacegDialog::setData(double cell[6], int sgIndex, const QString &spg, const QString &content, int findspace)
{
    ui->symmetryWidget->setData(cell, sgIndex, spg);
    ui->contentWidget->updateFormula(content,ui->symmetryWidget->cellVolume());
    ui->findSgCheckBox->setChecked(findspace);
    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(ui->contentWidget->isValidFormula());
}

QString SpacegDialog::getContent() const
{
    return ui->contentWidget->formula();
}

int SpacegDialog::getSpaceGroup(QString &spg) const
{    
    return ui->symmetryWidget->getSpaceGroup(spg);
}

void SpacegDialog::getCellParameters(double cell[6]) const
{    
    ui->symmetryWidget->getCellParameters(cell);
}

int SpacegDialog::getFindSpace() const
{
    return ui->findSgCheckBox->isChecked();
}

void SpacegDialog::accept()
{
#ifndef Q_OS_MACOS
    //'ok' get focus if you hide the dialog. This avoid that enter key close the dialog
    if (!ui->buttonBox->button(QDialogButtonBox::Ok)->hasFocus()) return;
#endif

    QDialog::accept();
}

void SpacegDialog::on_findSgCheckBox_toggled(bool checked)
{
    ui->symmetryWidget->enableSpacegWidget(!checked);
}

void SpacegDialog::enableOkButton()
{
    bool enable;
    if (ui->findSgCheckBox->isChecked()) {
        enable = ui->contentWidget->isValidFormula() &&
                      ui->symmetryWidget->isValidCell();
    } else {
        enable = ui->contentWidget->isValidFormula() &&
                      ui->symmetryWidget->isValidSpaceGroup() &&
                      ui->symmetryWidget->isValidCell();
    }

    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(enable);
}
