#ifndef SPACEGROUPLISTDIALOG_H
#define SPACEGROUPLISTDIALOG_H

#include "absrefdialog.h"

#include <QDialog>
#include <QAbstractButton>
#include <QStandardItemModel>
#include <QSortFilterProxyModel>

typedef struct  {
     int          num;        // Space group number
     char         symb[12];   // Space group symbol
     char         exts[21];   // Extinction symbol
     float        fig;        // Figure of merit
     int          nabs;       // Number of absent reflections
     int          nasym;      // Atoms in the asymmetric unit
     int          freq_no;    // No. in CSD
     float        freq_perc;  // % in CSD
     int          freq_rank;  // Rank in CSD
     int          chiral;     // Chiral space group?
                } spgext_info_type;

namespace Ui {
class SpaceGroupListDialog;
}

class SpaceGroupListDialog : public QDialog
{
    Q_OBJECT

public:
    explicit SpaceGroupListDialog(QWidget *parent = nullptr);
    ~SpaceGroupListDialog();
    void createModel(spgext_info_type spaceList[], int listSize, int selectedSpace);
    void waitSignalIsEmitted();
    int selectedSpaceGroup() const;

private slots:
    void onSpaceGroupListDialogFinished(int);
    void onButtonBoxClicked(QAbstractButton *button);

private:
    Ui::SpaceGroupListDialog *ui;
    QStandardItemModel *spaceListModel;
    QSortFilterProxyModel *proxyModel;
    AbsRefDialog *absRefDialog;
};

#endif // SPACEGROUPLISTDIALOG_H
