#include "symmetrywidget.h"
#include "ui_symmetrywidget.h"

SymmetryWidget::SymmetryWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::SymmetryWidget)
{
    ui->setupUi(this);

    ui->cellWidget->hideRefineCheckBox();

    connect(ui->spacegWidget, &SpaceGroupWidget::spaceGroupChanged, this, [=](const QString &spg){        
        ui->cellWidget->updateCell(spg);
        emit spaceWidgetChanged(spg);
    });

    connect(ui->cellWidget, &CellParametersWidget::cellParametersModified, this, &SymmetryWidget::cellWidgetChanged);
}

SymmetryWidget::~SymmetryWidget()
{
    delete ui;
}

void SymmetryWidget::setData(double cell[], int sgIndex, const QString &spg)
{
    ui->cellWidget->setCell(cell, spg);
    ui->spacegWidget->setSpaceGroup(sgIndex, spg);
}

void SymmetryWidget::enableSpacegWidget(bool enable)
{
    ui->spacegWidget->setEnabled(enable);
}

double SymmetryWidget::cellVolume() const
{
    return ui->cellWidget->cellVolume();
}

int SymmetryWidget::getSpaceGroup(QString &spg) const
{
    return ui->spacegWidget->getSpaceGroup(spg);
}

void SymmetryWidget::getCellParameters(double cell[]) const
{
    ui->cellWidget->getCellParameters(cell);
}

bool SymmetryWidget::isValidSpaceGroup() const
{
    return ui->spacegWidget->isValidSpaceGroup();
}

bool SymmetryWidget::isValidCell() const
{
    return !ui->cellWidget->anyIsEmpty();
}
