#include <QFile>
#include <QWidgetAction>
#include <QSettings>
#include <QLabel>
#include <QTextStream>
#include <QFontDialog>
#include <QFileDialog>
#include <QMessageBox>
#include "texteditor.h"
#include "qt_utils.h"
#include "ui_texteditor.h"
#include "savedialog.h"
#include "javkeysettings.h"

TextEditor::TextEditor(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::TextEditor)
{
    ui->setupUi(this);
    mSaveAs = true;
    mFilename = " ";

    connect(ui->actionExit,SIGNAL(pressed()),this,SLOT(on_actionExit_triggered()));
    connect(ui->actionFont ,SIGNAL(pressed()),this,SLOT(on_actionFont_triggered()));
    connect(ui->actionPrint,SIGNAL(pressed()),this,SLOT(on_actionPrint_triggered()));
    connect(ui->actionSave, SIGNAL(pressed()), this, SLOT(on_actionSave_triggered()));
    connect(ui->actionFind, SIGNAL(returnPressed()), this,  SLOT(on_actionFind()));
#if !defined(QT_PRINTSUPPORT_LIB) || !QT_CONFIG(printer)
    ui->actionPrint->setEnabled(false);
#endif

}

TextEditor::~TextEditor()
{
    delete ui;
}

QTextEdit *TextEditor::getTextEdit()
{
    return ui->text;
}

void TextEditor::SetEditable(bool editable)
{
    ui->text->setReadOnly(!editable);
}

void TextEditor::on_actionFind()
{
    QString text = ui->actionFind->text();
    ui->statusBar->setText("");
    if(!ui->text->find(text))
    {
        QTextCursor cursor = ui->text->textCursor();
        cursor.setPosition(0, QTextCursor::MoveAnchor);
        ui->text->setTextCursor(cursor);
        ui->statusBar->setText("search hit bottom, continuing at top");
        ui->text->find(text);
    }
}

void TextEditor::on_actionExit_triggered()
{
    if(ui->text->document()->isModified())
    {
        QString titolo = qApp->applicationName();
        QString testo = "Save changes to file " + mFilename + " before closing?";
        if(QMessageBox::Yes == QMessageBox::question(this, titolo, testo,
                         QMessageBox::Yes | QMessageBox::No))
        {
           //save the file
            mSaveAs = false;
            on_actionSave_triggered();
        }
    }
    this->close();
}

void TextEditor::on_actionSave_triggered()
{
    QSettings settings;
    QString selectedFilter = "Text file (*.txt);; All Files (*.*)";
    QString fileName;
    if(mSaveAs)
        fileName = SaveDialog::run(this, tr("Export as"),
                                        settings.value(DEFAULT_DIR_KEY).toString(),
                                        "", "Text file", "txt", selectedFilter);
    else
        fileName = mFilename;
    QFile file(fileName);
    if (!file.open(QFile::WriteOnly | QFile::Text)) {
        QMessageBox::warning(this, "Warning", "Cannot save file: " + file.errorString());
        return;
    }
    QTextStream ts(&file);
    QString text = ui->text->toPlainText();
    ts << text;
    file.close();
    ui->text->document()->setModified(false);
}

void TextEditor::on_actionPrint_triggered()
{
#if defined(QT_PRINTSUPPORT_LIB) && QT_CONFIG(printer)
    QPrinter printDev;
#if QT_CONFIG(printdialog)
    QPrintDialog dialog(&printDev, this);
    if (dialog.exec() == QDialog::Rejected)
        return;
#endif // QT_CONFIG(printdialog)
    ui->text->print(&printDev);
#endif // QT_CONFIG(printer)}
}

void TextEditor::on_actionFont_triggered()
{
    bool fontSelected;
    QFont font = QFontDialog::getFont(&fontSelected, this);
    if (fontSelected)
        ui->text->setFont(font);

}

void TextEditor::LoadFile(QString filename)
{
    QFile file( filename ); // Read the text from a file
    if ( file.open(QFile::ReadOnly) ) {
        QTextStream ts( &file );
        ui->text->setText( ts.readAll() );
        setWindowTitle(filename);
        mFilename = filename;
    }    file.close();
}

void TextEditor::LoadFromString(QString &str)
{
    ui->text->setText( str );    
}
