﻿#include <cstring>
#include <sstream>
#include <iostream>
#include "libmath.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif
#include "models.h"
#include "properties.h"
#include "qt_utils.h"

#define color_helix NUM_COLORS-6
#define color_sheet NUM_COLORS-5
#define color_turn  NUM_COLORS-4
#define color_coils NUM_COLORS-3

// #if defined(JAV_WIDGET)
// enum codici {IFCELL, IFSYMM, PLOTSTYLE, IFLABELS, SYMLEVEL, IFPOLYHEDRA,
//               IFGROW, IFHBONDS, IFSHORTC, IFDELHANG};
// #endif

typedef struct {
                 int vertex;
                 int faces[4];
               } _VertVett;

extern STL_class stl_file;
//extern "C" void  getsecondary_(char sec[], int *n, size_t dum);

// utilities

float side(float val, float lim1, float lim2)
{
   return((val-lim1)/(lim2-lim1));
}

int sidepp(int val, int nbin)
{
   int result = val;
   while(result < 0)
         result += nbin;
   while(result >= nbin)
         result -= nbin;
   return result;
}

void CalcolaH(int n, float Hdit[4][NKNOTS-2])
{
   float t;
   float d = 1.f/(float)(n-1);
   for(int i=1; i<n/2; i++)
   {
       t = (float) i * d;
       Hdit[0][i-1] = (1 + 2*t) * (1 - t) * (1 - t);
       Hdit[1][i-1] = t * (1 - t) * (1 - t);
       Hdit[2][i-1] = t * t * (3 - 2*t);
       Hdit[3][i-1] = t * t * (t - 1);
       t = (float) (i + n/2) * d;
       Hdit[0][i+n/2-1] = (1 + 2*t) * (1 - t) * (1 - t);
       Hdit[1][i+n/2-1] = t * (1 - t) * (1 - t);
       Hdit[2][i+n/2-1] = t * t * (3 - 2*t);
       Hdit[3][i+n/2-1] = t * t * (t - 1);
   }
   t = 0.5;
   Hdit[0][n/2-1] = (1 + 2*t) * (1 - t) * (1 - t);
   Hdit[1][n/2-1] = t * (1 - t) * (1 - t);
   Hdit[2][n/2-1] = t * t * (3 - 2*t);
   Hdit[3][n/2-1] = t * t * (t - 1);
}

void CalcolaNormale(vet3 a, vet3 b, vet3 c, vet3 *vn)
{
   vet3 v1 = MRVectorSub(b, a);
   vet3 v2 = MRVectorSub(c, b);
   vet3 v3 = MRVectorCrossProduct(v1, v2);
   v3 = MRVectorNormalize(v3);
   MRVectorCopy(v3, vn);
}

int NumberOfFaces(polyhedron_type Poly)
{
  int nf = 8;
  switch(Poly.nvert)
  {
  case 4:
       nf = 4;
       break;
  case 5:
       nf = 5;
       break;
  case 6:
  default:
       nf = 8;
       break;
  }
  return nf;
}

static void GetSize(Knot *knot)
{
    if( !knot->vsize )
        knot->vsize = sqrt( knot->vnx*knot->vnx +
                            knot->vny*knot->vny +
                            knot->vnz*knot->vnz );

    if( !knot->hsize )
        knot->hsize = sqrt( knot->vnx*knot->hnx +
                            knot->vny*knot->hny +
                            knot->vnz*knot->hnz );
}

void drawSphere(float xc, float yc, float zc, float rad, float color1[], QOpenGLFunctions *f)
{
    f->glDisable(GL_LIGHTING);
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    f->glEnable(GL_COLOR_MATERIAL);
    f->glEnable (GL_BLEND);
    glShadeModel(GL_SMOOTH);
    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;
    glColor4fv(color1);

    glPushMatrix();
    glTranslated(xc, yc, zc);
    int lats=10, longs=10;
    for(int i = 0; i <= lats; i++)
    {
        double lat0 = M_PI * (-0.5 + (double) (i - 1) / lats);
        double z0  =  rad * sin(lat0);
        double zr0 =  rad * cos(lat0);

        double lat1 = M_PI * (-0.5 + (double) i / lats);
        double z1 =  rad * sin(lat1);
        double zr1 = rad * cos(lat1);

        glBegin(GL_QUAD_STRIP);
        for(int j = 0; j <= longs; j++)
        {
            double lng = 2 * M_PI * (double) (j - 1) / longs;
            double x = cos(lng);
            double y = sin(lng);
            glNormal3d(x * zr0, y * zr0, z0);
            glVertex3d(x * zr0, y * zr0, z0);
            glNormal3d(x * zr1, y * zr1, z1);
            glVertex3d(x * zr1, y * zr1, z1);
        }
        glEnd();
    }
    glPopMatrix();
}

Model* Model::Clone() const
{
  return new Model(*this);
}

Model::Model()
{
      int status[] = {1, 0, 1, 0, 0, 0, 0, -1, 1, 0, 0, 0, 0,
                      1, 1, 1, 1};
      float fstatus[] = {0.1f, 0.1f, 0.1f};

      if(Properties::defaultPlotStyle >= 0)
         status[3] = Properties::defaultPlotStyle;
      if(Properties::defaultSymmetry >= 0)
         status[1] = Properties::defaultSymmetry;
      if(Properties::defaultLabels >= 0)
         status[5] = Properties::defaultLabels;
      if(Properties::defaultColor >= 0)
         status[4] = Properties::defaultColor;

      jav = getJav();

      created = false;
      setValues(status, fstatus);
      nknots = NKNOTS;
      CalcolaH(nknots, H);
      ifSurface = false;
      Surface_c = false;
      VWSurface = Mappa();
}

void Model::SetModel(int status[], float fstatus[], statusSav vec)
{
      setValues(status, fstatus);

      for(int i=0; i<NUM_COLORS; i++)
          light_diffuse[i] = vec.light_diffuse[i];
      labelcolor = vec.labelcolor;
      light_chain = vec.light_chain;
      memcpy(LabelS, vec.LabelS, N_ELEMENTS*sizeof(int));

      created = true;
}

void Model::freeStructs()
{
  if(!Chains.empty())
  {
        for(int i=0; i<(int) Chains.size(); i++)
        {
           if(!Chains[i].Residues.empty())
           {
              for(int j=0; j<(int) Chains[i].Residues.size(); j++)
                  Chains[i].Residues[j].atoms.clear();
              Chains[i].Residues.clear();
            }
           if(!Chains[i].AminoRes.empty())
           {
              for(int j=0; j<(int) Chains[i].AminoRes.size(); j++)
                  Chains[i].AminoRes[j].atoms.clear();
              Chains[i].AminoRes.clear();
           }
        }
       Chains.clear();
  }
  if(!Conns.empty())
         for(int i=0; i<(int) Conns.size(); i++)
            Conns[i].clear();
  c_alpha.clear();
  c_alphab.clear();
  nChains = 0;
  atoms_sel.clear();
  bonds_sel.clear();
  resid_sel.clear();
  phlist.clear();
  atom_types.clear();
  AtomTypes.clear();
  Conns.clear();
}

void Model::freeVectors()
{
    if(!Equivs.empty())
       for(int i=0; i<(int) Equivs.size(); i++)
           Equivs[i].clear();
    Equivs.clear();
    if(atm != NULL)
          delete[] atm;
    if(atm_au != NULL)
          delete[] atm_au;
    if(iEqui)
          delete[] iEqui;
    if(Secondary)
          delete[] Secondary;
    if(bonds)
          delete[] bonds;
    if(bonds_ca)
          delete[] bonds_ca;
    if(bonds_bb)
          delete[] bonds_bb;
    if(bonds_nuc)
          delete[] bonds_nuc;
    if(bonds_lig)
          delete[] bonds_lig;
    if(bonds_mix)
          delete[] bonds_mix;
    if(anglebonds)
          delete[] anglebonds;
    if(torsions)
          delete[] torsions;
    if(Polyhedra)
          delete[] Polyhedra;
    if(PolyStyle)
          delete[] PolyStyle;
    if (legsH)
          delete[] legsH;
    if (legsW)
          delete[] legsW;
    if (NConns)
          delete[] NConns;
    if (NEquivs)
          delete[] NEquivs;
    atm = NULL;
    atm_au = NULL;
    iEqui = NULL;
    bonds = NULL;
    bonds_ca = NULL;
    bonds_bb = NULL;
    bonds_nuc = NULL;
    bonds_lig = NULL;
    bonds_mix = NULL;
    anglebonds = NULL;
    torsions = NULL;
    legsH = NULL;
    legsW = NULL;
    Polyhedra = NULL;
    PolyStyle = NULL;
    Secondary = NULL;
    NConns = NULL;
    NEquivs = NULL;
}

void Model::freeModel()
{
      if(Surface_c)
      {
	      Properties::usedC[nModello] = -1;
          VWSurface.freeMap();
      }
      Surface_c = false;
      freeVectors(); //true);
      freeStructs(); //(true);
      created = false;
      ifSymmetry = false;
      ifPoly = false;
}

void Model::InitColors()
{
      for(int i=0; i<NUM_COLORS; i++)
          light_diffuse[i] = Properties::GetColor(i);
}

void Model::setValues(int status[], float fstatus[])
{
      InitColors();
      NConns = nullptr;
      NEquivs = nullptr;
      PolyStyle = nullptr;
      Polyhedra = nullptr;
      Secondary = nullptr;
      anglebonds = nullptr;
      atm = nullptr;
      atm_au = nullptr;
      bonds = nullptr;
      bonds_bb = nullptr;
      bonds_ca = nullptr;
      bonds_lig = nullptr;
      bonds_mix = nullptr;
      bonds_nuc = nullptr;
      hasSecondary = false;
      iEqui = nullptr;
      isPDB = false;
      legsH = nullptr;
      legsW = nullptr;
      nPoly = 0;
      nPolyStyles = 0;
      nResAmino = 0;
      nangbonds = 0;
      nats = 0;
      nats_au = 0;
      nconn = 0;
      nconn_bb = 0;
      nconn_ca = 0;
      nconn_lig = 0;
      nconn_mix = 0;
      nconn_nuc = 0;
      nconn_prot = 0;
      nlegsH = 0;
      nlegsW = 0;
      nlegs_au = 0;
      ntorsions = 0;
      nucleic = 0;
      numSpec = 0;
      torsions = nullptr;

      ifResize = true;
      Bmin = Bmax = Bmin_ca = Bmax_ca = 0;
      RF = 0;
      ifVis = (status[0] == 1);
      ifSymmetry = (status[1] == 1);
      ifDrawH = (status[2] == 1);
      plotStyle  = status[3];
      colorMode = status[4];
      ifLabels = (status[5] != 0);
      ifChangedSymmetry = (status[6] != 0);
      drawLigand = status[7];
      ifPoly = (status[9] != 0);
      ifDrawW = (status[10] == 1);
      specialLab = status[11];
      labelType = status[12];
      klevel[0] = fstatus[0];
      klevel[1] = fstatus[1];
      klevel[2] = fstatus[2];
      FitOpt = status[16];
      for(int i=0; i<N_ELEMENTS; i++)
          LabelS[i] = 0;
      for(int i=0; i<4; i++)
          numbStructs[i] = 0;
      for(int i=0; i<3; i++)
          baricentro[i] = 0;
      for (int i=0; i< MAX_SPECIES; i++)
      {
           Species[i] = 0;
           Content[i] = 0;
           ptab[i] = 0;
           radii[i][0] = 0;
           radii[i][1] = 0;
      }
      for(int i=0; i<NHDistances; i++)
         distHn[i] = 0;
      switch(jav->GetBgType())
      {
      case ColorWidget::TYPE_BLACK:      //Black background
          labelcolor = ToQColor(0.85f, 0.85f, 0.85f);
          break;
      case ColorWidget::TYPE_WHITE:      //White background
          labelcolor = ToQColor(0.0f, 0.0f, 0.7f);
          break;
      default:      //Other
          labelcolor = ToQColor(1.0f, 1.0f, 1.0f);
          break;
      }
      light_chain = ToQColor(0.7f, 0.7f, 0.7f);

      nResTot = 0;
      ifSurface = false;
      Surface_c = false;
}

void Model::FillAtoms(atom_type *atml)
{
  atm = new jav_atom_type [nats]();
  iEqui = new int [nats]();
  for(int i=1; i<=nats_au; i++)
  {
      atm[i-1].lab = string(atml[i-1].lab, MAX_LABLEN);
      atm[i-1].deleted = false;
      int sp = abs(atml[i-1].nz)/nzc - 1;
      if(sp >= 0)
         atm[i-1].color = sp % MAX_SPECIES;
      else
         atm[i-1].color = -1;
      atm[i-1].ptab =  atml[i-1].ptab;
      atm[i-1].Z =  ZFromPtab(atm[i-1].ptab-1);
      if(atm[i-1].Z <= 0)
            atm[i-1].Z =  OTHERS;

      atm[i-1].op.op = 1;
      iEqui[i-1] = i - 1;

      atm[i-1].numResiduo = 0;
      atm[i-1].catena = 0;
      atm[i-1].catseq = 0;
      for(int j=1; j<=3; j++)
      {
          atm[i-1].xyzc[j-1] = atml[i-1].xc[j-1];
          baricentro[j-1] += atm[i-1].xyzc[j-1];
      }
      atm[i-1].B = atml[i-1].biso;
      atm[i-1].ocry = atml[i-1].ocry;
      atm[i-1].och = atml[i-1].och;
      atm[i-1].H = atml[i-1].inte;
      if(atm[i-1].B < Bmin)
         Bmin = atm[i-1].B;
      else if(atm[i-1].B > Bmax)
         Bmax = atm[i-1].B;
      for(int ll=0; ll<6; ll++)
          atm[i-1].Bij[ll] = atml[i-1].bij[ll];
  }
  GetEquivs();
}

string filtrastr(const string &s, const char c)
{
   string s1;
   for(int i=0; i<(int)s.length(); i++)
   {
       if(s.at(i) != c)
          s1.append(1, s.at(i));
   }
   return s1;
}

void Model::FillSecondaryAndHet()
{
    for(int i=0; i<nats_au; i++)
    {
        int l = atm_au[i].prot_info.length() - 1;
        int p = (int) atm_au[i].prot_info.find_last_of( "_" );
        if ((p >= 0) && ((p+2) <= l))
            Secondary[i] = atm_au[i].prot_info.at(p+2);
        else
            Secondary[i] = ' ';
        if(Secondary[i] == 'L')
           atm_au[i].heta = 1;
    }
}

void Model::FillAtoms(atom_type *atml, structure_info_type StructInfo, bool au)
{
    int natoms;
    jav_atom_type *atoms;
    if(au)
    {
       natoms = nats_au;
       atoms = atm_au;
    }
    else
    {
       natoms = nats;
       atoms = atm;
    }
    for(int i=1; i<=natoms; i++)
    {
        atoms[i-1].lab = filtrastr(string(atml[i-1].lab, MAX_LABLEN), '\0');
        if(atml[i-1].chain[0] != '/')
           atoms[i-1].prot_info = trim(atoms[i-1].lab) + "_" + string(atml[i-1].chain, 16);
        else
           atoms[i-1].prot_info = trim(atoms[i-1].lab) + string(atml[i-1].chain, 16);
        atoms[i-1].deleted = false;
        atoms[i-1].hidden = false;
        atoms[i-1].ptab =  atml[i-1].ptab;
        atoms[i-1].Z =  ZFromPtab(atoms[i-1].ptab-1);
        if(atoms[i-1].Z <= 0)
              atoms[i-1].Z =  OTHERS;
        int sp = abs(atml[i-1].nz)/nzc - 1;
        if(sp >= 0)
           atoms[i-1].color = sp % MAX_SPECIES;
        else
           atoms[i-1].color = -1;

        if(!au)
        {
           atoms[i-1].op.op = atml[i-1].op.op;
           memcpy(atoms[i-1].op.tra, atml[i-1].op.tra, 3*sizeof(int));
           if(atoms[i-1].op.op <= 0)
              atoms[i-1].op.op = 1;
           if((atoms[i-1].op.op <= 0) || (atoms[i-1].op.op > StructInfo.nsymmstr))
           {
               qDebug() << "C\'e\' qualcosa di strano in oper \n";
               qDebug() << "Atom " << i << "oper= " << atoms[i-1].op.op << "\n";
           }
           iEqui[i-1] = (atml[i-1].asym)?(atml[i-1].asym - 1):i-1;
           if((iEqui[i-1] < 0) || (iEqui[i-1] >= nats_au))
           {
                 qDebug() << "C\'e\' qualcosa di strano in asym\n";
                 qDebug() << "Atom " << i << "asym= " << iEqui[i-1] << "\n";
           }
        }
        else
           atoms[i-1].op.op = 1;
        atoms[i-1].numResiduo = 0;
        atoms[i-1].catena = 0;
        atoms[i-1].catseq = 0;
        float xyz[3];
        for(int j=1; j<=3; j++)
        {
            atoms[i-1].xyzc[j-1] = atml[i-1].xc[j-1];
            if(!au)
               baricentro[j-1] += atoms[i-1].xyzc[j-1];
            xyz[j-1] = atml[i-1].xc[j-1];
        }
        vet3 xyzo = Properties::FractToCart(xyz);    //fraz. --> orto
        for(int j=1; j<=3; j++)
            atoms[i-1].xyz[j-1] = (float)xyzo.v[j-1];
        atoms[i-1].B = atml[i-1].biso;
        atoms[i-1].ocry = atml[i-1].ocry;
        atoms[i-1].och = atml[i-1].och;
        atoms[i-1].H = atml[i-1].inte;
        if(!au)
        {
        if(atoms[i-1].B < Bmin)
           Bmin = atoms[i-1].B;
        else if(atoms[i-1].B > Bmax)
           Bmax = atoms[i-1].B;
        }
        for(int ll=0; ll<6; ll++)
            atoms[i-1].Bij[ll] = atml[i-1].bij[ll];

        //CheckDistAng();
        atoms[i-1].heta = 0;
    }
    if(isPDB && au)
    {
       int n = nats_au;
       memset(Secondary, ' ', n);
       FillSecondaryAndHet();
#if 0
       getsecondary_(Secondary, &n, (size_t) n);
       for(int i=0; i<n; i++)
       {
           if(Secondary[i] == 'L')
              atoms[i].heta = 1;
       }
#endif
    }

}

void Model::DrawPeaks(QOpenGLFunctions *context)
{
  if(!ifVis)

     return;
  int subdiv;
  GLboolean val;
  float Scale = Properties::Scale;
  float sc_factor = Properties::sc_factor;

  context->glGetBooleanv(GL_LIGHTING, &val);
  context->glDisable (GL_LIGHTING);

  context->glBindTexture(GL_TEXTURE_2D, Properties::iTexture[0]);
  context->glEnable (GL_TEXTURE_2D);

  float subd;
  if((plotStyle == STICK_AND_BALLS) || (plotStyle == ELLIPSOIDS))
     subd = (float) (float) (300. * maxRad[0]) * sc_factor;
  else if(plotStyle == VANDERWAALS)
     subd = (float) (float) (300. * maxRad[1]) * sc_factor;
  else
     subd = (float) (float) (300. * maxRad[2]) * sc_factor;
  if( (plotStyle >= STICK_AND_BALLS) && (plotStyle <= VANDERWAALS) )
     subd *= (float) Properties::sc_atoms;
  subdiv = (Scale > 1.)? ((int) (subd * Scale)) : ((int) subd);

  for(int i=0; i<nats; i++)
  {
      int z = atm[i].Z-1;
      if((z< 0) || (z > N_ELEMENTS))
         z = N_ELEMENTS;
      if(!IsPeak(i))
         continue;
      float radius;
      if(plotStyle == VANDERWAALS)
         radius = Properties::scale_atoms[z] * atm[i].radN[1];
      else
         radius = Properties::scale_atoms[z] * atm[i].radN[0];
      renderSphere(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                   radius, subdiv, true);
  }
  context->glDisable (GL_TEXTURE_2D);
  if(val == GL_TRUE)
     context->glEnable (GL_LIGHTING);
}

void Model::DrawModel(QOpenGLFunctions *context)
{
    context->glDisable (GL_BLEND);
    if((plotStyle >= POLYPEPTIDE) && (nconn_prot == 0) && (nconn_ca != 0))
       plotStyle = C_ALPHA;
    if((nconn_bb == 0) && (plotStyle >= POLYPEPTIDE))
        plotStyle = STICK_AND_BALLS;
    if((nconn_bb == 0) && (colorMode >= BY_RESIDUE))
        colorMode = BY_SPECIES;
    if(plotStyle < RIBBONS)
    {
       context->glDisable(GL_COLOR_MATERIAL);
       context->glEnable (GL_LIGHTING);
       context->glEnable (GL_NORMALIZE);
    }
    else
    {
       context->glDisable(GL_FOG);
       context->glDisable(GL_CLIP_PLANE0);
       context->glDisable(GL_CLIP_PLANE1);
    }
    glShadeModel(GL_SMOOTH);
    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;
    int colorSave = colorMode;

    switch(plotStyle)
    {
    case WIRES:
    case BONDSTYLE:
         DrawSticks(context);
         DrawCircles(1);
         break;

    case SIMPLE:
         DrawSticks(context);
         for(int i=0; i<nats;i++)
             atm[i].disegnato = 0;
         DrawCircles();
         break;

    case VANDERWAALS:
         DrawBalls(context, 1);
         break;

    case ELLIPSOIDS:
         DrawBalls(context, 0, true);
         DrawCyls (context);
         break;
    case STICK_AND_BALLS:
         DrawCyls (context);
         for(int i=0; i<nats;i++)
             atm[i].disegnato = 0;
         DrawBalls(context, Properties::RadType?3:0);
         break;
    case RIBBONS:
    case STRANDS:
    case CARTOONS:
         DrawRibbons(context);
         if(nucleic == 1)
            DrawScaletta(context);
    case C_ALPHA:
    case RODS:
    case POLYPEPTIDE:
         if(plotStyle < RIBBONS)
            DrawCyls (context);
         if(plotStyle == RODS)
            DrawBalls(context, 2);
         if((plotStyle >= POLYPEPTIDE) && ((drawLigand > 0) || ifDrawW ))
         {
             colorMode = colorSave;
             context->glDisable(GL_COLOR_MATERIAL);
             if(colorMode >= BY_RESIDUE)
                colorMode = BY_SPECIES;
             if(drawLigand > 0)
             {
             switch(drawLigand-1)
             {
             case WIRES:
             case BONDSTYLE:
                  DrawSticks(context, 0, nconn_lig, bonds_lig, true);
                  for(int i=0; i<nats;i++)
                      if(!atm[i].disegnato)
                          atm[i].disegnato = !IsHetero(i);
                  DrawIsolated(context);
                  break;
             case SIMPLE:
                 for(int i=0; i<nats;i++)
                     atm[i].disegnato = 1;
                 DrawSticks(context, 0, nconn_lig, bonds_lig, true);
                 for(int i=0; i<nats;i++)
                         atm[i].disegnato = !IsHetero(i);
                 DrawCircles(1);
                 break;
            case STICK_AND_BALLS:
            case RODS:
                 for(int i=0; i<nats;i++)
                     atm[i].disegnato = 1;
                 DrawCyls(context, false, true);
                 if((drawLigand - 1) == STICK_AND_BALLS)
                 {
                    for(int i=0; i<nats;i++)
                        atm[i].disegnato = !IsHetero(i);
                    DrawBalls(context, 0);
                 }
                 else
                 {
                    for(int i=0; i<nats;i++)
                        if(!atm[i].disegnato)
                           atm[i].disegnato = !IsHetero(i);
                    DrawBalls(context, 2);
                 }
            }
            }
            else
            {
                for(int i=0; i<nats;i++)
                    atm[i].disegnato = !atm_au[iEqui[i]].isW;
                DrawBalls(context, 0);
             }
             colorMode = colorSave;
             if(plotStyle == POLYPEPTIDE)
                DrawSBonds(context, nconn_mix, bonds_mix,'M');
         }
         break;
    }
    if(ifPoly && (nPoly > 0))
       DrawPolyhedra(context);
    if(Properties::ifHBonds)
       DrawSBonds(context, nlegsH, legsH,'H');
    if(Properties::ifShortc)
       DrawSBonds(context, nlegsW, legsW, 'W');

    if(Properties::Fog)
       context->glEnable(GL_FOG);
    if(Properties::Clip)
    {
       context->glEnable(GL_CLIP_PLANE0);
       context->glEnable(GL_CLIP_PLANE1);
    }
}

void Model::DrawBalls(QOpenGLFunctions *context, int ind, bool ifEll)
{
  float color1[4];
  int subdiv;

  float subd = (float) (300. * maxRad[ind]) *Properties::sc_factor;
  if(ind != 2)
     subd *= (float) Properties::sc_atoms;
  subdiv = (Properties::Scale > 1.)? ((int) (subd * Properties::Scale)) : ((int) subd);
  for(int i=0; i<nats; i++)
  {
      int z = atm[i].Z-1;
      if((z< 0) || (z > N_TRUE_ELEMENTS))
         z = N_TRUE_ELEMENTS;
      if( (!ifDrawH) &&  IsHydrogen(i) )
          continue;
      if( (!ifDrawW) &&  atm_au[iEqui[i]].isW )
          continue;
      if(atm[i].deleted || atm[i].hidden)
            continue;
      if(IsPeak(i))
         continue;
      if( (ind != 1) && atm[i].disegnato)
            continue;

      (void) SetColor(i, color1);
      if(Properties::ifDesaturate())
         Desaturate(color1);
      glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
      context->glEnable (GL_BLEND);
      context->glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      float radius = atm[i].radN[ind];
      if(ind != 2)
         radius *=  Properties::scale_atoms[z];
      else
         radius *= (float) Properties::sc_bonds;

      if(ifEll && (!IsHydrogen(i) || !Properties::ell_hydro))
      {
         GLfloat mat_planes[] = { color1[0], color1[1], color1[2], 0.8f};
         GLfloat mat_ell[]    = { color1[0], color1[1], color1[2], 1};
         QColor temp = Properties::ell_color; // Darker(ToQColor(mat_planes,4));
         temp.setAlphaF(mat_planes[3]);
         FromQColor(temp, mat_planes, 4);
         float semi = atm[i].seminN * Properties::scale_atoms[z];
         glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_planes);
         if(Properties::ell_princ)
            renderCircs(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                     semi, subdiv, atm[i].scala, atm[i].mat);
         mat_ell[3] = 0.01f * (100 - Properties::ell_tr);
         glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_ell);
         renderEll(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                   semi, subdiv, atm[i].scala, atm[i].mat);
      }
      else
      {
         renderSphere(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                   radius, subdiv);
      }
      glDisable (GL_BLEND);
  }
}

void Model::DrawCyls(QOpenGLFunctions *context, bool selected, bool ifLigands)
{
  int i0=0, i1=0;
  jav_bond_type *legs=NULL;

  if(ifLigands)
  {
     i0 = 0;
     i1 = nconn_lig;
     legs = bonds_lig;
  }
  else if(plotStyle < VANDERWAALS)
  {
     i0 = 0;
     i1 = nconn_prot;
     legs = bonds;
  }
  else if ( (plotStyle == POLYPEPTIDE) || (plotStyle == RIBBONS) )
  {
     i0 = 0;
     i1 = nconn_bb;
     legs = bonds_bb;
  }
  else if (plotStyle == C_ALPHA)
  {
     i0 = 0;
     i1 = nconn_ca;
     legs = bonds_ca;
  }
  else if (plotStyle == CARTOONS)
  {
     i0 = 0;
     i1 = nconn_nuc;
     legs = bonds_nuc;
  }

  for(int i=0; i<nats;i++)
      atm[i].disegnato = ((plotStyle == STICK_AND_BALLS)||(plotStyle == ELLIPSOIDS));
  if(legs != NULL)
  {
     if (selected)
         DrawSelectedCylinder(context, i0, i1, legs);
     else
         Draw2Cylinder(context, i0, i1, legs);
  }
  if (selected)
  {
     if(nlegsH)
         DrawSelectedCylinder(context, 0, nlegsH, legsH);
     if(nlegsW)
         DrawSelectedCylinder(context, 0, nlegsW, legsW);
  }
}

void Model::DrawSelectedCylinder(QOpenGLFunctions *context, int i0, int i1, jav_bond_type *leg)
{
    vet3 p1, p2;
    double radius_c = Properties::cylinder*Properties::sc_bonds*1.5;

    context->glEnable (GL_NORMALIZE);
    context->glEnable (GL_LIGHTING);
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    context->glEnable(GL_COLOR_MATERIAL);
    context->glEnable (GL_BLEND);
    context->glBlendFunc (GL_SRC_ALPHA,  GL_SRC_ALPHA);

    for(int i=i0; i<i1;i++)
    {
         if(!leg[i].selected)
            continue;
        int j1 = leg[i].n1 - 1;
        if(j1 >= nats)
            continue;
        if(atm[j1].deleted || atm[j1].hidden)
            continue;
        if(j1 < 0)
           return;
        int j2 = leg[i].n2 - 1;
        if(j2 >= nats)
            continue;
        if(atm[j2].deleted || atm[j2].hidden)
            continue;
        p1 =  MRVectorMake(atm[j1].xyzN);
        p2 =  MRVectorMake(atm[j2].xyzN);
        glPushMatrix();
        double v;
        mat4x4 m = MRCylinderMatrix(&v, p1, p2, MRVectorZero);
        glTranslated( p1.x, p1.y, p1.z);
        glMultMatrixd (m.m);
        DrawCylinder(v, radius_c, Properties::subdivisions);
        atm[j1].disegnato = 1;
        atm[j2].disegnato = 1;
        glPopMatrix();
    }

    context->glDisable(GL_COLOR_MATERIAL);
    context->glDisable (GL_BLEND);
    SetLightForSelected(context, 1);
}

void Model::Draw2Cylinder(QOpenGLFunctions *context, int i0, int i1, jav_bond_type *legs)
{
    vet3 p1, p2, p3;
    int index1, index2;
    float color1[4], color2[4];
    float lc[4];
    FromQColor(light_chain, lc);
    double radius_c = Properties::cylinder*Properties::sc_bonds;
    if(legs == NULL)
       legs = bonds;

    context->glEnable (GL_LIGHTING);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, lc);

    for(int i=i0; i<i1;i++)
    {
        int j1 = legs[i].n1 - 1;
        if(j1 >= nats)
            continue;
        if(j1 < 0)
           return;
         if(atm[j1].deleted || atm[j1].hidden)
            continue;
        int j2 = legs[i].n2 - 1;
        if(j2 >= nats)
            continue;
         if(atm[j2].deleted || atm[j2].hidden)
            continue;
        if( (!ifDrawH) &&  (IsHydrogen(j1) || IsHydrogen(j2)) )
            continue;
        if( (!ifDrawW) && ( atm_au[iEqui[j1]].isW  || atm_au[iEqui[j2]].isW ) )
            continue;
        index1 = SetColor(j1, color1);
        index2 = SetColor(j2, color2);
        int ordleg = legs[i].ordleg;

        if(!Properties::ShowMultBonds)
           ordleg = 1;
        p1 = MRVectorMake(atm[j1].xyzN);
        p2 = MRVectorMake(atm[j2].xyzN);
        p3 = MRVectorMedium(p1, p2);

        double v;
        mat4x4 m;
        vet3 p4 = MRVectorZero;
        if(ordleg > 1)                    //07/08/2019 per legami deformati
        {
           for(int kat=0; kat<atm[j1].numleg; kat++)
           {
               int j3 = atm[j1].leg[kat];
               if(j3 != j2)
               {
                  p4 = MRVectorMake(atm[j3].xyzN);
                  break;
               }
           }
           if(IsZero(MRVectorLength(p4)))
           {
              for(int kat=0; kat<atm[j2].numleg; kat++)
              {
                  int j3 = atm[j2].leg[kat];
                  if(j3 != j1)
                  {
                     p4 = MRVectorMake(atm[j3].xyzN);
                     break;
                  }
              }
           }
        }
        if(Properties::ifDesaturate())
           Desaturate(color1);
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
        if(index1 == index2)
        {
           glPushMatrix();
           m = MRCylinderMatrix(&v, p1, p2, p4);
           glTranslated( p1.x, p1.y, p1.z);
           glMultMatrixd (m.m);
           DrawCylinder(v, radius_c, Properties::subdivisions,
                         atm[j1].disegnato, atm[j2].disegnato, ordleg);
        }
        else
        {
           glPushMatrix();
           m = MRCylinderMatrix(&v, p1, p3, p4);
           glTranslated( p1.x, p1.y, p1.z);
           glMultMatrixd (m.m);
           DrawCylinder(v, radius_c, Properties::subdivisions,
                         atm[j1].disegnato, 1, ordleg);
           if(Properties::ifDesaturate())
              Desaturate(color2);
           glPopMatrix();
           glPushMatrix();
           m = MRCylinderMatrix(&v, p3, p2, p4);
           glTranslated( p3.x, p3.y, p3.z);
           glMultMatrixd (m.m);
           glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color2);
           DrawCylinder(v, radius_c, Properties::subdivisions,
                      1, atm[j2].disegnato, ordleg);
        }
        atm[j1].disegnato = 1;
        atm[j2].disegnato = 1;
        glPopMatrix();
    }
}

int Model::SetColor(int whichAtom, float color1[])
{
     int index1, ieq;
     for(int i=0; i<3; i++)
         color1[i] = 0;
     switch(colorMode)
     {
     case BY_SPECIES:
           index1  = atm[whichAtom].Z-1;
           if((index1  < 0)||(index1 >= N_ELEMENTS))
              index1 = OTHERS;
           break;
     case BY_SYMMETRY:
           index1 = atm[whichAtom].op.op - 1;
           if(index1 >= N_AMINO)
              index1 = OTHERS;
           else
              index1 += N_ELEMENTS;
           break;
     case BY_RESIDUE:
           ieq = iEqui[whichAtom];
           index1 = atm_au[ieq].numResiduo;
           if(index1 >= (N_AMINO + N_DNA_NUCLEOTIDES))
              index1 = OTHERS;
           else
              index1 += N_ELEMENTS;// + N_CHAINS;
           break;
     case BY_CHAIN:
           ieq = iEqui[whichAtom];
           index1 = atm_au[ieq].catena;
           if(index1 >= N_CHAINS)
              index1 = OTHERS;
           else
              index1 += N_ELEMENTS+N_AMINO;
           break;
     case BY_STRUCT:
           ieq = whichAtom;     //iEqui[whichAtom];
           if(IsHelix(ieq))
              index1 = color_helix;
           else if(IsBetaSheet(ieq))
              index1 = color_sheet;
           else if(IsTurn(ieq))
              index1 = color_turn;
           else if(IsDna(ieq))
              index1 = is_valid_symbol("P") - 1;
           else
              index1 = color_coils;
           break;
     case MONOC:
           index1 = MONOCOL;
           break;
     default:
           index1 = OTHERS;
           break;
     }
     if(colorMode == BY_GROUP)
     {
         ieq = iEqui[whichAtom];
         int ch = atm_au[ieq].catseq;
         int nRestot = Chains[ch].nResidues;
         int r = nRestot - atm_au[ieq].seq;
         TuneH(nRestot, r, color1);
         color1[3] = 1.0f;
     }
     else if(colorMode == BY_B)
     {
         float b, b2;
         if(plotStyle == C_ALPHA)
         {
            b  = Bmax_ca - atm[whichAtom].B;
            b2 = Bmax_ca - Bmin_ca;
         }
         else
         {
            b  = Bmax - atm[whichAtom].B;
            b2 = Bmax - Bmin;
         }
         TuneH(b2, b, color1);
         color1[3] = 1.0f;
     }
     else if(colorMode == BY_SOF)
     {
         float r = 1.f - atm[whichAtom].ocry;
         TuneH(1.f, r, color1);
         color1[3] = 1.0f;
     }
     else
         FromQColor(light_diffuse[index1], color1);
      return index1;
 }

bool Model::IsHeavy(int which)
{
  return(atm[which].Z >= 10);
}

bool Model::IsHydrogen(int which)
{
  return(atm[which].Z == 1);
}

bool Model::IsPeak(int which)
{
  return (atm[which].color == -1);
}

bool Model::IsDna(int which)
{
   int i = iEqui[which];
   return ( Secondary[i] == 'D') ;
}

bool Model::IsLigand(int which)
{
   int i = iEqui[which];
   return ( (Secondary[i] == 'L') | (Secondary[i] == 'l') );
}

bool Model::IsHelix(int which)
{
   int i = iEqui[which];
   return ( (Secondary[i] == 'H') | (Secondary[i] == 'h') |
            (Secondary[i] == 'P') | (Secondary[i] == 'p') |
            (Secondary[i] == 'G') | (Secondary[i] == 'g')
          );
}

bool Model::IsBetaSheet(int which)
{
    int i = iEqui[which];
    return ( (Secondary[i] == 'E') | (Secondary[i] == 'e') |
             (Secondary[i] == 'S') | (Secondary[i] == 's'));
}

bool Model::IsTurn(int which)
{
    int i = iEqui[which];
    return ( (Secondary[i] == 'T') | (Secondary[i] == '&')  |
             (Secondary[i] == '1') | (Secondary[i] == '2')  |
             (Secondary[i] == '3') | (Secondary[i] == '4')  |
             (Secondary[i] == '5') | (Secondary[i] == '6')  |
             (Secondary[i] == '7') | (Secondary[i] == '8')  |
             (Secondary[i] == '@') );
}

void Model::Normalizza()
{
  float pi2_8 = (float) (8*M_PI*M_PI);
  for(int j=0; j<3; j++)
      maxRad[j] = 0;
  for(int i=0; i<nats; i++)
  {
      int index = atm[i].color;
      if(index >=0)
      {
         atm[i].radN[0] = radii[index][0] / Properties::deltav;
         atm[i].radN[1] = 2.f * radii[index][1] / Properties::deltav;
      }
      else
      {
         atm[i].radN[0] = 0.37f / Properties::deltav;
         atm[i].radN[1] = 2.2f  / Properties::deltav;
      }
      atm[i].radN[2] = (float) Properties::cylinder;
      float bf = abs(atm[i].B);
      if(bf < 0.01f)
         bf = 0.01f;
       atm[i].radN[3] = 3.5f * (float) (sqrt(bf/pi2_8))/Properties::deltav;

      atm[i].seminN = atm[i].semin / Properties::deltav;
      for(int j=0; j<4; j++)
      {
         if((atm[i].radN[j] / Properties::cylinder) <= 1.1)
             atm[i].radN[j] = (float) (Properties::cylinder * 1.15);
          maxRad[j] = max(maxRad[j], atm[i].radN[j]);
      }
      vet3 a = Properties::Trasforma(MRVectorMake(atm[i].xyz));
      for(int j=0; j<3; j++)
          atm[i].xyzN[j] = (float) a.v[j];
  }
}

void Model::FillLegs(int *nl, int *nl_au, f_bond_type *legm, bool if_edit)
{
  nconn_prot = *nl;
  nconn_bb = 0;
  nconn_lig = 0;
  nconn_ca = 0;
  nlegs_au = *nl_au;
  nconn = nconn_prot;
  bonds = new jav_bond_type [nconn]();
  for(int i=1; i<=nconn; i++)
  {
      int n1 = legm[i-1].n1;
      int n2 = legm[i-1].n2;
      bonds[i-1].n1 = n1;
      bonds[i-1].n2 = n2;
      bonds[i-1].dist = legm[i-1].dist;
      bonds[i-1].sigma = legm[i-1].sigma;
      if(legm[i-1].ord <= 0)
         bonds[i-1].ordleg = 1;
      else
         bonds[i-1].ordleg = legm[i-1].ord;
      if(!if_edit)
      {
         int numleg = atm[n1-1].numleg;
         atm[n1-1].leg[numleg] = n2-1;
         atm[n1-1].dist[numleg] = bonds[i-1].dist;
         numleg = atm[n2-1].numleg;
         atm[n2-1].leg[numleg] = n1-1;
         atm[n2-1].dist[numleg] = bonds[i-1].dist;
         atm[n1-1].numleg++;
         atm[n2-1].numleg++;
      }
  }
}

void Model::GetEquivs()
{
   NEquivs = new int [nats_au]();
   for(int nat=0; nat<nats_au; nat++)
   {
       std::vector<int> eq;
       NEquivs[nat] = 1;
       Equivs.push_back(eq);
       for(int i=0; i<nats; i++)
       {
          if(iEqui[i] == nat)
             Equivs[nat].push_back(i);
       }
       NEquivs[nat] = (int) Equivs[nat].size();
   }
}

void Model::AddProb()
{
    float P = (float) Properties::ell_prob / 100.f; //0.5f;
    float FProb;
    if(P <= 0.985f)
        FProb = (((((23.326f*P)-55.693f)*P+51.552f)*P-22.75f)*P+6.447f)*P+0.296f;
    else
        FProb = -136.496 + 141.072 * P;
    double radius_c = Properties::cylinder*Properties::sc_bonds;

    for(int i=0; i<nats; i++)
    {
        atm[i].semin = 2 * atm[i].seminD * FProb;
        atm[i].seminN = atm[i].semin / Properties::deltav;
        if(atm[i].seminN < radius_c)
            atm[i].seminN = radius_c;
    }
}

void Model::PreparaEllissoidi()
{
    float el[3][3];
    float pp2 = (float) (M_PI * M_PI * 2);
    int ifIso = 0;
    int pnat = 0;
    int pdraw = 0;
    std::string outStr;
    outStr.assign("Problems with anisotropic thermal parameters of atom(s)\n");
    std::string drawStr;
    drawStr.assign("Problems with ellipsoid semiaxes of atom(s)\n");
    for(int i=0; i<nats; i++)
    {
        float Ell[3][3], CD[3][3], semax[3];
        if (atm[i].Bij[0] > 0.f)
        {
            Ell[0][0] = atm[i].Bij[0];
            Ell[0][1] = atm[i].Bij[3];    //[1];
            Ell[1][0] = atm[i].Bij[3];    //[1];
            Ell[0][2] = atm[i].Bij[4];    //[2];
            Ell[2][0] = atm[i].Bij[4];    //[2];
            Ell[1][1] = atm[i].Bij[1];    //[3];
            Ell[1][2] = atm[i].Bij[5];    //[4];
            Ell[2][1] = atm[i].Bij[5];    //[4];
            Ell[2][2] = atm[i].Bij[2];    //[5];
            for(int jj=0; jj<3;jj++)
            {
              for(int kk=jj;kk<3;kk++)
              {
                  el[jj][kk] = Ell[jj][kk] / pp2;
                  if(jj != kk)
                     el[jj][kk] = el[jj][kk]/2.f;
              }
            }
            el[1][0] = el[0][1];
            el[2][0] = el[0][2];
            el[2][1] = el[1][2];

            ifIso = 0;
            NorEll(i, el, Ell, CD, semax, &ifIso);
            if(ifIso)
            {
               pnat++;
               outStr.append( trim(atm[i].lab) );
               outStr.append( " " );
            }
         }
         else
            SetEllVal(i, Ell, CD, semax);

        int X = 0, Y = 1, Z = 2;
        float Mat[] = {
                        CD[X][0], CD[Y][0], CD[Z][0], 0,
                        CD[X][1], CD[Y][1], CD[Z][1], 0,
                        CD[X][2], CD[Y][2], CD[Z][2], 0,
                        0, 0, 0, 1};
        memcpy(atm[i].mat, Mat, 16*sizeof(float));

        float semin = 9999.f;
        for(int kk=0; kk<3; kk++)
        {
            if(semin > semax[kk])
               semin =  semax[kk];
        }
        atm[i].scala[3] = 0;
        for(int kk=0; kk<3; kk++)
        {
            atm[i].scala[kk] = semax[kk]/semin;
            if(atm[i].scala[3] < atm[i].scala[kk])
               atm[i].scala[3] = atm[i].scala[kk];
        }
        if(atm[i].scala[3] < 1e-8)
           atm[i].scala[3] = 1;

        if(atm[i].scala[3] > 20)
        {
           SetEllVal(i, Ell, CD, semax);
           for(int kk=0; kk<4; kk++)
               atm[i].scala[kk] = 1;
           semin = 0.2f;
           pdraw++;
           drawStr.append( trim(atm[i].lab) );
           drawStr.append( " " );
        }
        atm[i].seminD = semin;
    }
    AddProb();

    //Corrado: Uncomment for silent-error
    //    return;

    if(pnat)
    {
        outStr.append(".\n");   //Set to isotropic.");
        qDebug() << outStr.c_str();
    }
    if(pdraw)
    {
        drawStr.append(". Draw as isotropic.");
        qDebug() << drawStr.c_str();
    }
}

void Model::SetEllVal(int na, float Ell[3][3], float CD[3][3], float semax[3])
{
      float bf = (float) (atm[na].B / (8 * M_PI * M_PI));
      float Ell_val;
      if (bf <= 0.0000001f)
      {
          int kspec = atm[na].color;
          float Rs2 = radii[kspec][0] * radii[kspec][0] / 9;
          bf = Rs2;
      }
      Ell_val = 1.f / bf;
      for(int j=0; j<3; j++)
      {
          for(int k=0; k<3;k++)
          {
              Ell[j][k] = 0;
              CD[j][k] = 0;
              if(j == k)
              {
                 Ell[j][j] = Ell_val;
                 CD[j][j] = 1.f;
              }
          }
      }
      semax[0] = semax[1] = semax[2] = (float)sqrt(bf);
}

void Model::NorEll(int na, float el[3][3], float Ell[3][3], float CD[3][3],
        float semax[3], int *ifIso)
{
   double del[3][3], eigen[3], t3[3][3];
   int iCod;
   mat3x3 trm = MRMatrix3Transpose(Properties::rmC);
   mat3x3 ba = MRMatrix3Multiply(MRMatrixMakeFromArray(el), trm);
   mat3x3 be = MRMatrix3Multiply(Properties::rmC, ba);
   bool result;
   mat3x3 resultM = MRMatrix3Invert(be, &result);
   if(!result)
   {
      *ifIso = 1;
      SetEllVal(na, Ell, CD, semax);
      return;
   }
   for(int i=0; i<3; i++)
   {
      for(int j=0; j<3; j++)
      {
          t3[i][j] = 0.;
          del[i][j] = resultM.m[i*3+j];//[i][j];
      }
   }
   autov(del, eigen, t3, &iCod);
   result = (iCod != 0) && (eigen[0] > 0) && (eigen[1] > 0) && (eigen[2] > 0);

   if(!result)
   {
      *ifIso = 1;
      SetEllVal(na, Ell, CD, semax);
      return;
   }
   for(int i=0; i<2 ; i++)
   {
       for(int j=i+1; j<3; j++)
       {
            double ps = t3[0][i]*t3[0][j]+t3[1][i]*t3[1][j]+t3[2][i]*t3[2][j];
            if (abs(ps) > 0.000001f)
                result = false;
       }
   }
   if(!result)
   {
      *ifIso = 1;
      SetEllVal(na, Ell, CD, semax);
      return;
   }

   for(int j=0; j<3; j++)
       semax[j] = (float) (1./sqrt(abs(eigen[j])));
   for(int j=0; j<3; j++)
   {
       double q = 0;
       for(int i=0; i<3; i++)
           q += t3[i][j] * t3[i][j];
       for(int i=0; i<3; i++)
       {
           if(abs(t3[i][j]) > 1.e-15)
               t3[i][j] = sqrt(t3[i][j]*t3[i][j]/q)*t3[i][j]/abs(t3[i][j]);
       }
   }
   for(int j=0; j<3; j++)
       for(int i=0; i<3; i++)
           CD[i][j] = (float) t3[i][j];
   return;
}

// Per proteine

bool getAtomTypes(std::string &label, std::string &typ, std::string &Typ)
{
   int p = (int) label.find_first_of( "_" );
   if (p >= 0)
   {
       typ = label.substr(0,p);
       p = (int) typ.find_first_of( "/" );
       if (p > 0)
           Typ = typ.substr(0,p);
       else
       {
           std::string typsav = typ;
           size_t found = typ.find("*");
           if(found != std::string::npos)
              typ.at(found) = 0x27;
           if(CheckAtom(typ.c_str()))
              Typ = typ;
           else
           {
              size_t found = typ.find_first_of("0123456789");
              if(found != std::string::npos)
                 Typ = typ.substr(0, found);
              else
                 Typ = typ;
#ifndef NDEBUG
              if(!is_valid_symbol(Typ.c_str()) && !is_ghost(Typ))
                 qDebug() << "Warning: Invalid atom id: " << Typ.c_str() << "\n";
#endif
              Typ = typ.substr(0,1);
           }
           typ = typsav;
       }
       return true;
   }
   return false;
}

string get_res_name(std::string &label)
{
   string ResName = string();
   int p = (int) label.find_first_of( "_" );
   if (p >= 0)
   {
       std::string lab1 =  label.substr(p+1);
       int q = (int) lab1.find_first_of("_");
       if(q >=0)
           ResName = lab1.substr(0,q);
   }
   return ResName;
}

string get_chain(const std::string &label)
{
   string Chain = string();
   int p = (int) label.find_last_of( "_" );
   if (p >= 0)
       Chain = label.substr(p+1, 1);
    return Chain; //trim(Chain);
}

string get_res_num(std::string &label)
{
    string ResNum = string();
   int p = (int) label.find_last_of( "_" );
   if (p >= 0)
   {
       std::string lab1 = label.substr(0,p);
       int q = (int) lab1.find_last_of( "_" );
       if(q >=0)
           ResNum = lab1.substr(q+1);
   }
   return trim(ResNum);
}

void AddResidue(std::string Label, int which, int istart, RESIDUE *rr)
{
    rr->breakF      = false;
    rr->name        = get_res_name(Label);
    rr->num         = which+1;
    rr->PDBnum      = get_res_num(Label);
    rr->atoms.clear();
    rr->atoms.push_back(istart);
    rr->secondary   = ' ';
    rr->ca          = -1;
    rr->c           = -1;
    rr->n           = -1;
    rr->o           = -1;
    rr->h           = -1;
    rr->code        = IsAminoNucle(rr->name);
}

int Model::FindChain(string &curChain)
{
    std::vector<CHAIN>::iterator it;
    for(it=Chains.begin();it!=Chains.end();it++)
    {
         string lastchain = (*it).id;
         if(lastchain.compare(curChain) == 0)
            return ((int)distance(Chains.begin(), it));
    }
    return -1;
}

int Model::GetNumResiduo(int ch, int r)
{
   int res0 = 0;
   for(int i=0; i<ch-1; i++)
       res0 = res0 + Chains[i].nAmino;
   return (res0+r);
}

std::string Model::GetResName(int res1)
{
   int res0 = 0;
   int ch1 = 0;
   int r1 = 0;
   for(int i=0; i<nChains; i++)
   {
       int resi = Chains[i].nResidues;
       if(res1 <= (res0+resi))
       {
          r1 = res1 - res0;
          ch1 = i;
          break;
       }
       res0 = res0 + resi;
   }
   std::string tname (Chains[ch1].Residues[r1].name);
   return tname;
}

void AddChain(string &chain_id, CHAIN *cc)
{
    cc->id        = chain_id;
    cc->nResidues = 0;
    cc->nAmino    = 0;
    cc->isDNA     = false;
    cc->isRNA     = false;
}

bool Model::IsType(int which, string type)
{
     std::string str = AtomTypes.at(which);
     size_t found = str.find("*");
     if(found != std::string::npos)
        str.at(found) = 0x27;   // '
     return (str.compare(type) == 0);
}

int Model::FindAtom(string type, RESIDUE rr, int tipo)
{
     for(int i=0; i<(int)rr.atoms.size(); i++)
     {
         if(IsType(rr.atoms[i], type))
         {
            if(tipo == 0)
               return rr.atoms[i];
            else if(atm[rr.atoms[i]].Z == tipo)
               return rr.atoms[i];
         }
     }
     return -1;
}

int Model::FindChainId(string &curChain)
{
    int kch = tolower(curChain[0]);
    if( (kch >= 'a') && (kch <='z') )
          return(kch - 'a');
    else
          return 0;
}

int Model::FindResidue(int ch, string &curResidue, RESIDUE *rr, int atom)
{
    for(int i=0; i<Chains[ch].nResidues; i++)
    {
         string lastresidue = Chains[ch].Residues[i].PDBnum;
         if(lastresidue.compare(curResidue) == 0)
         {
            rr = &(Chains[ch].Residues[i]);
            Chains[ch].Residues[i].atoms.push_back(atom);
            return i;
         }
    }
    return -1;
}

void Model::AllCAlpha()
{
   if(c_alpha.empty())
      return;
   if(!ifSymmetry)
   {
        for(int i=0; i<(int) c_alpha.size(); i++)
        {
             C_Alpha c  = c_alpha[i];
             if(i == (int) c_alpha.size() -1)
                c.breakF = true;
             c_alphab.push_back(c);
        }
        return;
   }
   std::vector<int> ca, o, o2, n, c;
   int nEqui=0;
   int **o_equi, **ca_equi, **o2_equi, **n_equi, **c_equi;
   o_equi = new int * [(int) c_alpha.size()]();
   o2_equi = new int * [(int) c_alpha.size()]();
   n_equi = new int * [(int) c_alpha.size()]();
   ca_equi = new int * [(int) c_alpha.size()]();
   c_equi = new int * [(int) c_alpha.size()]();
   for(int ic=0; ic<(int) c_alpha.size(); ic++)
   {
       C_Alpha c_alfa = c_alpha[ic];
       int CA = c_alfa.atom;
       int O  = c_alfa.o;
       int O2 = c_alfa.o2;
       int N  = c_alfa.n;
       int C  = c_alfa.c;
       ca.clear();
       o.clear();
       n.clear();
       c.clear();
       o2.clear();
       for(int i=0; i<nats; i++)
       {
           if(iEqui[i] == O)
              o.push_back(i);
           if(iEqui[i] == CA)
              ca.push_back(i);
           else if(iEqui[i] == O2)
              o2.push_back(i);
           else if(iEqui[i] == N)
              n.push_back(i);
           else if(iEqui[i] == C)
              c.push_back(i);
       }
       nEqui = ((int) ca.size());
       ca_equi[ic] = new int [nEqui]();
       c_equi[ic]  = new int [nEqui]();
       n_equi[ic]  = new int [nEqui]();
       o_equi[ic]  = new int [nEqui]();
       o2_equi[ic] = new int [nEqui]();
       for(int k=0; k<nEqui; k++)
       {
           if(k < (int) o2.size())
              o2_equi[ic][k] = o2[k];
           else
              o2_equi[ic][k] = -1;
           if(k < (int) o.size())
              o_equi[ic][k] = o[k];
           else
              o_equi[ic][k] = -1;
           if(k < (int) n.size())
              n_equi[ic][k] = n[k];
           else
              n_equi[ic][k] = -1;
           if(k < (int) c.size())
              c_equi[ic][k] = c[k];
           else
              c_equi[ic][k] = -1;
           ca_equi[ic][k] = ca[k];
       }
   }
   for(int k=0; k<nEqui; k++)
   {
        for(int i=0; i<(int) c_alpha.size(); i++)
        {
             C_Alpha c  = c_alpha[i];
             c.o = o_equi[i][k];
             c.n = n_equi[i][k];
             c.c = c_equi[i][k];
             c.o2 = o2_equi[i][k];
             c.atom = ca_equi[i][k];
             c.selected = false;
             c_alphab.push_back(c);
        }
    }
    c_alphab[c_alphab.size()-1].breakF = true;
    for(int i=0; i<(int) c_alpha.size(); i++)
    {
        delete [](n_equi[i]);
        delete [](o_equi[i]);
        delete [](o2_equi[i]);
        delete [](c_equi[i]);
        delete [](ca_equi[i]);
    }
    delete [](n_equi);
    delete [](o_equi);
    delete [](c_equi);
    delete [](o2_equi);
    delete [](ca_equi);
    ca.clear();
    o.clear();
    n.clear();
    c.clear();
    o2.clear();
}

void Model::CreaLegsCA()
{
//   float ds[] = { 3.f, 4.3f, 4.f, 8.f};
//   double ds[] = { 2.5, 4.3, 4., 8.};
   double ds[] = { 2.5, 4.3, 3.8, 8.};  // gianluca 22012021
   double dmin, dmax;

   int n = (int)c_alphab.size();
   int *ca = new int [n]();
   std::vector<jav_bond_type> vett;
   for(int i=0; i<n; i++)
       ca[i]   = c_alphab[i].atom;
   for(int i=0; i<n-1; i++)
   {
       if((c_alphab[i].isAmino) && (c_alphab[i+1].isAmino))
       {
          dmin = ds[0];
          dmax = ds[1];
       }
       else if((!c_alphab[i].isAmino) && (!c_alphab[i+1].isAmino))
       {
          dmin = ds[2];
          dmax = ds[3];
       }
       else
       {
          c_alphab[i].breakF = true;
          int ch = c_alphab[i].chain;
          for(int r=0; r<Chains[ch].nResidues; r++)
              if(r == c_alphab[i].residue)
                 Chains[ch].Residues[r].breakF = true;
          continue;
       }
       if((ca[i] != -1) && (ca[i+1] != -1) && (!c_alphab[i].breakF))
       {
           vet3 x1 = {{atm[ca[i]].xyz[0], atm[ca[i]].xyz[1], atm[ca[i]].xyz[2]}};
           vet3 x2 = {{atm[ca[i+1]].xyz[0], atm[ca[i+1]].xyz[1], atm[ca[i+1]].xyz[2]}};
           double dist = MRVectorDistance(x1, x2);
           if((dist >= dmin) && (dist <= dmax))
           {
               jav_bond_type b;
               b.n1 = ca[i] + 1;
               b.n2 = ca[i+1] + 1;
               b.dist = (float) dist;
               vett.push_back(b);
           }
           else
           {
               c_alphab[i].breakF = true;
               int ch = c_alphab[i].chain;
               for(int r=0; r<Chains[ch].nResidues; r++)
                   if(r == c_alphab[i].residue)
                      Chains[ch].Residues[r].breakF = true;
           }
       }
   }
   delete[] (ca);
   nconn_ca = (int) vett.size();
   if(bonds_ca != NULL)
      delete[] (bonds_ca);
   bonds_ca = new jav_bond_type [nconn_ca]();
   for(int i=0; i<nconn_ca; i++)
   {
      bonds_ca[i].n1 = vett[i].n1;
      bonds_ca[i].n2 = vett[i].n2;
      bonds_ca[i].dist = vett[i].dist;
      bonds_ca[i].selected = false;
   }
   vett.clear();
}

int Model::IsTypeBB(int at, const char *types[], int n)
{
    for(int i=0; i<n; i++)
    {
        if(IsType(at, types[i]))
           return (i+1);
    }
    return 0;
}

void Model::CreaLegsBB()
{
   int *ifbb = new int [nats]();
   int *ifnuc = new int [nats]();
   std::vector<int> vett;
   std::vector<int> vettnuc;
   const char *types[] = {"N", "C", "CA", "O", "CB", "CD", "CD1", "CD2",
    "CE", "CE1", "CE2", "CE3", "CG", "CG1", "CG2", "CH2", "CH3",
    "CZ", "CZ2", "CZ3", "SG", "NE2"};
   int ntypes = (int) sizeof(types)/sizeof(types[0]);
   const char *types1[] = {"P", "C1'", "C2'", "C3'", "C4'", "C5'",
                           "O3'", "O4'", "O5'"};
   int ntypes1 = (int) sizeof(types1)/sizeof(types1[0]);
   const char *types2[] = {"P", "O3'", "O4'", "O5'"};
   int ntypes2 = (int) sizeof(types2)/sizeof(types2[0]);
   for(int ch=0; ch<nChains; ch++)
   {
       for(int j=0; j< Chains[ch].nResidues; j++)
       {
           RESIDUE cur = Chains[ch].Residues[j];
           if(cur.code == 0)
              continue;
           if(!Is_Acid( cur.code))
              continue;
           for(int i=0; i<(int)cur.atoms.size(); i++)
           {
               int at = cur.atoms[i];
               if(Is_AminoG(cur.code))
                  ifbb[at] = IsTypeBB(at, types, ntypes);
               else if(Is_DnaG(cur.code))
               {
                  ifbb[at] = IsTypeBB(at, types1, ntypes1);
                  ifnuc[at] = (IsTypeBB(at, types2, ntypes2))?0:1;
               }
               for(int ieq=1; ieq <NEquivs[at]; ieq++)
               {
                   int jat = Equivs[at].at(ieq);
                   ifbb[jat] = ifbb[at];
                   ifnuc[jat] = ifnuc[at];
               }
           }
       }
   }
   for(int i=0; i<nconn_prot; i++)
    {
        int i1 = bonds[i].n1 - 1;
        int i2 = bonds[i].n2 - 1;
        if(ifbb[i1] && ifbb[i2])
          vett.push_back(i);
        if(ifnuc[i1] && ifnuc[i2])
          vettnuc.push_back(i);
    }

    nconn_bb = (int) vett.size();
    if(bonds_bb != NULL)
       delete [] bonds_bb;
    bonds_bb = new jav_bond_type [nconn_bb]();
    for(int i=0; i<nconn_bb; i++)
    {
       bonds_bb[i].n1 = bonds[vett[i]].n1;
       bonds_bb[i].n2 = bonds[vett[i]].n2;
       bonds_bb[i].dist = bonds[vett[i]].dist;
       bonds_bb[i].ordleg = bonds[vett[i]].ordleg;
       bonds_bb[i].selected = false;
    }
    delete [] ifbb;
    vett.clear();
    nconn_nuc = (int) vettnuc.size();
    if(bonds_nuc != NULL)
       delete [](bonds_nuc);
    bonds_nuc = new jav_bond_type [nconn_nuc]();
    for(int i=0; i<nconn_nuc; i++)
    {
       bonds_nuc[i].n1 = bonds[vettnuc[i]].n1;
       bonds_nuc[i].n2 = bonds[vettnuc[i]].n2;
       bonds_nuc[i].dist = bonds[vettnuc[i]].dist;
       bonds_nuc[i].ordleg = bonds[vettnuc[i]].ordleg;
       bonds_nuc[i].selected = false;
    }
    delete [](ifnuc);
    vettnuc.clear();
 }

void Model::CreaLegsLig()
{
   std::vector<int>ib, is;
   for(int i=0; i<nconn_prot; i++)
   {
       int n1 = bonds[i].n1;
       int n2 = bonds[i].n2;
       if(IsHetero(n1-1) && IsHetero(n2-1))
          ib.push_back(i);
       else if(IsHetero(n1-1) || IsHetero(n2-1))
          is.push_back(i);
   }
   nconn_lig = (int) ib.size();
   nconn_mix = (int) is.size();
   if(bonds_lig != NULL)
      delete [](bonds_lig);
   bonds_lig = new jav_bond_type [nconn_lig]();
   for(int i=0; i<nconn_lig; i++)
   {
      bonds_lig[i].n1 = bonds[ib[i]].n1;
      bonds_lig[i].n2 = bonds[ib[i]].n2;
      bonds_lig[i].dist = bonds[ib[i]].dist;
      bonds_lig[i].ordleg = bonds[ib[i]].ordleg;
      bonds_lig[i].selected = false;
   }
   if(bonds_mix != NULL)
      delete [](bonds_mix);
   bonds_mix = new jav_bond_type [nconn_mix]();
   for(int i=0; i<nconn_mix; i++)
   {
      bonds_mix[i].n1 = bonds[is[i]].n1;
      bonds_mix[i].n2 = bonds[is[i]].n2;
      bonds_mix[i].dist = bonds[is[i]].dist;
      bonds_mix[i].ordleg = bonds[is[i]].ordleg;
      bonds_mix[i].selected = false;
   }
}

int Model::PreparaTabelle(string &msg)
{
    CHAIN cc;
    RESIDUE rr;

    freeStructs(); //(true);
    for(int i=0; i<nats_au; i++)
    {
        int ieq = i;             //Equivs[i].at(0);
//        std::string Label = atm_au[ieq].lab, types, Types;
        std::string Label = atm_au[ieq].prot_info, types, Types;
        getAtomTypes(Label, types, Types);
        string chain_id = get_chain(Label);
        if(types.empty() || chain_id.empty())
        {
           isPDB = false;
           return 0;
        }
        atom_types.push_back(types);
        AtomTypes.push_back(Types);
        if(atm_au[ieq].deleted)
           continue;
        int ch = 0;
        if(nChains == 0)
        {
           nChains = 1;
           AddChain(chain_id, &cc);
           Chains.push_back(cc);
        }
        else
        {
           ch = FindChain( chain_id);
           if(ch == -1)
           {
              ch = nChains;
              nChains++;
              AddChain(chain_id, &cc);
              Chains.push_back(cc);
           }
        }
        atm_au[ieq].catseq = ch;   //FindChain( chain_id );
        atm_au[ieq].catena = FindChainId( chain_id );
        int nr = IsAminoNucle( get_res_name(Label) );
        if(nr > 0)
           atm_au[ieq].numResiduo = nr - 1;
        else
           atm_au[ieq].numResiduo = N_AMINO + N_DNA_NUCLEOTIDES;
    }
    int ch = 0;
    for(int i=0; i<nats_au; i++)
    {
        //int ieq = Equivs[i].at(0);
        int ieq = i;
        if(atm_au[ieq].deleted)
           continue;
//        string Label = atm_au[ieq].lab;
        string Label = atm_au[ieq].prot_info;
        string chain_id = get_chain(Label);
        string res_num = get_res_num(Label);
        string res_nam = get_res_name(Label);
        if(chain_id.empty() || res_num.empty() || res_nam.empty())
        {
           isPDB = false;
           return 0;
        }
        atm_au[ieq].ResNum = atoi(res_num.c_str()) - 1;
        atm_au[ieq].isW = IsWater(res_nam);

        ch = FindChain( chain_id);
        if(ch < 0)
        {
           msg = "Some trouble reading PDB.";
           return -1;
        }

        bool to_add = true;
        int curResidue;
        if(Chains[ch].nResidues != 0)
        {
           curResidue = FindResidue( ch, res_num, &rr, ieq);
           if(curResidue != -1)
              to_add = false;
        }
        if(to_add)
        {
           curResidue = Chains[ch].nResidues;
           AddResidue(Label, Chains[ch].nResidues, ieq, &rr);
           Chains[ch].nResidues++;
           Chains[ch].Residues.push_back(rr);
        }
        atm_au[ieq].seq = rr.num;
        atm_au[ieq].resseq = curResidue;
     }
    nResAmino = 0;
    Bmin_ca = 1.e10;
    Bmax_ca = -1.e10;
    int n_c, n_o, n_n;
    n_c = n_o = n_n = 0;
    c_alpha.clear();
    for(int ch=0; ch<nChains; ch++)
    {
        Chains[ch].nAmino = 0;
        for(int r=0; r<Chains[ch].nResidues; r++)
        {
            RESIDUE *rr = &(Chains[ch].Residues[r]);
            int tipo_residuo = 0;
            int at = -1;
            if(Is_Water(rr->code))
               continue;
            if(IsAmino(rr->name))
            {
               at = FindAtom("CA", *rr, 6);
               if(at == -1)
               {
                  qDebug() << "warning: Calpha not found at residue "
                     << rr->PDBnum.c_str() << "(Chain "
                     << Chains[ch].id[0] <<") \n";
                  continue;
               }
               tipo_residuo = 1;
            }
            else if(IsDNA(rr->name))
            {
               at = FindAtom("P", *rr);
               if(at == -1)
               {
                  at = FindAtom("O5'", *rr);
                  if(at == -1)
                     continue;
               }
               tipo_residuo = 2;
            }
            else
            {
                rr->code = N_AMINO + N_DNA_NUCLEOTIDES + 1;
                at = FindAtom("CA", *rr, 6);
                if(at != -1)
                {
                   if((FindAtom("O", *rr) != -1) &&
                      (FindAtom("N", *rr) != -1) &&
                      (FindAtom("C", *rr) != -1) )
                   {
                      tipo_residuo = 1;
                      rr->code = 500;     // non standard amino acid
                   }
                }
                else
                {
                   at = FindAtom("P", *rr);
                   if(at != -1)
                   {
                      if((FindAtom("OP1", *rr) != -1) &&
                         (FindAtom("OP2", *rr) != -1) &&
                         (FindAtom("O5'", *rr) != -1) )
                      {
                         tipo_residuo = 2;
                         rr->code = 600;     // non standard nucleic acid
                      }
  //                    else
  //                       continue;
                   }
                   else
                   {
                      at = FindAtom("O5'", *rr);
                      if(at == -1)
                         continue;
                   }
                   tipo_residuo = 2;
                   rr->code = 600;     // non standard nucleic acid
                }
             }

             if(atm_au[at].B < Bmin_ca)
                Bmin_ca = atm_au[at].B;
             else if(atm_au[at].B > Bmax_ca)
                 Bmax_ca = atm_au[at].B;
              C_Alpha c;
              c.atom = at;
              rr->ca = at;
              c.breakF = (r == Chains[ch].nResidues-1);   //temp
//              std::string Label = atm_au[at].lab;
              std::string Label = atm_au[at].prot_info;
              c.ResLab.assign(get_chain(Label));
              c.ResLab.append(get_res_name(Label));
              c.ResLab.append(get_res_num(Label));
              c.chain = ch;
              c.residue = r;
              if(tipo_residuo == 1)                // amino acid
              {
                 at = FindAtom("O", *rr);
                 c.o = at;
                 rr->o = at;
                 if(at != -1)
                    n_o++;
                 at = FindAtom("C", *rr);
                 c.c = at;
                 rr->c = at;
                 if(at != -1)
                    n_c++;
                 at = FindAtom("N", *rr);
                 c.n = at;
                 rr->n = at;
                 if(at != -1)
                    n_n++;
                 at = FindAtom("H", *rr);
                 if((at == -1) && (!IsProline(rr->name)))
                    at = FindAtom("D", *rr);
                 rr->h = at;
                 c.h = at;
                 c.o2 = -1;
                 c.isAmino = true;
              }
              else if(tipo_residuo == 2)  //nucleic acid
              {
                 at = FindAtom("O5'", *rr);
                 c.o = at;
                 c.isAmino = false;
                 rr->o = at;
                 if(at != -1)
                    n_o++;
                 at = FindAtom("C5'", *rr);
                 c.c = at;
                 rr->c = at;
                 if(at != -1)
                    n_c++;
                 at = FindAtom("OP1", *rr);
                 c.o2 = at;
                 at = FindAtom("O2'", *rr);
                 c.n = at;
                 rr->n = at;
                 if(at != -1)
                    n_n++;
                 at = FindAtom("H", *rr);
                 rr->h = at;
                 c.h = at;
                 Chains[ch].isDNA = true;
                 if(IsUracil(rr->code))
                     Chains[ch].isRNA = true;
              }
              if(tipo_residuo != 0)
              {
                 c.selected = false;
                 c_alpha.push_back(c);
                 RESIDUE am = *rr;
                 am.num = Chains[ch].nAmino;
                 Chains[ch].AminoRes.push_back(am);
                 Chains[ch].nAmino++;
                 nResAmino++;
              }
          }
      }
      nResTot = 0;
      for(int ch=0; ch<nChains; ch++)
         nResTot += Chains[ch].nAmino;
      bool ifDetectSecondary = false;
      hasSecondary = true;
      nucleic = 0;
      AllCAlpha();
      CreaLegsBB();
      CreaLegsCA();
      CreaLegsLig();
      FillPhList();
      if(IsProtein())
      {
         ifDetectSecondary = true;
         for(int i=0; i<nats_au; i++)
         {
             if((Secondary[i] != ' ') && (Secondary[i] != 'L'))
                ifDetectSecondary = false;
         }
      }

      if((n_n <= 0 ) || (n_c <= 0) || (n_o <= 0) || nucleic)
          ifDetectSecondary = false;

      if(ifDetectSecondary)
      {
         if(!DetectSecondary(this, msg))
         {
            hasSecondary = false;
            return 0;
         }
      }
      else
      {
         for(int ch=0; ch<nChains; ch++)
         {
             for(int j=0; j< Chains[ch].nAmino; j++)
             {
                 RESIDUE *cur = &(Chains[ch].AminoRes[j]);
                 if(Is_DnaG(cur->code))
                 {
                    Secondary[cur->atoms[0]] = 'D';
                    cur->secondary = Secondary[cur->atoms[0]];
                 }
                 else
                    cur->secondary = Secondary[cur->atoms[0]];
             }
         }
      }
      for(int ch=0; ch<nChains; ch++)
      {
          for(int j=0; j< Chains[ch].nResidues; j++)
          {
              RESIDUE *cur = &(Chains[ch].Residues[j]);
              if(Is_DnaG(cur->code))
              {
                 Secondary[cur->atoms[0]] = 'D';
                 cur->secondary = Secondary[cur->atoms[0]];
              }
              else
                 cur->secondary = Secondary[cur->atoms[0]];
          }
      }

      QString os;
      PrintSecondary(this, os);
      FillSecondary(this);
      return  1;
}

bool Model::IsHetero(int num)
{
    int ieq = iEqui[num];
    return(atm_au[ieq].heta != 0);
}

void Model::FillPhList()
{
   for(int ch=0; ch<nChains; ch++)
   {
       if(!Chains[ch].isDNA)
          continue;
       for(int j=0; j< Chains[ch].nResidues; j++)
       {
           RESIDUE cur = Chains[ch].Residues[j];
           if(!Is_Acid(cur.code))
              continue;
           if(!IsPurine(cur.code))
              continue;
           int n1 = FindAtom("N1", cur);
           if(n1 == -1)
              continue;
           double distmax = 25.; //12.5;
           //int at=-1, res=-1;
           for(int ch2=0; ch2<nChains; ch2++)
           {
               int at=-1, res=-1;
               if((ch==ch2) && !Chains[ch].isRNA )
                   continue;
               if(!Chains[ch2].isDNA)
                   continue;
               for(int k=0; k< Chains[ch2].nResidues; k++)
               {
                   if((ch == ch2) && (j == k))
                       continue;
                   RESIDUE next = Chains[ch2].Residues[k];
                   if(!Is_Acid(next.code))
                       continue;
                   if(IsPair(cur.code, next.code))
                   {
                       int n3 = FindAtom("N3", next);
                       if(n3 == -1)
                          continue;
                       double dist = MRVectorDistance2(
                                     MRVectorMake(atm[n1].xyz),
                                     MRVectorMake(atm[n3].xyz));
                       if(dist >= distmax)
                          continue;
                       distmax = dist;
                       at = n3;
                       res = k;
                   }
               }
               if(at != -1)
               {
                  RESIDUE resi = Chains[ch2].Residues[res];
                  int ph1 = FindAtom("P", cur);
                  int ph2 = FindAtom("P", resi);
                  if(ph1 == -1)
                     ph1 = FindAtom("O5'", cur);
                  if(ph2 == -1)
                     ph2 = FindAtom("O5'", resi);
                  if((ph1 != -1) && (ph2 != -1))
                  {
                      phlist.push_back(ph1);
                      phlist.push_back(ph2);
                      nucleic = 1;
                  }
               }
           }
       }
    }
}

void Model::DrawSelected(QOpenGLFunctions *context)
{
  context->glDisable (GL_BLEND);
  if((plotStyle >= POLYPEPTIDE) && (nconn_prot == 0) && (nconn_ca != 0))
     plotStyle = C_ALPHA;

  if(plotStyle < RIBBONS)
  {
     SetLightForSelected(context, 1);
     DrawCyls(context, true);
     for(int i=0; i<nats;i++)
         atm[i].disegnato = 0;
     if(plotStyle >= POLYPEPTIDE)
        DrawCyls(context, true, true);
     else
        DrawSelectedBalls();
     ResetLightForSelected(context);
  }
  else
  {
      context->glDisable(GL_FOG);
      context->glDisable(GL_CLIP_PLANE0);
      context->glDisable(GL_CLIP_PLANE1);
      DrawRibbons(context, true);
  }

  if(Properties::Fog)
     context->glEnable(GL_FOG);
  if(Properties::Clip)
  {
     context->glEnable(GL_CLIP_PLANE0);
     context->glEnable(GL_CLIP_PLANE1);
  }
}

void Model::DrawSelectedBalls()
{
  int subdiv;

  float Scale = Properties::Scale;
  bool ifEll = (plotStyle == ELLIPSOIDS);
  float subd;
  if((plotStyle == STICK_AND_BALLS) || (plotStyle == ELLIPSOIDS))
     subd = (float) (float) (300. * maxRad[0]) * Properties::sc_factor;
  else if(plotStyle == VANDERWAALS)
     subd = (float) (float) (300. * maxRad[1]) * Properties::sc_factor;
  else
     subd = (float) (float) (300. * maxRad[2]) * Properties::sc_factor;
  if( (plotStyle >= STICK_AND_BALLS) && (plotStyle <= VANDERWAALS) )
     subd *= (float) Properties::sc_atoms;
  subdiv = (Scale > 1.)? ((int) (subd * Scale)) : ((int) subd);
  float add = (float)(0.2 / Properties::deltav);

  for(int i=0; i<nats; i++)
  {
      int z = atm[i].Z-1;
      if((z< 0) || (z > N_ELEMENTS))
         z = N_ELEMENTS;
      if(!atm[i].selected || atm[i].deleted || atm[i].hidden)
            continue;
      float radius;
      if(plotStyle == STICK_AND_BALLS)
         radius = (float) Properties::scale_atoms[z] * atm[i].radN[(Properties::RadType?3:0)];
      else if(plotStyle == ELLIPSOIDS)
         radius = (float) Properties::scale_atoms[z] * atm[i].radN[0];
      else if(plotStyle == VANDERWAALS)
         radius = (float) Properties::scale_atoms[z] * atm[i].radN[1];
      else
         radius = (float) Properties::sc_bonds * atm[i].radN[2];

      if(ifEll && !IsHydrogen(i))
      {
         float semi = atm[i].seminN * Properties::scale_atoms[z];
         renderEll(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                   semi+add, subdiv, atm[i].scala, atm[i].mat);
      }
      else
          renderSphere(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                   radius+add, subdiv);

  }
}

void Model::DrawLabels(QOpenGLFunctions *context, bool ifSingle)
{
  GLboolean val;
  context->glGetBooleanv(GL_LIGHTING, &val);
  context->glDisable (GL_LIGHTING);
  //glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

  float *model = new float [16]();
  float *proj = new float [16]();
  GLint view[4];
  GLdouble winx, winy, winz;
  GLdouble x, y, z;
  context->glGetFloatv(GL_MODELVIEW_MATRIX, model);
  context->glGetFloatv(GL_PROJECTION_MATRIX, proj);
  context->glGetIntegerv(GL_VIEWPORT, view);
  view[2] /= jav->devicePixelRatio();
  view[3] /= jav->devicePixelRatio();
  float molt = (float) (Properties::sc_atoms*Properties::Scale*Properties::sc_factor)/SCALA;
  if(plotStyle < POLYPEPTIDE)
  {
     int ind = 0;
     if(plotStyle == VANDERWAALS)
        ind = 1;

     for(int i=0; i<nats; i++)
     {
         if(ifSingle)
         {
            if(atm[i].drawLabel < 1)
               continue;
         }
         else
         {
            if(atm[i].deleted || (atm[i].hidden))
               continue;
            if(atm[i].drawLabel < 2)
            {
                if( (!ifDrawH || (specialLab == NONH)) &&  IsHydrogen(i) )
                   continue;
                if(((specialLab == HEAVY) &&  !IsHeavy(i)) )
                   continue;
                if((specialLab < 0) && (LabelS[atm[i].Z-1] == 0))
                   continue;
            }
            //if(((specialLab == SELECTED) &&  !atm[i].selected) )
            //   continue;
         }
// NEW
         FromCoordToWin(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                    model, proj, view, &winx, &winy, &winz);
         double delta = atm[i].radN[ind]*molt;
         if(plotStyle < RODS)
            delta = delta/4;
         double deltax = 0.1 * delta * Properties::xoffset * Properties::npixels / 2;
         double deltay = 0.1 * delta * Properties::yoffset * Properties::npixels / 2 -10;
         FromWinToCoord(winx+deltax, winy+deltay,
                winz-delta/10, model, proj, view, &x, &y, &z);
         DrawAtomLabel(atm[i], i, x, y, z);
     }
  }
  else
  {
     if(ifSingle || ifLabels || (specialLab == NONH))
     {
        for(int i=0; i<(int)c_alphab.size(); i++)
        {
            int ca = c_alphab[i].atom;
            if(atm[ca].deleted || atm[ca].hidden)
               continue;
            if(ifSingle)
            {
               if(atm[ca].drawLabel < 1)
                  continue;
            }
            FromCoordToWin(atm[ca].xyzN[0],atm[ca].xyzN[1], atm[ca].xyzN[2],
                    model, proj, view, &winx, &winy, &winz);
            double delta = 0.01*molt;
            FromWinToCoord(winx+delta*Properties::Width,
                winy+delta*Properties::Height,
                winz-delta, model, proj, view, &x, &y, &z);
            DrawAtomLabel(atm[ca], i, x, y, z, c_alphab[i].ResLab.c_str());
        }
     }
     if(drawLigand > 0)
        if(ifLabels || (specialLab != 0))
           DrawLigandsLabels(model, proj, view);
  }
  if(val == GL_TRUE)
     glEnable (GL_LIGHTING);
}


void Model::DrawLigandsLabels(float model[], float proj[], GLint view[])
{
     GLdouble winx, winy, winz;
     GLdouble x, y, z;

     float molt = (float) (Properties::sc_atoms*Properties::Scale*Properties::sc_factor)/SCALA;
     int ind = 0;
     for(int i=0; i<nats; i++)
     {
         if(!IsHetero(i))
             continue;
         if(atm[i].deleted || (atm[i].hidden))
             continue;
         if( (!ifDrawH || (specialLab == NONH)) &&  IsHydrogen(i) )
             continue;
         if(((specialLab == HEAVY) &&  !IsHeavy(i)) )
             continue;
         //if(((specialLab == SELECTED) &&  !atm[i].selected) )
         //    continue;
         if(atm[i].drawLabel < 2)
            continue;
         if((specialLab < 0) && (LabelS[atm[i].Z-1] == 0))
             continue;
         if( (!ifDrawW) &&  atm_au[iEqui[i]].isW )
            continue;
// NEW
         FromCoordToWin(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                    model, proj, view, &winx, &winy, &winz);
         double delta = atm[i].radN[ind]*molt;
         if(plotStyle < RODS)
            delta = delta/4;
         //PangoRectangle *rect1 = GetLabelRect(atm[i], i, font_desc);
         double deltax = delta * Properties::xoffset * Properties::npixels / 2;
         double deltay = delta * Properties::yoffset * Properties::npixels / 2; // - rect1->height/2;
         FromWinToCoord(winx+deltax, winy+deltay,
                winz-delta/10, model, proj, view, &x, &y, &z);
         DrawAtomLabel(atm[i], i, x, y, z);
     }
}

void Model::DrawAtomLabel(jav_atom_type at, int i, GLdouble x, GLdouble y,
                   GLdouble z, const char *LabelAt)
{

     stringstream str1;
     string str;
     if(LabelAt != NULL)
        str.assign(LabelAt);
     else
     {
        switch(labelType)
        {
        default:
        case LABEL_ATOM:
             str.assign(at.lab);
             break;
        case LABEL_SPECIES:
             str.assign(get_nth_element(at.Z-1));
             break;
        case LABEL_NUMBER:
             str1 << i+1;
             str.assign(str1.str());
             break;
        }
     }

     vet3 xy = {{x, y, z}};
     size_t found;
     found=str.find_last_not_of(" ");
     if (found!=std::string::npos)
       str.erase(found+1);
     else
       str.clear();
     jav->JavDrawText(xy, str.c_str(), labelcolor);
}

void Model::ToggleLabels(bool flag)
{
     if(flag != ifLabels)
     {
        jav->setLabelOnOff(flag);
        ifLabels = flag;
     }
}

void Model::SetLabelsSelected(bool stato)
{
    if(stato)
    {
        for(int i=0; i< nats; i++)
            if(atm[i].selected)
            {
               atm[i].selected = false;
               atm[i].drawLabel = 2;
            }
        for(int j=0; j<nconn_prot; j++)
            bonds[j].selected = false;
        atoms_sel.clear();
        bonds_sel.clear();
        resid_sel.clear();
    }
    else
        for(int i=0; i< nats; i++)
            atm[i].drawLabel = 0;
}

int Model::IsAtom(QOpenGLFunctions *f, int x, int y, int ind)
{
  float *model = new float[16]();
  float *proj = new float[16]();
  GLint view[4];
  GLdouble winx, winy, winz;
  GLdouble winx1, winy1, winz1;
  GLdouble xx = (GLdouble)x;
  GLdouble yy = (GLdouble)y;
  f->glGetFloatv(GL_MODELVIEW_MATRIX, model);
  f->glGetFloatv(GL_PROJECTION_MATRIX, proj);
  f->glGetIntegerv(GL_VIEWPORT, view);
  view[2] /= jav->devicePixelRatio();
  view[3] /= jav->devicePixelRatio();

  for(int i=0; i<nats; i++)
  {
     FromCoordToWin(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                    model, proj, view, &winx, &winy, &winz);
     FromCoordToWin(atm[i].xyzN[0]+atm[i].radN[ind],atm[i].xyzN[1]+atm[i].radN[ind], atm[i].xyzN[2]+atm[i].radN[ind],
                    model, proj, view, &winx1, &winy1, &winz1);
     GLdouble rad = sqrt((winx-winx1)*(winx-winx1)+
                     (winy-winy1)*(winy-winy1));

     if((abs(winx - xx) < rad) && (abs(winy - yy) < rad))
        return i;
  }
  return -1;
}

void Model::FillPreview(QTextStream &strm)
{
    for (int i=0; i<nats_au; i++){
        if (atm_au[i].color >= 0){
            strm << LabSpec[atm_au[i].color].leftJustified(4);
        }else{
            strm << "Q     ";
        }
        for (int j=0; j<3; j++){
            strm << QString::number(atm_au[i].xyz[j], 'f', 5).rightJustified(10) << "   1   ";
        }
        strm << "\n";
    }
}

void Model::FillPreview(QTextStream &strm, float I_coor[][3], int conn[][3])
{
    for (int i=0; i<nats_au; i++){
        if (atm_au[i].color >= 0){
            strm << LabSpec[atm_au[i].color].leftJustified(4);
        }else{
            strm << "Q     ";
        }
        if(i == 0)
        {
           for(int j=0; j<3; j++)
              strm << QString::number(0, 'f', 5).rightJustified(10) << "   1   ";
           strm << "0 0 0";
        }
        else if(i == 1)
        {
           strm << QString::number(I_coor[1][0], 'f', 5).rightJustified(10) << "   1   ";
           for(int j=0; j<2; j++)
              strm << QString::number(0, 'f', 5).rightJustified(10) << "   1   ";
           strm << conn[0][0];
           strm << " 0 0";
        }
        else if(i == 2)
        {
           strm << QString::number(I_coor[2][0], 'f', 5).rightJustified(10) << "   1   ";
           strm << QString::number(I_coor[2][1], 'f', 5).rightJustified(10) << "   1   ";
           strm << QString::number(0, 'f', 5).rightJustified(10) << "   1   ";
           strm << conn[2][0] << " ";
           strm << conn[2][1];
           strm << " 0";
        }
        else
        {
           if (I_coor[i][2]< 0)
               I_coor[i][2] = 360 + I_coor[i][2];
           strm << QString::number(I_coor[i][0], 'f', 5).rightJustified(10) << "   1   ";
           strm << QString::number(I_coor[i][1], 'f', 5).rightJustified(10) << "   1   ";
           strm << QString::number(I_coor[i][2], 'f', 5).rightJustified(10) << "   1   ";
           strm << conn[i][0] << " ";
           strm << conn[i][1] << " ";
           strm << conn[i][2];
        }
        strm << "\n";
    }
}

void Model::DrawAtoms(int j)
{
    int subdiv;
    subdiv = 8;

    glColor3f(1.0f, 1.0f, 1.0f);
    glPushName(0);
    glPushName(j);

    for(int i=0; i<nats; i++)
    {
        glPushName(i);
        renderSphere(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                     atm[i].radN[0], subdiv);
        glPopName();
    }
    glPopName();
    glPopName();
}

void Model::DrawBonds(QOpenGLFunctions *f, int j0)
{
    vet3 p1, p2;
    double radius_c = Properties::cylinder*Properties::sc_bonds;
    jav_bond_type *legami;

    int i0, i1;
    if(plotStyle < VANDERWAALS)
    {
       i0 = 0;
       i1 = nconn_prot;
       legami = bonds;
    }
    else if ( (plotStyle == POLYPEPTIDE) || (plotStyle == RIBBONS) )
    {
       i0 = 0;
       i1 = nconn_bb;
       legami = bonds_bb;
    }
    else if (plotStyle == C_ALPHA)
    {
       i0 = 0;
       i1 = nconn_ca;
       legami = bonds_ca;
    }
    else
       return;
    glColor3f(1.0f, 1.0f, 1.0f);
    glPushName(1);
    glPushName(j0);

    for(int i=i0; i<i1;i++)
    {
        int j1 = legami[i].n1 - 1;
        if(j1 >= nats)
            continue;
        if(j1 < 0)
           return;
         if(atm[j1].deleted || atm[j1].hidden)
            continue;
        int j2 = legami[i].n2 - 1;
        if(j2 >= nats)
            continue;
         if(atm[j2].deleted || atm[j2].hidden)
            continue;
         if( (!ifDrawH) && (IsHydrogen(j1) || IsHydrogen(j2)))
            continue;
        if( (!ifDrawW) && ( atm_au[iEqui[j1]].isW  || atm_au[iEqui[j2]].isW ) )
            continue;
        p1 =  MRVectorMake(atm[j1].xyzN);
        p2 =  MRVectorMake(atm[j2].xyzN);
        glPushName(i);
        //renderCylinder(f, p1, p2, radius_c);
        drawCylinder(p1, p2, (float) radius_c);
        glPopName();
    }
    glPopName();
    glPopName();
}

void Model::DrawBonds2(QOpenGLFunctions *f, int j0, bool ifLigands)
{
    vet3 p1, p2, p3;
    double radius_c = Properties::cylinder*Properties::sc_bonds;
    jav_bond_type *legami;

    int i0, i1, nomep = 1;
    if(ifLigands)
    {
       i0 = 0;
       i1 = nconn_lig;
       legami = bonds_lig;
       nomep = 4;
    }
    else if(plotStyle < VANDERWAALS)
    {
       i0 = 0;
       i1 = nconn_prot;
       legami = bonds;
    }
    else if ( (plotStyle == POLYPEPTIDE) || (plotStyle == RIBBONS))
    {
       i0 = 0;
       i1 = nconn_bb;
       legami = bonds_bb;
       nomep = 3;
    }
    else if (plotStyle == C_ALPHA)
    {
       i0 = 0;
       i1 = nconn_ca;
       legami = bonds_ca;
       nomep = 2;
    }
    else
       return;

    glColor3f(1.0f, 1.0f, 1.0f);
    glPushName(nomep);
    glPushName(j0);
    for(int i=i0; i<i1;i++)
    {
        int j1 = legami[i].n1 - 1;
        if(j1 >= nats)
            continue;
        if(j1 < 0)
           return;
         if(atm[j1].deleted || atm[j1].hidden)
            continue;
        int j2 = legami[i].n2 - 1;
        if(j2 >= nats)
            continue;
         if(atm[j2].deleted || atm[j2].hidden)
            continue;
         if( (!ifDrawH) && (IsHydrogen(j1) || IsHydrogen(j2)))
            continue;
        if( (!ifDrawW) && ( atm_au[iEqui[j1]].isW  || atm_au[iEqui[j2]].isW ) )
            continue;
        p1 =  MRVectorMake(atm[j1].xyzN);
        p2 =  MRVectorMake(atm[j2].xyzN);
        p3 = MRVectorMedium(p1, p2);
        glPushName(i);
        renderCylinder(f, p1, p3, radius_c);
        renderCylinder(f, p3, p2, radius_c);
        glPopName();
    }
    glPopName();
    glPopName();


}

void Model::DrawTubes(QOpenGLFunctions *f, int npoints, Knot knot[], float color[], double factor)
{
     for(int i=0; i<npoints-1; i++)
     {
          vet3 x1 = {{ knot[i].px, knot[i].py, knot[i].pz}};
          vet3 x2 = {{ knot[i+1].px, knot[i+1].py, knot[i+1].pz}};
          renderCylinder(f, x1, x2, factor*1.5*Properties::cylinder,color);
     }
}

void Model::DrawStrands(int npoints, Knot knot[], float color[])
{
   glColor4fv(color);
   float c[NKNOTS][5][3];

   for(int k=-1; k<4; k++)
   {
      glBegin(GL_LINE_STRIP);
      for(int i=0; i<npoints; i++)
      {
          if(k == -1)
          {
          c[i][k+1][0] = knot[i].px;
          c[i][k+1][1] = knot[i].py;
          c[i][k+1][2] = knot[i].pz;
          }
          else if(k == 0)
          {
          c[i][k+1][0] = knot[i].px - knot[i].wx;
          c[i][k+1][1] = knot[i].py - knot[i].wy;
          c[i][k+1][2] = knot[i].pz - knot[i].wz;
          }
          else if(k == 1)
          {
          c[i][k+1][0] = knot[i].px - knot[i].wx/2;
          c[i][k+1][1] = knot[i].py - knot[i].wy/2;
          c[i][k+1][2] = knot[i].pz - knot[i].wz/2;
          }
          else if(k == 2)
          {
          c[i][k+1][0] = knot[i].px + knot[i].wx/2;
          c[i][k+1][1] = knot[i].py + knot[i].wy/2;
          c[i][k+1][2] = knot[i].pz + knot[i].wz/2;
          }
          else
          {
          c[i][k+1][0] = knot[i].px + knot[i].wx;
          c[i][k+1][1] = knot[i].py + knot[i].wy;
          c[i][k+1][2] = knot[i].pz + knot[i].wz;
          }
          glVertex3f(c[i][k+1][0], c[i][k+1][1], c[i][k+1][2]);
      }
      glEnd();
   }
}

void Model::DrawSolidRibbon(Knot src, Knot dst, float colore[])
{
    Vert v[4];

    for(int i=0; i<4; i++)
        memcpy(v[i].c, colore, 4*sizeof(float));
    TuneHSV( v[0].c, src.vinten);
    TuneHSV( v[1].c, dst.vinten);
    TuneHSV( v[2].c, dst.vinten);
    TuneHSV( v[3].c, src.vinten);
// Top Surface
    v[0].x = src.px+src.wx+src.dx;
    v[0].y = src.py+src.wy+src.dy;
    v[0].z = src.pz+src.wz+src.dz;

    v[1].x = dst.px+dst.wx+dst.dx;
    v[1].y = dst.py+dst.wy+dst.dy;
    v[1].z = dst.pz+dst.wz+dst.dz;

    v[2].x = dst.px-dst.wx+dst.dx;
    v[2].y = dst.py-dst.wy+dst.dy;
    v[2].z = dst.pz-dst.wz+dst.dz;

    v[3].x = src.px-src.wx+src.dx;
    v[3].y = src.py-src.wy+src.dy;
    v[3].z = src.pz-src.wz+src.dz;
    DrawPolygon(v);

// Bottom Surface
    v[0].x = src.px+src.wx-src.dx;
    v[0].y = src.py+src.wy-src.dy;
    v[0].z = src.pz+src.wz-src.dz;

    v[1].x = dst.px+dst.wx-dst.dx;
    v[1].y = dst.py+dst.wy-dst.dy;
    v[1].z = dst.pz+dst.wz-dst.dz;

    v[2].x = dst.px-dst.wx-dst.dx;
    v[2].y = dst.py-dst.wy-dst.dy;
    v[2].z = dst.pz-dst.wz-dst.dz;

    v[3].x = src.px-src.wx-src.dx;
    v[3].y = src.py-src.wy-src.dy;
    v[3].z = src.pz-src.wz-src.dz;
    DrawPolygon(v);

    for(int i=0; i<4; i++)
        memcpy(v[i].c, colore, 4*sizeof(float));
    TuneHSV( v[0].c, src.hinten);
    TuneHSV( v[1].c, dst.hinten);
    TuneHSV( v[2].c, dst.hinten);
    TuneHSV( v[3].c, src.hinten);

// Left Surface
    v[0].x = src.px+src.wx+src.dx;
    v[0].y = src.py+src.wy+src.dy;
    v[0].z = src.pz+src.wz+src.dz;

    v[1].x = dst.px+dst.wx+dst.dx;
    v[1].y = dst.py+dst.wy+dst.dy;
    v[1].z = dst.pz+dst.wz+dst.dz;

    v[2].x = dst.px+dst.wx-dst.dx;
    v[2].y = dst.py+dst.wy-dst.dy;
    v[2].z = dst.pz+dst.wz-dst.dz;

    v[3].x = src.px+src.wx-src.dx;
    v[3].y = src.py+src.wy-src.dy;
    v[3].z = src.pz+src.wz-src.dz;
    DrawPolygon(v);

// Right Surface
    v[0].x = src.px-src.wx+src.dx;
    v[0].y = src.py-src.wy+src.dy;
    v[0].z = src.pz-src.wz+src.dz;

    v[1].x = dst.px-dst.wx+dst.dx;
    v[1].y = dst.py-dst.wy+dst.dy;
    v[1].z = dst.pz-dst.wz+dst.dz;

    v[2].x = dst.px-dst.wx-dst.dx;
    v[2].y = dst.py-dst.wy-dst.dy;
    v[2].z = dst.pz-dst.wz-dst.dz;

    v[3].x = src.px-src.wx-src.dx;
    v[3].y = src.py-src.wy-src.dy;
    v[3].z = src.pz-src.wz-src.dz;
    DrawPolygon(v);
}

void Model::DrawRibbon(QOpenGLFunctions *f, int kc,  Knot knot[], int npoints, bool selected)
{
    float color1[4];
//    float colors[4] = {1.f, 0.6f, 1.f, 0.85f};
    float colors[4] = {1.f, 0.85f, 1.f, 0.65f};
    int ca = c_alphab[kc].atom;

    f->glDisable(GL_LIGHTING);
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    f->glEnable(GL_COLOR_MATERIAL);
    f->glEnable (GL_BLEND);
    (void) SetColor(ca, color1);
    if(selected && c_alphab[kc].selected)
    {
//       glBlendFunc (GL_SRC_ALPHA,  GL_SRC_ALPHA);
       memcpy(color1, colors, 4*sizeof(float));
       glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;
    }
    else if(selected)
       return;
    f->glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    if(Properties::ifDesaturate())
       Desaturate(color1);

    int plotStyleSav = plotStyle;
    if(selected)
       plotStyle = RIBBONS;
    if(plotStyle == RIBBONS)
    {
       for(int kk=0; kk<npoints-1; kk++)
//240112           DrawFlatRibbon(gl, knot[kk], knot[kk+1], color1);
           DrawSolidRibbon(knot[kk], knot[kk+1], color1);
    }
    else if(plotStyle == STRANDS)
        DrawStrands(npoints, knot, color1);
    else
    {
        if(IsBetaSheet(ca))
        {
            for(int kk=0; kk<npoints-1; kk++)
                DrawSolidRibbon(knot[kk], knot[kk+1], color1);
        }
        else if(IsHelix(ca))
        {
           for(int kk=0; kk<npoints-1; kk++)
              DrawSolidRibbon(knot[kk], knot[kk+1], color1);
        }
        else if(IsDna(ca))
           DrawTubes(f, npoints, knot, color1, 3);
        else
           DrawTubes(f, npoints, knot, color1);
    }
    plotStyle = plotStyleSav;
}


void Model::DrawRibbons (QOpenGLFunctions *f, bool selected, int kmodel)
{
    Knot knot[NKNOTS];
    float wide, hwide, size, kf = 1.5f;
    float ca[3], canext[3], o[3], o2[3];
    bool prev = false;
    bool ifArrow;
    if(kmodel != -1)
    {
       int nomep = 5;
       glPushName(nomep);
       glPushName(kmodel);
    }

    memset(knot, 0, nknots*sizeof(Knot));
    int chain = -1;
    for(int i=0; i<(int)c_alphab.size(); i++)
    {
        if(selected && !c_alphab[i].selected)
           continue;
        if(kmodel != -1)
           glPushName(i);
// Defining peptide planes
        if(!prev)
           memset(knot, 0, nknots*sizeof(Knot));

        if(c_alphab[i].chain != chain)
        {
           prev = false;
           chain = c_alphab[i].chain;
           memset(knot, 0, nknots*sizeof(Knot));
        }
        if(c_alphab[i].o == -1)
        {
           prev = false;
           if(kmodel != -1)
              glPopName();
           continue;
        }
 
        for(int j=0; j<3; j++)
        {
            ca[j] = atm[c_alphab[i].atom].xyzN[j];
            if(c_alphab[i].isAmino)
               o[j] = atm[c_alphab[i].o].xyzN[j];
            else if(c_alphab[i].n == -1) // deoxyribonucleic acid
            {
               if(!c_alphab[i].breakF)
               {
                  o[j] = atm[c_alphab[i+1].o].xyzN[j];
                  if(c_alphab[i].o2 != -1)
                     o2[j] = atm[c_alphab[i].o2].xyzN[j];
               }
               else
                  o[j] = atm[c_alphab[i].o].xyzN[j];
            }
            else                          // ribonucleic acid
               o[j] = atm[c_alphab[i].o].xyzN[j];
            if(c_alphab[i].breakF)
               continue;
            canext[j] = atm[c_alphab[i+1].atom].xyzN[j];
        }
        if(c_alphab[i].breakF)            // siamo alla fine
        {
           if( IsBetaSheet(c_alphab[i].atom) && (plotStyle == CARTOONS))
           {
               wide = (float)(12*Properties::cylinder);
               knot[nknots-1].px = ca[0] + (knot[nknots-1].tx/2);
               knot[nknots-1].py = ca[1] + (knot[nknots-1].ty/2);
               knot[nknots-1].pz = ca[2] + (knot[nknots-1].tz/2);
               ifArrow = true;
           } 
           else
           {
               wide = (float)(6*Properties::cylinder);
               if(selected)
                  wide = (float)(8*Properties::cylinder);
               knot[nknots-1].px = ca[0];
               knot[nknots-1].py = ca[1];
               knot[nknots-1].pz = ca[2];
               ifArrow = false;
           }
           if( (knot[0].wide!=wide) && knot[0].hsize )
           {   
               size = knot[0].hsize;
               knot[0].wx = (wide*knot[0].hnx)/size;
               knot[0].wy = (wide*knot[0].hny)/size;
               knot[0].wz = (wide*knot[0].hnz)/size;

               if( !ifArrow )
               {   knot[nknots-1].wx = knot[0].wx;
                   knot[nknots-1].wy = knot[0].wy;
                   knot[nknots-1].wz = knot[0].wz;
               } 
               else
               {   
                   knot[nknots-1].wx = 0;
                   knot[nknots-1].wy = 0;
                   knot[nknots-1].wz = 0;
               }
            } 
            else if( ifArrow )
            {   
                wide = (float)(12*Properties::cylinder);
                knot[nknots-1].wx = 0;
                knot[nknots-1].wy = 0;
                knot[nknots-1].wz = 0;
            }
            knot[1] = knot[nknots-1];
            if(!prev)
            {
            knot[0] = knot[1];
            knot[0].px = o[0];
            knot[0].py = o[1];
            knot[0].pz = o[2];
            }
            prev = false;
            DrawRibbon(f, i, knot, 2, selected);
            c_alphab[i].xyz[0] = knot[0].px;
            c_alphab[i].xyz[1] = knot[0].py;
            c_alphab[i].xyz[2] = knot[0].pz;
            if((IsDna(c_alphab[i].atom))  && (plotStyle == CARTOONS))
            {
               float color1[4];
               SetColor(c_alphab[i].atom, color1);
               TuneHSV( color1, knot[1].vinten);
               drawSphere(knot[1].px, knot[1].py, knot[1].pz, 4.5f*(float)Properties::cylinder,
                         color1, f);
            }
            if(kmodel != -1)
               glPopName();
            continue;
        }
// Get vector A = Ca(i+l) - Ca(i)
        knot[nknots-1].tx = canext[0] - ca[0];
        knot[nknots-1].ty = canext[1] - ca[1];
        knot[nknots-1].tz = canext[2] - ca[2];
// Get vector B = O(i) - Ca(i) 
        float bx, by, bz;
        if(!c_alphab[i].isAmino && (c_alphab[i].o2 != -1) && (c_alphab[i].n == -1))
        {
           bx = (o[0]+o2[0])/2 - ca[0];
           by = (o[1]+o2[0])/2 - ca[1];
           bz = (o[2]+o2[0])/2 - ca[2];
        }
        else
        {
           bx = o[0] - ca[0];
           by = o[1] - ca[1];
           bz = o[2] - ca[2];
        }

// Form point P as the midpoint of Ca(i) and Ca(i+1)
        knot[nknots-1].px = (ca[0] + canext[0])/2;
        knot[nknots-1].py = (ca[1] + canext[1])/2;
        knot[nknots-1].pz = (ca[2] + canext[2])/2;

// Form vector C = A x B
// Vector C is normal to the peptide plane approximated by atoms Ca(i), O(i), and Ca(i+1
        knot[nknots-1].vnx = knot[nknots-1].ty*bz - knot[nknots-1].tz*by;
        knot[nknots-1].vny = knot[nknots-1].tz*bx - knot[nknots-1].tx*bz;
        knot[nknots-1].vnz = knot[nknots-1].tx*by - knot[nknots-1].ty*bx;

// If the peptide plane is part of a helix, translate in the direction of vector C, 
// away from the helix axis
        if(IsHelix(c_alphab[i].atom) && IsHelix(c_alphab[i+1].atom))
        {
           size = sqrt(knot[nknots-1].vnx*knot[nknots-1].vnx + 
                       knot[nknots-1].vny*knot[nknots-1].vny + 
                       knot[nknots-1].vnz*knot[nknots-1].vnz);
        
           knot[nknots-1].vsize = size;
           if( size )
           {   // 1.00 Angstrom Displacement
               wide = 0.7f/Properties::deltav;
               knot[nknots-1].px += ((wide*knot[nknots-1].vnx)/size);
               knot[nknots-1].py += ((wide*knot[nknots-1].vny)/size);
               knot[nknots-1].pz += ((wide*knot[nknots-1].vnz)/size);
           } 
        }
        else 
            knot[nknots-1].vsize = 0;

// Form vector D = C x A
// Vector D lies parallel to the peptide plane and is perpendicular to vector A. 
        float dx = knot[nknots-1].vny*knot[nknots-1].tz - knot[nknots-1].vnz*knot[nknots-1].ty;
        float dy = knot[nknots-1].vnz*knot[nknots-1].tx - knot[nknots-1].vnx*knot[nknots-1].tz;
        float dz = knot[nknots-1].vnx*knot[nknots-1].ty - knot[nknots-1].vny*knot[nknots-1].tx;

// Scale vector D by one half the desired ribbon width
        dx /= (float)(3*Properties::cylinder);
        dy /= (float)(3*Properties::cylinder);
        dz /= (float)(3*Properties::cylinder);
        knot[nknots-1].hsize = sqrt(dx*dx + dy*dy + dz*dz);

// Handle Carbonyl Oxygen Flip 
        if( prev && ((knot[0].hnx*dx + knot[0].hny*dy + knot[0].hnz*dz)<0) )
        {
            knot[nknots-1].hnx = -dx;   knot[nknots-1].vnx = -knot[nknots-1].vnx;
            knot[nknots-1].hny = -dy;   knot[nknots-1].vny = -knot[nknots-1].vny;
            knot[nknots-1].hnz = -dz;   knot[nknots-1].vnz = -knot[nknots-1].vnz;
        }
        else
        {
            knot[nknots-1].hnx = dx;
            knot[nknots-1].hny = dy;
            knot[nknots-1].hnz = dz;
        }

//  Set Ribbon Width 
        wide = (float)(6*Properties::cylinder);
        if(selected)
           wide = (float)(8*Properties::cylinder);

        ifArrow = false;
        if(plotStyle == CARTOONS)
        {
           if(IsBetaSheet(c_alphab[i].atom))
           {
              if(!IsBetaSheet(c_alphab[i+1].atom))
              {
                 ifArrow = true;
                 wide = (float)(12*Properties::cylinder);
              }
           }
           else if(!IsHelix(c_alphab[i].atom))
                wide = (float)(kf*Properties::cylinder);
        }
        if( knot[nknots-1].hsize && !ifArrow)
        {
            size = knot[nknots-1].hsize;
            knot[nknots-1].wx = (wide*knot[nknots-1].hnx)/size;
            knot[nknots-1].wy = (wide*knot[nknots-1].hny)/size;
            knot[nknots-1].wz = (wide*knot[nknots-1].hnz)/size;
            knot[nknots-1].wide = wide;
        }
        else
        {
            knot[nknots-1].wide = 0;
            knot[nknots-1].wx = 0;
            knot[nknots-1].wy = 0;
            knot[nknots-1].wz = 0;
        }

        if(prev && knot[0].hsize)
        {
           size = knot[0].hsize;
           knot[0].wx = (float)(wide*knot[0].hnx)/size;
           knot[0].wy = (float)(wide*knot[0].hny)/size;
           knot[0].wz = (float)(wide*knot[0].hnz)/size;
        }
        hwide = (float)(kf*Properties::cylinder);
//240112        if(IsHelix(c_alphab[i].atom))
        if(IsHelix(c_alphab[i].atom) || (plotStyle == RIBBONS))
           hwide /= 3.;
        if(prev && knot[0].vsize)
        {
           size = knot[0].vsize;
           knot[0].dx = (float)(hwide*knot[0].vnx)/size;
           knot[0].dy = (float)(hwide*knot[0].vny)/size;
           knot[0].dz = (float)(hwide*knot[0].vnz)/size;
        }
        GetSizeandInten(&knot[nknots-1]);

        size = knot[nknots-1].vsize;
        knot[nknots-1].dx = (float)(hwide)*knot[nknots-1].vnx/size;
        knot[nknots-1].dy = (float)(hwide)*knot[nknots-1].vny/size;
        knot[nknots-1].dz = (float)(hwide)*knot[nknots-1].vnz/size;

        if(prev)
        {
           for(int k=0; k<nknots-2; k++)
           {
// Calculate Hermite Spline Points 
               knot[k+1].px = H[0][k]*knot[0].px + H[1][k]*knot[0].tx +
                              H[2][k]*knot[nknots-1].px + H[3][k]*knot[nknots-1].tx;
               knot[k+1].py = H[0][k]*knot[0].py + H[1][k]*knot[0].ty +
                              H[2][k]*knot[nknots-1].py + H[3][k]*knot[nknots-1].ty;
               knot[k+1].pz = H[0][k]*knot[0].pz + H[1][k]*knot[0].tz +
                              H[2][k]*knot[nknots-1].pz + H[3][k]*knot[nknots-1].tz;

//  Calculate Hermite Spline Widths 
               knot[k+1].wx = H[0][k]*knot[0].wx + H[2][k]*knot[nknots-1].wx;
               knot[k+1].wy = H[0][k]*knot[0].wy + H[2][k]*knot[nknots-1].wy;
               knot[k+1].wz = H[0][k]*knot[0].wz + H[2][k]*knot[nknots-1].wz;

// Calculate Surface Intensity 
               knot[k+1].vsize = 0;
               knot[k+1].vnx = H[0][k]*knot[0].vnx + H[2][k]*knot[nknots-1].vnx;
               knot[k+1].vny = H[0][k]*knot[0].vny + H[2][k]*knot[nknots-1].vny;
               knot[k+1].vnz = H[0][k]*knot[0].vnz + H[2][k]*knot[nknots-1].vnz;
               GetSizeandInten(&knot[k+1]);

//240112               if(plotStyle == CARTOONS)
               {
// Calculate Spline Heights 
                  knot[k+1].dx =  (float)(hwide*knot[k+1].vnx/knot[k+1].vsize); 
                  knot[k+1].dy =  (float)(hwide*knot[k+1].vny/knot[k+1].vsize);
                  knot[k+1].dz =  (float)(hwide*knot[k+1].vnz/knot[k+1].vsize);
// Calculate Surface Intensity 
                  knot[k+1].hsize = 0;
                  knot[k+1].hnx = H[0][k]*knot[0].hnx + H[2][k]*knot[nknots-1].hnx;
                  knot[k+1].hny = H[0][k]*knot[0].hny + H[2][k]*knot[nknots-1].hny;
                  knot[k+1].hnz = H[0][k]*knot[0].hnz + H[2][k]*knot[nknots-1].hnz;
                  GetSizeandInten(&knot[k+1]);
               }
           }

           DrawRibbon(f, i, knot, nknots, selected);
           knot[0] = knot[nknots-1];
            c_alphab[i].xyz[0] = knot[4].px;
            c_alphab[i].xyz[1] = knot[4].py;
            c_alphab[i].xyz[2] = knot[4].pz;
        }
        else                     // siamo all'inizio
        {
           knot[1] = knot[nknots-1];
           knot[0] = knot[nknots-1];
           knot[0].px = ca[0];
           knot[0].py = ca[1];
           knot[0].pz = ca[2];
           knot[0].hnx = dx;
           knot[0].hny = dy;
           knot[0].hnz = dz;
           c_alphab[i].xyz[0] = knot[0].px;
           c_alphab[i].xyz[1] = knot[0].py;
           c_alphab[i].xyz[2] = knot[0].pz;
           if(kmodel != -1)
              DrawRibbon(f, i, knot, 2, selected);
           if((IsDna(c_alphab[i].atom)) && (plotStyle == CARTOONS))
           {
               float color1[4];
               SetColor(c_alphab[i].atom, color1);
               TuneHSV( color1, knot[0].vinten);
               drawSphere(knot[0].px, knot[0].py, knot[0].pz, 4.5f*(float)Properties::cylinder,
                         color1, f);
           }

        }
        prev = true;
        if(kmodel != -1)
           glPopName();
    }
    if(kmodel != -1)
    {
       glPopName();
       glPopName();
    }
}

void Model::UpdateSelectedAtom(size_t whichAtom, bool to_add)
{
     bool stato = atm[whichAtom].selected;
     atm[whichAtom].selected = !stato;
     if(stato && !to_add)
     {
        std::vector<int>::iterator it;
        it = std::find(atoms_sel.begin(), atoms_sel.end(), (whichAtom+1));
        if(it != atoms_sel.end())
          atoms_sel.erase(it);
     }
     else
        atoms_sel.push_back(whichAtom+1);
}

void Model::UpdateSelectedResidue(size_t whichAlpha, bool to_add)
{
     bool stato = c_alphab[whichAlpha].selected;
     c_alphab[whichAlpha].selected = !stato;
     if(stato && !to_add)
     {
        std::vector<int>::iterator it;
        it = std::find(resid_sel.begin(), resid_sel.end(), (whichAlpha+1));
        if(it != resid_sel.end())
          resid_sel.erase(it);
     }
     else
        resid_sel.push_back(static_cast<int>(whichAlpha)+1);
}

void Model::UpdateSelectedBond(size_t whichBond, int whichtype)
{
     int wb = static_cast<int>(whichBond);
     bool stato;
     switch(whichtype)
     {
     default:
     case 1:
          stato = bonds[wb].selected;
          bonds[wb].selected = !stato;
          if(stato)
          {
             std::vector<int>::iterator it;
             it = std::find(bonds_sel.begin(), bonds_sel.end(), (wb+1));
             if(it != bonds_sel.end())
               bonds_sel.erase(it);
          }
          else
             bonds_sel.push_back(wb+1);
          break;
     case 2:
          break;
     case 3:
          break;
     case 4:
          break;
     }
}

void Model::DrawSticks(QOpenGLFunctions *f)
{
     DrawSticks(f, 0, nconn_prot, bonds, false, NULL);
}

void Model::DrawSticks(QOpenGLFunctions *f, int i0, int i1, jav_bond_type *legami,
                       bool iflig, float *colore)
{
      int i;
      float color1[4];
      float color2[4];

      glPushMatrix();
      f->glDisable (GL_LIGHTING);
      if(Properties::smooth)
      {
         f->glEnable (GL_LINE_SMOOTH);
         f->glEnable (GL_BLEND);
         f->glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
         f->glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
      }
      for(i=0; i<nats; i++)
          atm[i].disegnato = !iflig && (plotStyle >= RODS);     //!= WIRES);
      if((plotStyle == BONDSTYLE) || (iflig && (drawLigand - 1) == BONDSTYLE))
         f-> glLineWidth(5) ;
      else
         f->glLineWidth(1.5f) ;

      glBegin(GL_LINES);
      for(i=i0; i<i1;i++)
      {
         float p1[3], p2[3], p3[3];
         int j1 = abs(legami[i].n1) - 1;
         if(j1 >= nats)
            continue;
         if(j1 < 0)
            continue;
         if(atm[j1].deleted || atm[j1].hidden)
            continue;
         int j2 = abs(legami[i].n2) - 1;
         if(j2 >= nats)
            continue;
         if(atm[j2].deleted || atm[j2].hidden)
            continue;
         if( (!ifDrawH) && (IsHydrogen(j1) || IsHydrogen(j2)))
            continue;
         if( (!ifDrawW) && ( atm_au[iEqui[j1]].isW  || atm_au[iEqui[j2]].isW ) )
            continue;
         for(int j=0; j<3; j++)
         {
             p1[j] =  atm[j1].xyzN[j];
             p2[j] =  atm[j2].xyzN[j];
         }
         atm[j1].disegnato = 1;
         atm[j2].disegnato = 1;
         int index1=0, index2=0;
         if(colore == NULL)
         {
            index1 = SetColor(j1, color1);
            index2 = SetColor(j2, color2);
         }
         else
         {
            memcpy(color1, colore, 4*sizeof(float));
            memcpy(color2, colore, 4*sizeof(float));
         }
         if(Properties::ifDesaturate())
         {
            Desaturate(color1);
            Desaturate(color2);
         }
         glColor4fv(color1);
         glVertex3f(p1[0], p1[1], p1[2]);
         if(index1 != index2)
         {
            for(int j=0; j<3; j++)
                p3[j] =  (p1[j] + p2[j]) / 2.f;
            glVertex3f(p3[0], p3[1], p3[2]);
            glColor4fv(color2);
            glVertex3f(p3[0], p3[1], p3[2]);
         }
         glVertex3f(p2[0], p2[1], p2[2]);
      }
      glEnd();
      f->glLineWidth(1.5f) ;
      f->glEnable (GL_LIGHTING);
      glPopMatrix();
}

void Model::DrawSBonds(QOpenGLFunctions *f, int nlegs, jav_bond_type *legs, int tipo)
{
      float colorMix[] = {0.3f, 0.3f, 1.f, 1.f};
      if(!nlegs)
         return;

      glLineStipple(1, 0x00FF);
      f->glEnable(GL_LINE_STIPPLE);
      float colore[4];
      if(tipo == 'H')
         FromQColor(Properties::ColorHB, colore);
      else if(tipo == 'W')
         FromQColor(Properties::ColorSC, colore);
      else
         memcpy(colore, colorMix, 4*sizeof(float));
     DrawSticks(f, 0, nlegs, legs, false, colore);
     f->glDisable(GL_LINE_STIPPLE);
}

void Model::DrawCircles(int ind)
{
      int i;
      float color1[4];

      glPushMatrix();

      int subd = (int)(Properties::Scale*Properties::sc_factor);
      if(subd <= 1)
         subd = 2;
      else if(subd > 20)
         subd = 20;
      for(i=0; i<nats; i++)
      {
         if( (!ifDrawH) &&  IsHydrogen(i) )
            continue;
         if( (!ifDrawW) &&  atm_au[iEqui[i]].isW )
            continue;
         if( ind && atm[i].disegnato)
            continue;
         if(atm[i].deleted || atm[i].hidden)
            continue;
         if(IsPeak(i))
            continue;
         (void) SetColor(i, color1);
         if(Properties::ifDesaturate())
            Desaturate(color1);
         glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
         glLoadIdentity();
         double Scale =  Properties::Scale;
         glTranslated( Properties::obj_pos.x, Properties::obj_pos.y, -Properties::obj_pos.z );
         glScaled( Scale, Scale, Scale);
         mat4x4 Rotm = jav->GetRotm();
         glMultMatrixd (Rotm.m);
         glTranslatef(atm[i].xyzN[0], atm[i].xyzN[1], atm[i].xyzN[2]);
         glCallList (subd);
//         glCallList (1);
      }
      glPopMatrix();
}

int Model::FindStyle(int whichPol)
{
  int vert1 = iEqui[Polyhedra[whichPol].xc];
  for(int i=0; i<nPolyStyles; i++)
  {
      if(PolyStyle[i].vert == vert1)
         return i;
  }
  return -1;
}

void DrawPolyhedron(Vert *points, Vert *norms, int nfaces, int *nlati)
{
  int p = 0;
  for(int i=0; i<nfaces; i++)
  {
    glBegin(GL_POLYGON);
    for(int k=0; k<nlati[i];k++)
    {
          glNormal3f(norms[p].x, norms[p].y, norms[p].z);
          glVertex3f(points[p].x, points[p].y, points[p].z);
          p++;
    }
    glEnd();
  }
}

void DrawPolyhedron_1n(Vert *points, Vert *norms, int nfaces, int *nlati)
{
  int p = 0;
  for(int i=0; i<nfaces; i++)
  {
      Vert n = norms[i];
      glBegin(GL_POLYGON);
      glNormal3f(n.x, n.y, n.z);
      for(int k=0; k<nlati[i];k++)
      {
          glVertex3f(points[p].x, points[p].y, points[p].z);
          p++;
      }
      glEnd();
  }
}


void Model::DrawPolyhedra(QOpenGLFunctions *f)
{
    if(Polyhedra == NULL)
       return;
    for (int i=0; i<nPoly; i++)
    {
         int ind = FindStyle(i);
         if(ind == -1)
            continue;
         if(!PolyStyle[ind].visible)
            continue;
         f->glEnable (GL_LIGHTING);
         f->glEnable (GL_NORMALIZE);
         int nf = NumberOfFaces(Polyhedra[i]);
         int *nlati =  new int [nf]();
         int np = 0;
         for(int j=0; j<nf; j++)
         {
             nlati[j] = Polyhedra[i].face[j].n;
             np += nlati[j];
         }
         Vert *points =  new Vert [np]();
         Vert *norms;
         if(Properties::normalToV)
            norms  =  new Vert [np]();
         else
            norms  =  new Vert [nf]();
         QColor color;
         color = PolyStyle[ind].colore;
         int jj = 0;
         for(int j=0; j<nf; j++)
         {
             for(int k=0; k<Polyhedra[i].face[j].n; k++)
             {
                 points[jj].x = atm[Polyhedra[i].face[j].fp[k]].xyzN[0];
                 points[jj].y = atm[Polyhedra[i].face[j].fp[k]].xyzN[1];
                 points[jj].z = atm[Polyhedra[i].face[j].fp[k]].xyzN[2];
                 if(Properties::normalToV)
                 {
                     norms[jj].x = atm[Polyhedra[i].face[j].fp[k]].vn[0];
                     norms[jj].y = atm[Polyhedra[i].face[j].fp[k]].vn[1];
                     norms[jj].z = atm[Polyhedra[i].face[j].fp[k]].vn[2];
                 }
                 jj++;
             }
             if(!Properties::normalToV)
                 norms[j] = norm(points[jj-3], points[jj-2], points[jj-1]);
         }
         f->glEnable (GL_BLEND);
         f->glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
         if(PolyStyle[ind].transparency)
         {
            f->glDepthMask (GL_FALSE);
            color.setAlphaF((float) (100 - PolyStyle[ind].transparency)/100.f);
         }
         else
            f->glDepthMask (GL_TRUE);
         float c1[4];
         FromQColor(color,c1);
         glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, c1);
         glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;
         if(Properties::normalToV)
            DrawPolyhedron(points, norms, nf, nlati);
         else
            DrawPolyhedron_1n(points, norms, nf, nlati);

         if(PolyStyle[ind].ifBorder)
         {
            color = PolyStyle[ind].Bcolore;
            float c1[4];
            FromQColor(color,c1);
            f->glLineWidth(PolyStyle[ind].line_width) ;
            f->glEnable (GL_LINE_SMOOTH);
            f->glDisable (GL_LIGHTING);
            glPolygonMode( GL_FRONT_AND_BACK, GL_LINE ) ;
            glColor4fv(c1);
            if(Properties::normalToV)
               DrawPolyhedron(points, norms, nf, nlati);
            else
               DrawPolyhedron_1n(points, norms, nf, nlati);
         }
         delete [] nlati;
         delete [] points;
         delete [] norms;
    }
    f->glEnable (GL_LIGHTING);
    f->glDepthMask (GL_TRUE);
}

void Model::FillPolyhedra(int npoly, polyhedron_type *poly)
{
   if(Polyhedra != NULL)
      delete [] Polyhedra;
   Polyhedra = NULL;
   nPoly = npoly;
   if(nPoly == 0)
      return;
   Polyhedra = new polyhedron_type [nPoly]();
   for (int i=0; i<nPoly; i++)
   {
        Polyhedra[i].nvert = poly[i].nvert;
        Polyhedra[i].xc = poly[i].xc-1;
//        g_print("Poliedro n.%d vertici %d\n", i+1, Polyhedra[i].nvert);
        int nf = NumberOfFaces(Polyhedra[i]);
        for(int j=0; j<nf; j++)
        {
           Polyhedra[i].face[j].n =  poly[i].face[j].n;
//           g_print("Poliedro n.%d faccia %d vertici %d\n", i+1, j+1, Polyhedra[i].face[j].n);
           for(int k=0; k<Polyhedra[i].face[j].n; k++)
               Polyhedra[i].face[j].fp[k] = poly[i].face[j].fp[k]-1;
        }
   }
   if(Properties::normalToV)
      CalcolaNormali();
}

void Model::FillPolyhedraStyles()
{
   if(nPoly == 0)
      return;
   std::vector<polyhedron_style> PS;
   std::vector<polyhedron_style>::iterator it;
   PS.clear();
   for (int i=0; i<nPolyStyles; i++)
        PS.push_back(PolyStyle[i]);

   for (int i=0; i<nPoly; i++)
   {
        int vert = iEqui[Polyhedra[i].xc];
        bool found=false;
        for(it=PS.begin() ; it < PS.end(); it++ )
        {
            if((*it).vert == vert)
            {
               found = true;
               break;
            }
        }
        if(!found)       // nuovo poliedro
        {
           polyhedron_style polstyl;
           polstyl.visible = true;
           polstyl.transparency = 0;
           polstyl.ifBorder = true;
           int index1 = atm_au[vert].Z-1;
           polstyl.colore = light_diffuse[index1];
           polstyl.Bcolore = light_diffuse[index1];
           TuneHSV(polstyl.Bcolore, 0.2f);
           polstyl.line_width = 1.5f;
           polstyl.vert = vert;
           PS.push_back(polstyl);
        }
   }
   for(it=PS.begin() ; it < PS.end(); it++ )
   {
        bool found=false;
        int vert = (*it).vert;
        for (int i=0; i<nPoly; i++)
        {
             int vert1 = iEqui[Polyhedra[i].xc];
             if(vert1 == vert)
             {
                found = true;
                break;
             }
        }
        if(!found)          // poliedro scomparso
           PS.erase(it);
   }
   nPolyStyles = (int) PS.size();
   if(PolyStyle != NULL)
   {
       delete [] PolyStyle;
       PolyStyle = NULL;
   }
   PolyStyle = new polyhedron_style [nPolyStyles]();
   for(int i=0; i<nPolyStyles; i++)
       PolyStyle[i] = PS.at(i);
       //memcpy(&PolyStyle[i], &PS.at(i), sizeof(polyhedron_style));
   PS.clear();
}

void Model::CalcolaNormali()
{
   if(nPoly == 0)
      return;
   std::vector<_VertVett> vertvett;
   for (int i=0; i<nPoly; i++)
   {
        int nf = NumberOfFaces(Polyhedra[i]);
        vertvett.clear();
        int found[NMAX_FACE_POINTS] = {-1, -1, -1, -1};
        vet3 *nv = new vet3 [nf]();
        for(int j=0; j<nf; j++)
        {
           int nvf = Polyhedra[i].face[j].n;
           int *nn = new int [nvf]();
           for(int l=0; l<nvf; l++)
               nn[l] = Polyhedra[i].face[j].fp[l];
           for(int ifound=0; ifound <NMAX_FACE_POINTS; ifound++)
           {
               for(size_t kk=0; kk<vertvett.size(); kk++)
               {
                   if(vertvett.at(kk).vertex == nn[ifound])
                      found[ifound] = (int) kk;
               }
               if(found[ifound] == -1)
               {
                  _VertVett v = {nn[ifound], {j, -1, -1, -1}};
                  vertvett.push_back(v);
               }
               else
               {
                   for(int ff=0; ff<4; ff++)
                   {
                       if(vertvett.at(found[ifound]).faces[ff] == -1)
                        {
                          vertvett.at(found[ifound]).faces[ff] = j;
                          break;
                       }
                   }
               }
           }
           vet3 a, b, c;
           for(int kk=0; kk<3; kk++)
           {
               a.v[kk] = (double)atm[Polyhedra[i].face[j].fp[0]].xyzN[kk];
               b.v[kk] = (double)atm[Polyhedra[i].face[j].fp[1]].xyzN[kk];
               c.v[kk] = (double)atm[Polyhedra[i].face[j].fp[2]].xyzN[kk];
           }
           CalcolaNormale(a, b, c, &(nv[j]));
           delete [] nn;
        }
        for(size_t kk=0; kk<vertvett.size(); kk++)
        {
            int nfc =(nf == 4)? 3:4;
            vet3 *v = new vet3 [nfc]();
            vet3 *vij = new vet3 [nfc]();
            for(int j=0; j<nfc;j++)
            {
                int jj = vertvett.at(kk).faces[j];
                MRVectorCopy(nv[jj], &v[j]);
            }
            for(int j=0; j<nfc;j++)
            {
                int jj = (j+1) % nfc;
                vij[j] = MRVectorCrossProduct(v[j], v[jj]);
                vij[j] = MRVectorNormalize(v[j]);
            }
            vet3 sumvet = {{0, 0, 0}};
            for(int j=0; j<nfc;j++)
                sumvet = MRVectorAdd(sumvet, vij[j]);
            sumvet = MRVectorNormalize(sumvet);
            memcpy(atm[vertvett.at(kk).vertex].vn, sumvet.v, 3*sizeof(float));
            delete [] v;
            delete [] vij;
        }
        delete [] nv;
   }
}

QColor Model::SetRungColor(int whichAtom)
{
    int index1, ieq;
     ieq = iEqui[whichAtom];
     index1 = atm_au[ieq].numResiduo;
     if(index1 >= (N_AMINO + N_DNA_NUCLEOTIDES))
         index1 = OTHERS;
     else
         index1 += N_ELEMENTS + N_CHAINS;
     return light_diffuse[index1];
}

void Model::DrawScaletta(QOpenGLFunctions *f)
{
    float ca[3], canext[3];
    vet3 p1, p2, p3;
    QColor c;
    float color[4];
    int d = (int)phlist.size();
    int lfin = 1;
    if(ifSymmetry)
       lfin = NEquivs[0];
    for(int l=0; l<lfin; l++)
    {
      for(int i=0; i<d-1; i+=2)
      {
        int at1 = Equivs[phlist[i]].at(l);
        int at2 = Equivs[phlist[i+1]].at(l);
        memcpy(ca,  atm[at1].xyzN, 3*sizeof(float));
        memcpy(canext,  atm[at2].xyzN, 3*sizeof(float));
        p1 = MRVectorMake(ca);
        p2 = MRVectorMake(canext);
        p3 = MRVectorMedium(p1, p2);
        c = SetRungColor(at1);
        FromQColor(c, color);
        renderCylinder(f, p1, p3, 1.5*Properties::cylinder,color);
        c = SetRungColor(at2);
        FromQColor(c, color);
        renderCylinder(f, p3, p2, 1.5*Properties::cylinder,color);
      }
    }
}

void Model::DrawIsolated(QOpenGLFunctions *f)
{
      int i;
      float color1[4];

      glPushMatrix();
      f->glDisable (GL_LIGHTING);

      for(i=0; i<nats; i++)
      {
         if(atm[i].disegnato)
            continue;
         if( (!ifDrawH) &&  IsHydrogen(i) )
            continue;
         if( (!ifDrawW) &&  atm_au[iEqui[i]].isW )
            continue;
         if(atm[i].deleted || atm[i].hidden)
            continue;
         if(IsPeak(i))
            continue;
         (void) SetColor(i, color1);
         if(Properties::ifDesaturate())
            Desaturate(color1);
         jav->JavDrawText(MRVectorMake(atm[i].xyzN), "+", ToQColor(color1, 4));

      }
      glPopMatrix();
}

void DrawTubesSTL(int npoints, Knot knot[], double factor=1)
{
     for(int i=0; i<npoints-1; i++)
     {
          vet3 x1 = {{ knot[i].px, knot[i].py, knot[i].pz}};
          vet3 x2 = {{ knot[i+1].px, knot[i+1].py, knot[i+1].pz}};
          stl_file.PrintCilindroSTL(x1, x2, factor*1.5*Properties::cylinder);
     }
}

void DrawSolidRibbonSTL(Knot src, Knot dst)
{
    vet3 v[4];

// Top Surface
    v[0].x = src.px+src.wx+src.dx;
    v[0].y = src.py+src.wy+src.dy;
    v[0].z = src.pz+src.wz+src.dz;

    v[1].x = dst.px+dst.wx+dst.dx;
    v[1].y = dst.py+dst.wy+dst.dy;
    v[1].z = dst.pz+dst.wz+dst.dz;

    v[2].x = dst.px-dst.wx+dst.dx;
    v[2].y = dst.py-dst.wy+dst.dy;
    v[2].z = dst.pz-dst.wz+dst.dz;

    v[3].x = src.px-src.wx+src.dx;
    v[3].y = src.py-src.wy+src.dy;
    v[3].z = src.pz-src.wz+src.dz;
    stl_file.Pol2Trigs(v);

// Bottom Surface
    v[0].x = src.px+src.wx-src.dx;
    v[0].y = src.py+src.wy-src.dy;
    v[0].z = src.pz+src.wz-src.dz;

    v[1].x = dst.px+dst.wx-dst.dx;
    v[1].y = dst.py+dst.wy-dst.dy;
    v[1].z = dst.pz+dst.wz-dst.dz;

    v[2].x = dst.px-dst.wx-dst.dx;
    v[2].y = dst.py-dst.wy-dst.dy;
    v[2].z = dst.pz-dst.wz-dst.dz;

    v[3].x = src.px-src.wx-src.dx;
    v[3].y = src.py-src.wy-src.dy;
    v[3].z = src.pz-src.wz-src.dz;
    stl_file.Pol2Trigs(v);

// Left Surface
    v[0].x = src.px+src.wx+src.dx;
    v[0].y = src.py+src.wy+src.dy;
    v[0].z = src.pz+src.wz+src.dz;
    v[1].x = dst.px+dst.wx+dst.dx;
    v[1].y = dst.py+dst.wy+dst.dy;
    v[1].z = dst.pz+dst.wz+dst.dz;

    v[2].x = dst.px+dst.wx-dst.dx;
    v[2].y = dst.py+dst.wy-dst.dy;
    v[2].z = dst.pz+dst.wz-dst.dz;

    v[3].x = src.px+src.wx-src.dx;
    v[3].y = src.py+src.wy-src.dy;
    v[3].z = src.pz+src.wz-src.dz;
    stl_file.Pol2Trigs(v);

// Right Surface
    v[0].x = src.px-src.wx+src.dx;
    v[0].y = src.py-src.wy+src.dy;
    v[0].z = src.pz-src.wz+src.dz;

    v[1].x = dst.px-dst.wx+dst.dx;
    v[1].y = dst.py-dst.wy+dst.dy;
    v[1].z = dst.pz-dst.wz+dst.dz;

    v[2].x = dst.px-dst.wx-dst.dx;
    v[2].y = dst.py-dst.wy-dst.dy;
    v[2].z = dst.pz-dst.wz-dst.dz;

    v[3].x = src.px-src.wx-src.dx;
    v[3].y = src.py-src.wy-src.dy;
    v[3].z = src.pz-src.wz-src.dz;
    stl_file.Pol2Trigs(v);
}

void Model::DrawRibbonSTL(int kc,  Knot knot[], int npoints)
{
    int ca = c_alphab[kc].atom;

    int plotStyleSav = plotStyle;

    if((plotStyle == RIBBONS) || (plotStyle == STRANDS))
    {
       for(int kk=0; kk<npoints-1; kk++)
           DrawSolidRibbonSTL(knot[kk], knot[kk+1]);
    }
    else
    {
        if(IsBetaSheet(ca))
        {
            for(int kk=0; kk<npoints-1; kk++)
                DrawSolidRibbonSTL(knot[kk], knot[kk+1]);
        }
        else if(IsHelix(ca))
        {
           for(int kk=0; kk<npoints-1; kk++)
              DrawSolidRibbonSTL(knot[kk], knot[kk+1]);
        }
        else if(IsDna(ca))
           DrawTubesSTL(npoints, knot, 3);
        else
           DrawTubesSTL(npoints, knot);
    }
    plotStyle = plotStyleSav;
}

void Model::DrawRibbonsSTL(std::vector<C_Alpha> &c_alphab)
{
    Knot knot[NKNOTS];
    float wide, hwide, size, kf = 1.5f;
    float ca[3], canext[3], o[3], o2[3];
    bool prev = false;
    bool ifArrow;

    memset(knot, 0, nknots*sizeof(Knot));
    int chain = -1;
    for(int i=0; i<(int)c_alphab.size(); i++)
    {
// Defining peptide planes
        if(!prev)
           memset(knot, 0, nknots*sizeof(Knot));

        if(c_alphab[i].chain != chain)
        {
           prev = false;
           chain = c_alphab[i].chain;
           memset(knot, 0, nknots*sizeof(Knot));
        }
        if(c_alphab[i].o == -1)
        {
           prev = false;
           continue;
        }
 
        for(int j=0; j<3; j++)
        {
            ca[j] = atm[c_alphab[i].atom].xyzN[j];
            if(c_alphab[i].isAmino)
               o[j] = atm[c_alphab[i].o].xyzN[j];
            else if(c_alphab[i].n == -1) // deoxyribonucleic acid
            {
               if(!c_alphab[i].breakF)
               {
                  o[j] = atm[c_alphab[i+1].o].xyzN[j];
                  if(c_alphab[i].o2 != -1)
                     o2[j] = atm[c_alphab[i].o2].xyzN[j];
               }
               else
                  o[j] = atm[c_alphab[i].o].xyzN[j];
            }
            else                          // ribonucleic acid
               o[j] = atm[c_alphab[i].o].xyzN[j];
            if(c_alphab[i].breakF)
               continue;
            canext[j] = atm[c_alphab[i+1].atom].xyzN[j];
        }
        if(c_alphab[i].breakF)            // siamo alla fine
        {
           if( IsBetaSheet(c_alphab[i].atom) && (plotStyle == CARTOONS))
           {
               wide = (float)(12*Properties::cylinder);
               knot[nknots-1].px = ca[0] + (knot[nknots-1].tx/2);
               knot[nknots-1].py = ca[1] + (knot[nknots-1].ty/2);
               knot[nknots-1].pz = ca[2] + (knot[nknots-1].tz/2);
               ifArrow = true;
           } 
           else
           {
               wide = (float)(6*Properties::cylinder);
               knot[nknots-1].px = ca[0];
               knot[nknots-1].py = ca[1];
               knot[nknots-1].pz = ca[2];
               ifArrow = false;
           }
           if( (knot[0].wide!=wide) && knot[0].hsize )
           {   
               size = knot[0].hsize;
               knot[0].wx = (wide*knot[0].hnx)/size;
               knot[0].wy = (wide*knot[0].hny)/size;
               knot[0].wz = (wide*knot[0].hnz)/size;

               if( !ifArrow )
               {   knot[nknots-1].wx = knot[0].wx;
                   knot[nknots-1].wy = knot[0].wy;
                   knot[nknots-1].wz = knot[0].wz;
               } 
               else
               {   
                   knot[nknots-1].wx = 0;
                   knot[nknots-1].wy = 0;
                   knot[nknots-1].wz = 0;
               }
            } 
            else if( ifArrow )
            {   
                wide = (float)(12*Properties::cylinder);
                knot[nknots-1].wx = 0;
                knot[nknots-1].wy = 0;
                knot[nknots-1].wz = 0;
            }
            knot[1] = knot[nknots-1];
            if(!prev)
            {
            knot[0] = knot[1];
            knot[0].px = o[0];
            knot[0].py = o[1];
            knot[0].pz = o[2];
            }
            prev = false;
            DrawRibbonSTL(i, knot, 2);
            c_alphab[i].xyz[0] = knot[0].px;
            c_alphab[i].xyz[1] = knot[0].py;
            c_alphab[i].xyz[2] = knot[0].pz;
            continue;
        }
// Get vector A = Ca(i+l) - Ca(i)
        knot[nknots-1].tx = canext[0] - ca[0];
        knot[nknots-1].ty = canext[1] - ca[1];
        knot[nknots-1].tz = canext[2] - ca[2];
// Get vector B = O(i) - Ca(i) 
        float bx, by, bz;
        if(!c_alphab[i].isAmino && (c_alphab[i].o2 != -1) && (c_alphab[i].n == -1))
        {
           bx = (o[0]+o2[0])/2 - ca[0];
           by = (o[1]+o2[0])/2 - ca[1];
           bz = (o[2]+o2[0])/2 - ca[2];
        }
        else
        {
           bx = o[0] - ca[0];
           by = o[1] - ca[1];
           bz = o[2] - ca[2];
        }

// Form point P as the midpoint of Ca(i) and Ca(i+1)
        knot[nknots-1].px = (ca[0] + canext[0])/2;
        knot[nknots-1].py = (ca[1] + canext[1])/2;
        knot[nknots-1].pz = (ca[2] + canext[2])/2;

// Form vector C = A x B
// Vector C is normal to the peptide plane approximated by atoms Ca(i), O(i), and Ca(i+1
        knot[nknots-1].vnx = knot[nknots-1].ty*bz - knot[nknots-1].tz*by;
        knot[nknots-1].vny = knot[nknots-1].tz*bx - knot[nknots-1].tx*bz;
        knot[nknots-1].vnz = knot[nknots-1].tx*by - knot[nknots-1].ty*bx;

// If the peptide plane is part of a helix, translate in the direction of vector C, 
// away from the helix axis
        if(IsHelix(c_alphab[i].atom) && IsHelix(c_alphab[i+1].atom))
        {
           size = sqrt(knot[nknots-1].vnx*knot[nknots-1].vnx + 
                       knot[nknots-1].vny*knot[nknots-1].vny + 
                       knot[nknots-1].vnz*knot[nknots-1].vnz);
        
           knot[nknots-1].vsize = size;
           if( size )
           {   // 1.00 Angstrom Displacement
               wide = 0.7f/Properties::deltav;
               knot[nknots-1].px += ((wide*knot[nknots-1].vnx)/size);
               knot[nknots-1].py += ((wide*knot[nknots-1].vny)/size);
               knot[nknots-1].pz += ((wide*knot[nknots-1].vnz)/size);
           } 
        }
        else 
            knot[nknots-1].vsize = 0;

// Form vector D = C x A
// Vector D lies parallel to the peptide plane and is perpendicular to vector A. 
        float dx = knot[nknots-1].vny*knot[nknots-1].tz - knot[nknots-1].vnz*knot[nknots-1].ty;
        float dy = knot[nknots-1].vnz*knot[nknots-1].tx - knot[nknots-1].vnx*knot[nknots-1].tz;
        float dz = knot[nknots-1].vnx*knot[nknots-1].ty - knot[nknots-1].vny*knot[nknots-1].tx;

// Scale vector D by one half the desired ribbon width
        dx /= (float)(3*Properties::cylinder);
        dy /= (float)(3*Properties::cylinder);
        dz /= (float)(3*Properties::cylinder);
        knot[nknots-1].hsize = sqrt(dx*dx + dy*dy + dz*dz);

// Handle Carbonyl Oxygen Flip 
        if( prev && ((knot[0].hnx*dx + knot[0].hny*dy + knot[0].hnz*dz)<0) )
        {
            knot[nknots-1].hnx = -dx;   knot[nknots-1].vnx = -knot[nknots-1].vnx;
            knot[nknots-1].hny = -dy;   knot[nknots-1].vny = -knot[nknots-1].vny;
            knot[nknots-1].hnz = -dz;   knot[nknots-1].vnz = -knot[nknots-1].vnz;
        }
        else
        {
            knot[nknots-1].hnx = dx;
            knot[nknots-1].hny = dy;
            knot[nknots-1].hnz = dz;
        }

//  Set Ribbon Width 
        wide = (float)(6*Properties::cylinder);

        ifArrow = false;
        if(plotStyle == CARTOONS)
        {
           if(IsBetaSheet(c_alphab[i].atom))
           {
              if(!IsBetaSheet(c_alphab[i+1].atom))
              {
                 ifArrow = true;
                 wide = (float)(12*Properties::cylinder);
              }
           }
           else if(!IsHelix(c_alphab[i].atom))
                wide = (float)(kf*Properties::cylinder);
        }
        if( knot[nknots-1].hsize && !ifArrow)
        {
            size = knot[nknots-1].hsize;
            knot[nknots-1].wx = (wide*knot[nknots-1].hnx)/size;
            knot[nknots-1].wy = (wide*knot[nknots-1].hny)/size;
            knot[nknots-1].wz = (wide*knot[nknots-1].hnz)/size;
            knot[nknots-1].wide = wide;
        }
        else
        {
            knot[nknots-1].wide = 0;
            knot[nknots-1].wx = 0;
            knot[nknots-1].wy = 0;
            knot[nknots-1].wz = 0;
        }

        if(prev && knot[0].hsize)
        {
           size = knot[0].hsize;
           knot[0].wx = (float)(wide*knot[0].hnx)/size;
           knot[0].wy = (float)(wide*knot[0].hny)/size;
           knot[0].wz = (float)(wide*knot[0].hnz)/size;
        }
        hwide = (float)(kf*Properties::cylinder);
//240112        if(IsHelix(c_alphab[i].atom))
        if(IsHelix(c_alphab[i].atom) || (plotStyle == RIBBONS))
           hwide /= 3.;
        if(prev && knot[0].vsize)
        {
           size = knot[0].vsize;
           knot[0].dx = (float)(hwide*knot[0].vnx)/size;
           knot[0].dy = (float)(hwide*knot[0].vny)/size;
           knot[0].dz = (float)(hwide*knot[0].vnz)/size;
        }
        GetSize(&knot[nknots-1]);

        size = knot[nknots-1].vsize;
        knot[nknots-1].dx = (float)(hwide)*knot[nknots-1].vnx/size;
        knot[nknots-1].dy = (float)(hwide)*knot[nknots-1].vny/size;
        knot[nknots-1].dz = (float)(hwide)*knot[nknots-1].vnz/size;

        if(prev)
        {
           for(int k=0; k<nknots-2; k++)
           {
// Calculate Hermite Spline Points 
               knot[k+1].px = H[0][k]*knot[0].px + H[1][k]*knot[0].tx +
                              H[2][k]*knot[nknots-1].px + H[3][k]*knot[nknots-1].tx;
               knot[k+1].py = H[0][k]*knot[0].py + H[1][k]*knot[0].ty +
                              H[2][k]*knot[nknots-1].py + H[3][k]*knot[nknots-1].ty;
               knot[k+1].pz = H[0][k]*knot[0].pz + H[1][k]*knot[0].tz +
                              H[2][k]*knot[nknots-1].pz + H[3][k]*knot[nknots-1].tz;

//  Calculate Hermite Spline Widths 
               knot[k+1].wx = H[0][k]*knot[0].wx + H[2][k]*knot[nknots-1].wx;
               knot[k+1].wy = H[0][k]*knot[0].wy + H[2][k]*knot[nknots-1].wy;
               knot[k+1].wz = H[0][k]*knot[0].wz + H[2][k]*knot[nknots-1].wz;

// Calculate Surface Intensity 
               knot[k+1].vsize = 0;
               knot[k+1].vnx = H[0][k]*knot[0].vnx + H[2][k]*knot[nknots-1].vnx;
               knot[k+1].vny = H[0][k]*knot[0].vny + H[2][k]*knot[nknots-1].vny;
               knot[k+1].vnz = H[0][k]*knot[0].vnz + H[2][k]*knot[nknots-1].vnz;
               GetSize(&knot[k+1]);

//240112               if(plotStyle == CARTOONS)
               {
// Calculate Spline Heights 
                  knot[k+1].dx =  (float)(hwide*knot[k+1].vnx/knot[k+1].vsize); 
                  knot[k+1].dy =  (float)(hwide*knot[k+1].vny/knot[k+1].vsize);
                  knot[k+1].dz =  (float)(hwide*knot[k+1].vnz/knot[k+1].vsize);
// Calculate Surface Intensity 
                  knot[k+1].hsize = 0;
                  knot[k+1].hnx = H[0][k]*knot[0].hnx + H[2][k]*knot[nknots-1].hnx;
                  knot[k+1].hny = H[0][k]*knot[0].hny + H[2][k]*knot[nknots-1].hny;
                  knot[k+1].hnz = H[0][k]*knot[0].hnz + H[2][k]*knot[nknots-1].hnz;
                  GetSize(&knot[k+1]);
               }
           }

           DrawRibbonSTL(i, knot, nknots);
           knot[0] = knot[nknots-1];
            c_alphab[i].xyz[0] = knot[4].px;
            c_alphab[i].xyz[1] = knot[4].py;
            c_alphab[i].xyz[2] = knot[4].pz;
        }
        else                     // siamo all'inizio
        {
           knot[1] = knot[nknots-1];
           knot[0] = knot[nknots-1];
           knot[0].px = ca[0];
           knot[0].py = ca[1];
           knot[0].pz = ca[2];
           knot[0].hnx = dx;
           knot[0].hny = dy;
           knot[0].hnz = dz;
           c_alphab[i].xyz[0] = knot[0].px;
           c_alphab[i].xyz[1] = knot[0].py;
           c_alphab[i].xyz[2] = knot[0].pz;
        }
        prev = true;
    }
}

void Model::PrintPoliedriSTL()
{
  for (int i=0; i<nPoly; i++)
  {
       int nf = NumberOfFaces(Polyhedra[i]);
       for(int j=0; j<nf; j++)
       {
           if(Polyhedra[i].face[j].n == 3)
           {
              Triangolo trig;
              for(int k=0; k<3; k++)
              {
                  trig.v1[k] = atm[Polyhedra[i].face[j].fp[0]].xyzN[k];
                  trig.v2[k] = atm[Polyhedra[i].face[j].fp[1]].xyzN[k];
                  trig.v3[k] = atm[Polyhedra[i].face[j].fp[2]].xyzN[k];
              }
              trig.nat = -1;
              stl_file.AddTrig(trig);
           }
           else
           {
              vet3 v[4];
              for(int k=0; k<4; k++)
              {
                  v[k].x = atm[Polyhedra[i].face[j].fp[k]].xyzN[0];
                  v[k].y = atm[Polyhedra[i].face[j].fp[k]].xyzN[1];
                  v[k].z = atm[Polyhedra[i].face[j].fp[k]].xyzN[2];
              }
              stl_file.Pol2Trigs(v);
           }
       }
  }
}

void Model::exportScene(std::vector<vet3> &xyzat)
{
  if(plotStyle > C_ALPHA)
  {
     DrawRibbonsSTL(c_alphab);
     return;
  }

  int subdiv;

  int ind = 0;
  if(plotStyle == VANDERWAALS)
     ind = 1;
  else if((plotStyle == RODS) || (plotStyle == C_ALPHA) || (plotStyle == POLYPEPTIDE))
     ind = 2;
  float subd = (float) (300. * maxRad[ind]) * Properties::sc_factor;
  if(ind != 2)
     subd *= (float) Properties::sc_atoms;
  subdiv = (Properties::Scale > 1.)? ((int) (subd * Properties::Scale)) : ((int) subd);
  for(int i=0; i<nats;i++)
      atm[i].disegnato = ((plotStyle == STICK_AND_BALLS)||(plotStyle == ELLIPSOIDS));
  int i0=0, i1=0;
  jav_bond_type *legs=NULL;
  if(plotStyle < VANDERWAALS)
  {
     i0 = 0;
     i1 = nconn_prot;
     legs = bonds;
  }
  else if ( (plotStyle == POLYPEPTIDE) || (plotStyle == RIBBONS) )
  {
     i0 = 0;
     i1 = nconn_bb;
     legs = bonds_bb;
  }
  else if (plotStyle == C_ALPHA)
  {
     i0 = 0;
     i1 = nconn_ca;
     legs = bonds_ca;
  }

  double radius_c = Properties::cylinder*Properties::sc_bonds;
  for(int i=i0; i<i1;i++)
  {
        int j1 = legs[i].n1 - 1;
        if(j1 >= nats)
            continue;
        if(j1 < 0)
           return;
         if(atm[j1].deleted || atm[j1].hidden)
            continue;
        int j2 = legs[i].n2 - 1;
        if(j2 >= nats)
            continue;
         if(atm[j2].deleted || atm[j2].hidden)
            continue;
        if( (!ifDrawH) &&  (IsHydrogen(j1) || IsHydrogen(j2)) )
            continue;
        if( (!ifDrawW) && ( atm_au[iEqui[j1]].isW  || atm_au[iEqui[j2]].isW ) )
            continue;
        vet3 p1 = MRVectorMake(atm[j1].xyzN);
        vet3 p2 = MRVectorMake(atm[j2].xyzN);
        stl_file.PrintCilindroSTL(p1, p2, radius_c);
  }
  for(int i=0; i<nats;i++)
      atm[i].disegnato = 0;

  if(plotStyle < POLYPEPTIDE)
  {
  int np = 0;
  if(!xyzat.empty())
     np = (int) xyzat.size();
  for(int i=0; i<nats; i++)
  {
      int z = atm[i].Z-1;
      if((z< 0) || (z > N_TRUE_ELEMENTS))
         z = N_TRUE_ELEMENTS;
      if( (!ifDrawH) &&  IsHydrogen(i) )
          continue;
      if( (!ifDrawW) &&  atm_au[iEqui[i]].isW )
          continue;
      if(atm[i].deleted || atm[i].hidden)
            continue;
      if(IsPeak(i))
         continue;
      double radius = atm[i].radN[ind];
      if(ind != 2)
         radius *=  Properties::scale_atoms[z];
      else
         radius *=  Properties::sc_bonds;
      stl_file.PrintSferaSTL(i+np, radius, subdiv*5);
      xyzat.push_back(MRVectorMake(atm[i].xyzN));
  }
  }

  if(ifPoly && (nPoly > 0))
     PrintPoliedriSTL();
}

QString Model::GetSpecies(int whichNo)
{
    if((whichNo >= 0) && (whichNo < numSpec))
         return LabSpec[whichNo];
    else
         return "Q";
}

void Model::PreparaTabelle()
{
    string msg;
    int ier = PreparaTabelle(msg);
    if(ier < 0)
       qDebug() << msg.c_str();
}

float Model::VolumePerAtom(float volume, int nsymt)
{
   if(nats == 0)
   {
      int nTot = 0;
      for(int i=0; i<numSpec; i++)
      {
          if(Species[i] != 1)
            nTot += Content[i];
      }
      if(nTot > 0)
         return (volume / (float) nTot);
   }
   else
   {
      float somma_occ = 0;
      for(int i=0; i<nats_au; i++)
      {
          if(!IsHydrogen(i))
             somma_occ = somma_occ + (atm[i].och*atm[i].ocry);
      }
      somma_occ = nsymt * somma_occ;
      if(somma_occ > 0)
         return (volume / somma_occ);
   }
   return 0;
}

void Model::MoveCenter(float posi[], int whichAtom, bool ifDrawCenter)
{
    float xyz1[3] = {(float) Properties::xyzC.x, (float) Properties::xyzC.y, (float) Properties::xyzC.z};
    float xyz2[3] = {posi[0], posi[1], posi[2]};
    float l[3];
    for(int j=0; j<3; j++)
        l[j] = xyz2[j]-xyz1[j];
    int nstep = 20;
    float step = 0.05f;
    float t = 0.f;
    for(int k=0; k<nstep; k++)
    {
        float xyz3[3];
        t += step;
        for(int j=0; j<3; j++)
            xyz3[j] = xyz1[j] + l[j] * t;
        for(int j=0; j<3; j++)
               Properties::xyzC.v[j] = (double) xyz3[j];
        if(Properties::ifDrawPar)
           Properties::CalcPar(Properties::xyzC, Properties::abcPar);

        jav->RiNormalizza();
        Properties::ifAxes = 0;
        jav->Draw();
    }
    if(Properties::ifDrawPar)
       Properties::CalcPar(Properties::xyzC, Properties::abcPar);
    jav->RiNormalizza();
    Properties::DrawCenter = ifDrawCenter;
    if(whichAtom > -1)
    {
       for(int i=0; i<nats; i++)
           if(atm[i].drawLabel < 2)
               atm[i].drawLabel = 0;
       atm[whichAtom].drawLabel = 1;
    }

    Properties::ifAxes = 1;
    jav->Draw();
 #ifdef MAPS
    jav->updateCenter();
 #endif
}

QList<QStandardItem *> *Model::FillListAtoms(bool is_edit, bool ifB, int *nelems)
{
   int natoms = nats;
   if(is_edit)
      natoms = nats_au;
   *nelems = natoms;
	
   QList<QStandardItem *>  *lista = new QList<QStandardItem *> [natoms]();
   for(int i=0; i<natoms; i++)
   {
       int ieq = i;
       if(is_edit)
           ieq = Equivs[i].at(0);
       QStandardItem *item = new QStandardItem();
       item->setData(i+1, Qt::DisplayRole);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(QString(atm[ieq].lab.c_str()).trimmed());
       lista[i].append(item);
       if(atm[i].color >= 0)
          item = new QStandardItem(LabSpec[atm[ieq].color].trimmed());
       else
          item = new QStandardItem("Q");
       lista[i].append(item);
       //if(!is_edit)
       {
          item = new QStandardItem();
          item->setData(atm[ieq].H,  Qt::DisplayRole);
          lista[i].append(item);
       }
       for(int k=0; k<3;k++)
       {
           item = new QStandardItem();
           item->setData(atm[ieq].xyzc[k], Qt::DisplayRole);
           lista[i].append(item);
       }
       float val;
       if(ifB)
          val = atm[ieq].B;
       else
          val = atm[ieq].B/(8*M_PI*M_PI);
       item = new QStandardItem();
       item->setData(val, Qt::DisplayRole);
       lista[i].append(item);
       val = atm[ieq].och;
       //if(!is_edit)
       //   val *= atm[ieq].ocry;
       item = new QStandardItem();
       item->setData(val, Qt::DisplayRole);
       lista[i].append(item);
       //if(!is_edit)
       {
          int ko = atm[ieq].op.op;
          item = new QStandardItem(QString(Properties::getStringSymm(&ko)).trimmed());
          lista[i].append(item);
       }
       item = new QStandardItem(QString::number(atm[ieq].Z));
       lista[i].append(item);
   }   
   return lista;
}

QList<QStandardItem *> *Model::FillListAtomsPDB(bool ifB,
		int ch, int res, int *nelems)
{
   *nelems = 0;
   if((ch < 0) || (res < 0))
       return nullptr;
   RESIDUE *r1 = &Chains[ch].Residues[res];

   int natoms = r1->atoms.size();
   *nelems = natoms;
	
   QList<QStandardItem *>  *lista = new QList<QStandardItem *> [natoms]();
   for(int i=0; i<natoms; i++)
   {
       int ieq = r1->atoms.at(i);
       QStandardItem *item = new QStandardItem();
       item->setData(i+1, Qt::DisplayRole);
       lista[i].append(item);
       item = new QStandardItem(QString(atom_types.at(ieq).c_str()).trimmed());
       lista[i].append(item);
       for(int k=0; k<3;k++)
       {
           item = new QStandardItem();
	   item->setData(atm[ieq].xyzc[k], Qt::DisplayRole);
           lista[i].append(item);
       }
       float val;
       val = atm[ieq].och;
       item = new QStandardItem();
       item->setData(val, Qt::DisplayRole);
       lista[i].append(item);       
       if(ifB)
          val = atm[ieq].B;
       else
          val = atm[ieq].B/(8*M_PI*M_PI);
       item = new QStandardItem();
       item->setData(val, Qt::DisplayRole);
       lista[i].append(item);
   }   
   return lista;
}

QList<QStandardItem *> Model::FillListChains()
{
    QList<QStandardItem *>  lista;
    for (int i=0;i<nChains;i++)
    {
	 QString s = "Chain ";
	 s.append(Chains[i].id.c_str());
         QStandardItem *item = new QStandardItem(s);
	 lista.append(item);
    }
    return lista;
}

QList<QStandardItem *> *Model::FillListResidues()
{
    QList<QStandardItem *>  *lista = new QList<QStandardItem *> [nChains]();
    for (int ch=0;ch<nChains;ch++)
    {
      for(int r=0; r<Chains[ch].nResidues; r++)
      {
           QString rn = Chains[ch].id.c_str();
           rn.append(" ");
           RESIDUE rr = Chains[ch].Residues[r];
           rn.append(rr.PDBnum.c_str());
           rn.append(" ");
           rn.append(rr.name.c_str());
           QStandardItem *item = new QStandardItem(rn);
	   lista[ch].append(item);
      }
    }
    return lista;
}

QString Model::GetType(int na, ...)
{
    va_list vl;
    QString labtype("");
    va_start(vl,na);
    int if_au = va_arg(vl,int);
    for (int i=0;i<na;i++)
    {
         int n1 = va_arg(vl,int);
         if(i)
            labtype.append("-");
         int ks;
         if(if_au)
            ks = atm_au[n1].color;
         else
            ks = atm[n1].color;
         if(ks >= 0)
            labtype.append(LabSpec[ks].trimmed());
         else
            labtype.append("Q");
    }
    va_end(vl);
    return labtype;
}

QList<QStandardItem *> *Model::FillListBonds(bool is_edit)
{
   QList<QStandardItem *>  *lista = new QList<QStandardItem *> [nconn_prot]();

   for(int i=0; i<nconn_prot; i++)
   {
       int n1 = bonds[i].n1 - 1;
       int n2 = bonds[i].n2 - 1;
       float dist = bonds[i].dist;
       QString label1 = atm[n1].lab.c_str();
       QString label2 = atm[n2].lab.c_str();
       QString labtype = GetType(2, 0, n1, n2);
       QStandardItem *item = new QStandardItem();
       item->setData(i+1, Qt::DisplayRole);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(labtype);	       
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(label1);	       
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(label2);	       
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem();
       item->setData(dist, Qt::DisplayRole);
       lista[i].append(item);
       if(!is_edit)
       {
          int ko = atm[n1].op.op;
          QString label3 = QString(Properties::getStringSymm(&ko)).trimmed();
          item = new QStandardItem(label3);	       
          lista[i].append(item);
          ko = atm[n2].op.op;
          label3 = QString(Properties::getStringSymm(&ko)).trimmed();
          item = new QStandardItem(label3);	       
          lista[i].append(item);
       }
   }
   return lista;
}

QList<QStandardItem *> *Model::FillListAngles(bool is_edit)
{
   QList<QStandardItem *>  *lista = new QList<QStandardItem *> [nangbonds]();

   for(int i=0; i<nangbonds; i++)
   {
       int n1 = anglebonds[i].n1 - 1;
       int n2 = anglebonds[i].n2 - 1;
       int n3 = anglebonds[i].n3 - 1;
       float ang = anglebonds[i].fv;
       if( (n1 >= nats_au) || (n2 >= nats_au) || (n3 >= nats_au) )
            continue;
       QString label1 = atm_au[n1].lab.c_str();
       QString label2 = atm_au[n2].lab.c_str();
       QString label3 = atm_au[n3].lab.c_str();
       QString labtype = GetType(3, 1, n1, n2, n3);
       QStandardItem *item = new QStandardItem();
       item->setData(i+1, Qt::DisplayRole);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(labtype);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(label1);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(label2);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       item = new QStandardItem(label3);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem();
       item->setData(ang, Qt::DisplayRole);
       lista[i].append(item);
   }
   return lista;
}

int Model::FindConn(int at1, int at2, int nlegs)
{
    if(nlegs < 0)
       nlegs = nconn;
    for(int i=0; i<nlegs; i++)
    {
        int j1 = bonds[i].n1 - 1;
        int j2 = bonds[i].n2 - 1;
        if( ((j1 == at1) && (j2 == at2)) ||
            ((j1 == at2) && (j2 == at1)) )
           return i;
    }
    return -1;
}

void Model::SelezionaAngolo(int which)
{
    if(anglebonds == nullptr)
    {
        qDebug() << "no angles";
        return;
    }
    int n1 = anglebonds[which].n1 - 1;
    int n2 = anglebonds[which].n2 - 1;
    int n3 = anglebonds[which].n3 - 1;
    if(Equivs[n1].empty())
        return;
     if(Equivs[n2].empty())
        return;
     if(Equivs[n3].empty())
        return;
     int n1eq = (int) Equivs[n1].size();
     int n2eq = (int) Equivs[n2].size();
     int n3eq = (int) Equivs[n3].size();
     int neq  = min(min(n1eq, n2eq), n3eq);
     for(int keq=0; keq<neq; keq++)
     {
         int ieq1 = Equivs[n1].at(keq);
         int ieq2 = Equivs[n2].at(keq);
         int ieq3 = Equivs[n3].at(keq);
         int bb1 = FindConn(ieq1, ieq2);
         int bb2 = FindConn(ieq2, ieq3);
         if((bb1 != -1) && (bb2 !=-1))
         {
             atm[ieq1].selected = true;
             atm[ieq2].selected = true;
             atm[ieq3].selected = true;
             atoms_sel.push_back(ieq1+1);
             atoms_sel.push_back(ieq2+1);
             atoms_sel.push_back(ieq3+1);
             bonds[bb1].selected = true;
             bonds_sel.push_back(bb1+1);
             bonds[bb2].selected = true;
             bonds_sel.push_back(bb2+1);
         }
     }
}

void Model::FillAngs(int nangles, jav_anglebond_type *angs)
{
  nangbonds = nangles;
  if(anglebonds)
     delete [] anglebonds;
  anglebonds = new jav_anglebond_type [nangbonds]();
  for(int i=1; i<=nangbonds; i++)
  {
      anglebonds[i-1].n1 = angs[i-1].n1;
      anglebonds[i-1].n2 = angs[i-1].n2;
      anglebonds[i-1].n3 = angs[i-1].n3;
      anglebonds[i-1].fv = angs[i-1].fv;
      anglebonds[i-1].sigma = angs[i-1].sigma;
  }
}

void Model::FillTors(int nangles, jav_torsion_type *angs)
{
  ntorsions = nangles;
  if(torsions)
     delete [] torsions;
  torsions = new jav_torsion_type [ntorsions]();
  for(int i=1; i<=ntorsions; i++)
  {
      torsions[i-1].n1 = angs[i-1].n1;
      torsions[i-1].n2 = angs[i-1].n2;
      torsions[i-1].n3 = angs[i-1].n3;
      torsions[i-1].n4 = angs[i-1].n4;
      torsions[i-1].fv = angs[i-1].fv;
  }
}


QList<QStandardItem *> *Model::FillListTorsions(bool is_edit)
{
   QList<QStandardItem *>  *lista = new QList<QStandardItem *> [ntorsions]();

   for(int i=0; i<ntorsions; i++)
   {
       int n1 = torsions[i].n1 - 1;
       int n2 = torsions[i].n2 - 1;
       int n3 = torsions[i].n3 - 1;
       int n4 = torsions[i].n4 - 1;
       float ang = torsions[i].fv;
       if( (n1 >= nats_au) || (n2 >= nats_au) ||
	   (n3 >= nats_au) || (n4 >= nats_au) )
            continue;
       QString label1 = atm_au[n1].lab.c_str();
       QString label2 = atm_au[n2].lab.c_str();
       QString label3 = atm_au[n3].lab.c_str();
       QString label4 = atm_au[n4].lab.c_str();
       QString labtype = GetType(4, 1, n1, n2, n3, n4);
       QStandardItem *item = new QStandardItem();
       item->setData(i+1, Qt::DisplayRole);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(labtype);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(label1);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(label2);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(label3);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem(label4);
       if(is_edit)
           item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
       lista[i].append(item);
       item = new QStandardItem();
       item->setData(ang, Qt::DisplayRole);
       lista[i].append(item);
   }
   return lista;
}

void Model::SelezionaTorsion(int which)
{
    if(torsions == nullptr)
    {
        qDebug() << "no torsions";
        return;
    }
     int n1 = torsions[which].n1 - 1;
     int n2 = torsions[which].n2 - 1;
     int n3 = torsions[which].n3 - 1;
     int n4 = torsions[which].n4 - 1;
     if(Equivs[n1].empty())
        return;
     if(Equivs[n2].empty())
        return;
     if(Equivs[n3].empty())
        return;
     if(Equivs[n4].empty())
        return;
     int n1eq = (int) Equivs[n1].size();
     int n2eq = (int) Equivs[n2].size();
     int n3eq = (int) Equivs[n3].size();
     int n4eq = (int) Equivs[n4].size();
     int neq  = min(min(n1eq, n2eq), min(n3eq, n4eq));
     for(int keq=0; keq<neq; keq++)
     {
         int ieq1 = Equivs[n1].at(keq);
         int ieq2 = Equivs[n2].at(keq);
         int ieq3 = Equivs[n3].at(keq);
         int ieq4 = Equivs[n4].at(keq);
         int bb1 = FindConn(ieq1, ieq2);
         int bb2 = FindConn(ieq2, ieq3);
         int bb3 = FindConn(ieq3, ieq4);
         if((bb1 != -1) && (bb2 !=-1) && (bb3 != -1))
         {
             atm[ieq1].selected = true;
             atm[ieq2].selected = true;
             atm[ieq3].selected = true;
             atm[ieq4].selected = true;
             atoms_sel.push_back(ieq1+1);
             atoms_sel.push_back(ieq2+1);
             atoms_sel.push_back(ieq3+1);
             atoms_sel.push_back(ieq4+1);
             bonds[bb1].selected = true;
             bonds_sel.push_back(bb1+1);
             bonds[bb2].selected = true;
             bonds_sel.push_back(bb2+1);
             bonds[bb3].selected = true;
             bonds_sel.push_back(bb3+1);
         }
     }	
}


QList<QStandardItem *> *Model::FillListContacts(int *ncontacts)
{
   int nc = 0;
   if(jav->GetInfo(IFHBONDS, 0))
      nc += nlegsH;
   if(jav->GetInfo(IFSHORTC, 0))
      nc += nlegsW;
   *ncontacts = nc;
   
   QList<QStandardItem *>  *lista = new QList<QStandardItem *> [nc]();

   nc = 0;
   QPixmap pixmap(12,12);

   if(jav->GetInfo(IFHBONDS, 0))
   {
      for(int i=0; i<nlegsH; i++)
      {
          int n1 = abs(legsH[i].n1) - 1;
          int n2 = abs(legsH[i].n2) - 1;
          float dist = legsH[i].dist;
          QString label1 = atm[n1].lab.c_str();
          QString label2 = atm[n2].lab.c_str();
          int ko = atm[n1].op.op;
          QString label3 = Properties::getStringSymm(&ko);
          ko = atm[n2].op.op;
          QString label4 = Properties::getStringSymm(&ko);
          pixmap.fill(Properties::ColorHB);
          QStandardItem *item = new QStandardItem();
          item->setData(i+1, Qt::DisplayRole);
	  item->setData(pixmap, Qt::DecorationRole);
          lista[i].append(item);
          item = new QStandardItem(label1);
          lista[i].append(item);
          item = new QStandardItem(label2);
          lista[i].append(item);
          item = new QStandardItem();
          item->setData(dist, Qt::DisplayRole);
          lista[i].append(item);
          item = new QStandardItem(label3);
          lista[i].append(item);
          item = new QStandardItem(label4);
          lista[i].append(item);
       }       
       nc += nlegsH;
   }
   if(jav->GetInfo(IFSHORTC, 0))
   {
       for(int i=0; i<nlegsW; i++)
       {
	   int num = i + nc;
           int n1 = abs(legsW[i].n1) - 1;
           int n2 = abs(legsW[i].n2) - 1;
           float dist = legsW[i].dist; 
           QString label1 = atm[n1].lab.c_str();
           QString label2 = atm[n2].lab.c_str();
           int ko = atm[n1].op.op;
           QString label3 = Properties::getStringSymm(&ko);
           ko = atm[n2].op.op;
           QString label4 = Properties::getStringSymm(&ko);
           pixmap.fill(Properties::ColorSC);
           QStandardItem *item = new QStandardItem();
           item->setData(num+1, Qt::DisplayRole);
	   item->setData(pixmap, Qt::DecorationRole);
           lista[num].append(item);
           item = new QStandardItem(label1);
           lista[num].append(item);
           item = new QStandardItem(label2);
           lista[num].append(item);
           item = new QStandardItem();
           item->setData(dist, Qt::DisplayRole);
           lista[num].append(item);
           item = new QStandardItem(label3);
           lista[num].append(item);
           item = new QStandardItem(label4);
           lista[num].append(item);
       }
   }

   return lista;
}

QVector<bar_info> Model::FillBarChart(QVector<bar_legend> &barLegend)
{
    QVector<bar_info> lista;
    bar_info bi;
    bar_legend bl;
#if 0    
    for(int j=0; j<numSpec; j++)
    {
        bl.label = QString::number(Species[j]) + " " + LabSpec[j];
        bl.val   = (float) Species[j];
        bl.color = light_diffuse[Species[j]-1];
	barLegend.append(bl);
    }
#endif
    QVector<int> vec_sp;
    for(int i=0; i<nats_au; i++)
    {
         bi.value = atm_au[i].H;
         bi.color = light_diffuse[atm_au[i].Z-1];
         bi.tip   = QString(atm_au[i].lab.c_str()).trimmed()  + 
		 " - Height = " + QString::number(atm_au[i].H, 'f', 2);
         lista.append(bi);
	 if(!vec_sp.contains(atm_au[i].Z))
            vec_sp.append(atm_au[i].Z);
    }
    std::sort(vec_sp.begin(), vec_sp.end());
    for (int j=0; j<vec_sp.count(); j++)
    {
        bl.label = QString::number(vec_sp[j]) + " " + get_nth_element(vec_sp[j]-1);
        bl.val   = (float) vec_sp[j];
        bl.color = light_diffuse[vec_sp[j]-1];
	barLegend.append(bl);
    }
    return lista;
}

void Model::CreaMappa(int whichmod, float pRadius, float step)
{
   vet3 delt, xsmin={{1e6, 1e6, 1e6}}, xsmax={{-1e6, -1e6, -1e6}};
   double due = 2.0;
   double ln2 = log(due);

   jav->RiNormalizza();
   for(int i=0; i<nats_au; i++)
   {
       xsmin = MRVectorMinimum(xsmin,
                MRVectorAddScalar(MRVectorMake(atm[i].xyz), -radMax));
       xsmax = MRVectorMaximum(xsmax,
                MRVectorAddScalar(MRVectorMake(atm[i].xyz),  radMax));
   }
   delt = MRVectorSub(xsmax, xsmin);

   vet3 xcmin = Properties::CartToFract(xsmin);
   vet3 xcmax = Properties::CartToFract(xsmax);
   int np[] = {(int) (delt.x/step + 0.5f)+1,
               (int) (delt.y/step + 0.5f)+1,
               (int) (delt.z/step + 0.5f)+1};
   float deltx  = (float)np[0] / (float)delt.x;
   float delty  = (float)np[1] / (float)delt.y;
   float deltz  = (float)np[2] / (float)delt.z;
   float caChem = (float) cos(DEGTORAD(Properties::Cell[3]));
   float cbChem = (float) cos(DEGTORAD(Properties::Cell[4]));
   float cgChem = (float) cos(DEGTORAD(Properties::Cell[5]));

   float ***mappa = new float ** [np[0]]();
   int   ***colormappa = new int ** [np[0]]();
   for(int i=0; i<np[0]; i++)
   {
       mappa[i] = new float * [np[1]]();
       colormappa[i] = new int * [np[1]]();
       for(int j=0; j<np[1]; j++)
       {
           mappa[i][j] = new float  [np[2]]();
           colormappa[i][j] = new int [np[2]]();
           for(int k=0; k<np[2]; k++)
           {
               mappa[i][j][k] = 0;
               colormappa[i][j][k] = 0;
           }
       }
   }
   float xs1[3];
   float alim = (float) radMax;
   for(int i=0; i<nats_au; i++)
   {
       int ieq = Equivs[i].at(0);
       for(int j=0; j<3; j++)
           xs1[j] = side(atm[ieq].xyzc[j], (float) xcmin.v[j], (float) xcmax.v[j]);
       int index = atm[ieq].color;
       float raggioVDW = radii[index][1];
       float DmaxSq = alim*alim;

       int iszu = (int)(xs1[2]*np[2]+alim*deltz);
       int iszl = (int)(xs1[2]*np[2]-alim*deltz);
       int isyu = (int)(xs1[1]*np[1]+alim*delty);
       int isyl = (int)(xs1[1]*np[1]-alim*delty);
       int isxu = (int)(xs1[0]*np[0]+alim*deltx);
       int isxl = (int)(xs1[0]*np[0]-alim*deltx);

       for(int  iz=iszl; iz<=iszu; iz++)
       {
           for(int iy=isyl; iy<=isyu; iy++)
           {
               for(int  ix=isxl; ix<=isxu; ix++)
               {
                   float a1 = (float)ix/deltx-xs1[0]*(float)delt.x;
                   float a2 = (float)iy/delty-xs1[1]*(float)delt.y;
                   float a3 = (float)iz/deltz-xs1[2]*(float)delt.z;
                   float ray = a1*a1 + a2*a2 + a3*a3 + 2.f*a1*a2*cgChem +
                       2.f*a1*a3*cbChem + 2.f*a2*a3*caChem;
                   if(ray < DmaxSq) 
                   {
                      int ix1 = sidepp(ix,np[0]);
                      int ix2 = sidepp(iy,np[1]);
                      int ix3 = sidepp(iz,np[2]);
                      double gauss = due*exp(-ln2*ray/(raggioVDW*raggioVDW));
//così è più smussata
//                      mappa[ix1][ix2][ix3] += (float)gauss;
                      if(mappa[ix1][ix2][ix3] < gauss)
                         colormappa[ix1][ix2][ix3] = atm[ieq].Z-1;
                      mappa[ix1][ix2][ix3] = max(mappa[ix1][ix2][ix3],
                                               (float) gauss);
                   }                           
               }
           }
       }
   }
   for(int i=0; i<np[0]; i++)
   {
       for(int j=0; j<np[1]; j++)
           for(int k=0; k<np[2];k++)
           mappa[i][j][k] *= pRadius;
   }

   if(VWSurface.created)
   {
      VWSurface.freeMap();
      Properties::usedC[whichmod] = -1;
   }
   VWSurface.created = true;
   VWSurface.name = QString("VDWSurf");
   VWSurface.isSurface = true;
   VWSurface.multicolor = false;
   VWSurface.ifDifference = 1;
   VWSurface.nrefl = 0;
   VWSurface.resm = 0;
   VWSurface.FillMap(np, mappa, colormappa);
   for(int j=0;j<3;j++)
   {
       VWSurface.xmin[j] = (float) xcmin.v[j];
       VWSurface.xmax[j] = (float) xcmax.v[j];
   }
   VWSurface.radius = pRadius;
   VWSurface.step = step;
   
   delete [] mappa;
   delete [] colormappa;
   Surface_c = true;
}

#ifdef MAPS
void Model::SettaSurface(bool ifView)
{
   ifSurface = ifView;
   VWSurface.ifVis = ifView;
   jav->Draw();
}

void Model::RicalcolaSurface()
{
   float step = VWSurface.step;
   if(step < 1e-7)
      return;
   float radius = VWSurface.radius;
   if(radius < 1e-7)
      return;
   QColor c1 = VWSurface.color_mappa;
   int wire1 = VWSurface.wireframe;
   VWSurface.freeMap();
   //Properties::usedC[nModello] = -1;
   Attesa();
   CreaMappa(nModello+1, radius, step);
   VWSurface.ifVis = true;
   VWSurface.color_mappa = c1;
   VWSurface.wireframe = wire1;
   FineAttesa();
   jav->RunMCAll();
   jav->Draw();
}

void Model::DrawSurface(QOpenGLFunctions *f)
{
    VWSurface.DrawMap(f);
}

#endif

bool Model::IsSideChain(int n)
{
    return(!IsType(n,"N") && !IsType(n, "CA") && !IsType(n, "C")
       &&  !IsType(n, "CB") && !IsType(n, "O"));
}

bool Model::IsAromatic(int n)
{
    QStringList aro = {"HOH", "CD", "CD1", "CD2", "CE", "CE1", "CE2",
    "CE3", "CG", "CG1", "CG2", "CH2", "CH3", "CZ", "CZ2", "CZ3"};

    QString str = QString(atom_types.at(n).c_str());
    for( int i=0; i<aro.count(); i++ )
      if( !str.compare(aro.at(i), Qt::CaseInsensitive))
        return true;
    return false;
}

void Model::DeleteResidue(int which)
{
    int nr = c_alpha[which].residue;
    int ch = c_alpha[which].chain;
    for(int i=0; i<(int) Chains[ch].Residues[nr].atoms.size(); i++)
    {
        int n = Chains[ch].Residues[nr].atoms[i];
        atm[n].deleted = true;
    }
}

void Model::DeleteSideChain(int which)
{
    int nr = c_alpha[which].residue;
    int ch = c_alpha[which].chain;
    for(int i=0; i<(int) Chains[ch].Residues[nr].atoms.size(); i++)
    {
        int n = Chains[ch].Residues[nr].atoms[i];
        if(IsSideChain(n))
           atm[n].deleted = true;
    }
}

void Model::DeleteRing(int which)
{
    int nr = c_alpha[which].residue;
    int ch = c_alpha[which].chain;
    for(int i=0; i<(int) Chains[ch].Residues[nr].atoms.size(); i++)
    {
        int n = Chains[ch].Residues[nr].atoms[i];
        if(IsAromatic(n))
           atm[n].deleted = true;
    }
}

QList<QStandardItem *> *Model::FillListSphere(int nrows, int atomi[],
                                       float vdist[], op_type vsym[])
{
    QList<QStandardItem *>  *lista = new QList<QStandardItem *> [nrows]();
    for(int i=0; i<nats; i++)
        if(atm[i].drawLabel < 2)
            atm[i].drawLabel = 0;
    atm[Properties::sphereCentre-1].drawLabel = 1;
    for(int i=0; i<nrows; i++)
    {
       int ii = atomi[i] - 1;
       atm[ii].drawLabel = 1;
       QStandardItem *item = new QStandardItem();
       item->setData(ii, Qt::DisplayRole);
       lista[i].append(item);
       item = new QStandardItem(QString(atm[ii].lab.c_str()).trimmed());
       lista[i].append(item);
       item = new QStandardItem();
       item->setData(vdist[i], Qt::DisplayRole);
       lista[i].append(item);
       int ko = vsym[i].op;
       int tra[3] = {vsym[i].tra[0], vsym[i].tra[1], vsym[i].tra[2]};
       QString label2 = QString(Properties::getStringSymm(&ko)).trimmed();

       if(tra[0] + tra[1] + tra[2])
           label2.append(QString("+(%1,%2,%3)").arg(tra[0]).arg(tra[1]).arg(tra[2]));
       item = new QStandardItem(label2);
       lista[i].append(item);
    }
    return lista;
}

void Model::CopyAtoms(jav_atom_type *atml, int natoms)
{
    atm = new jav_atom_type  [natoms]();
    for(int i=0; i<natoms; i++)
        atm[i] = atml[i];
    nats = natoms;
}

void Model::CopyAtomsAU(jav_atom_type *atml, int natoms)
{
    atm_au = new jav_atom_type  [natoms]();
    nats_au = natoms;
    for(int i=0; i<natoms; i++)
        atm_au[i] = atml[i];
}

void Model::CopySpecies(QString species[], int nspecies)
{
    numSpec = nspecies;
    for(int i=0; i<numSpec; i++)
        LabSpec[i] = species[i];
}

void Model::ClearAtomsSelection()
{
    int natoms = min(nats_au, nats);
    for(int i=0; i<natoms; i++)
    {
        if(Equivs[i].empty())
           continue;
        int neq = (int) Equivs[i].size();
        for(int keq=0; keq<neq; keq++)
        {
            int ieq = Equivs[i].at(keq);
            atm[ieq].selected = false;
        }
    }
    atoms_sel.clear();
}

void Model::SelezionaAtomo(int row)
{
    if(atm == nullptr)
    {
        qDebug() << "no atoms";
        return;
    }

    int natoms = min(nats_au, nats);
    if(row >= natoms)
       return;
    if(Equivs[row].empty())
       return;
    int neq = (int) Equivs[row].size();
    for(int keq=0; keq<neq; keq++)
    {
        int ieq = Equivs[row].at(keq);
        atm[ieq].selected = true;
        atoms_sel.push_back(ieq+1);
    }
}

void Model::ClearBondsSelection()
{
    for(int i=0; i<nconn_prot; i++)
        bonds[i].selected = false;
    bonds_sel.clear();
}

void Model::SelezionaBond(int which, int nn1, int nn2)
{
    if(bonds == nullptr)
    {
        qDebug() << "no bonds";
        return;
    }
    if((nn1 == -1) || (nn2 == -1))
    {
        bonds[which].selected = true;
        bonds_sel.push_back(which+1);
    }
    else
    {
        for(int i=0; i<nconn_prot; i++)
        {
            int n1 = bonds[i].n1;
            int n2 = bonds[i].n2;
            int n1eq = iEqui[n1];
            int n2eq = iEqui[n2];
            if((n1eq == nn1) && (n2eq == nn2))
            {
               bonds[i].selected = true;
               bonds_sel.push_back(i+1);
            }
        }
    }
}

void Model::FillSite(QVector<QVector<int> > &site)
{      
    const float TOL = 0.01;
    std::vector<bool> siteok(nats_au, false);
    int nsite = -1;
    site.clear();
    for (int i = 0; i < nats_au; i++) {
        if (!siteok[i]) {
            nsite++;
            QVector<int> vec;
            vec.push_back(i);
            if (atm_au[i].och < 1.0) {
                for (int j=i+1; j < nats_au; j++) {                                        
                    if (distanzaCart(i,j) < TOL) {
                        vec.push_back(j);
                        siteok[j] = true;
                    }
                }
            }
            site.push_back(vec);
            qInfo() << "Site:" << site[nsite];
        }
    }
}

int Model::FindSpec(QString &spec)
{
    QString spect = spec.trimmed();
    for(int i=0; i<numSpec; i++)
    {
        QString spec1t = LabSpec[i].trimmed();
        if(spect.compare(spec1t, Qt::CaseInsensitive)  == 0)
           return (ptab[i]);
    }
    return 0;
}

atom_type *Model::FromListToAtom(QList<QStandardItem *> *lista, int *natoms)
{
    int nelems = *natoms;
    atom_type *atml = new atom_type [nelems]();
    for(int i=0; i<nelems; i++)
    {
        QStandardItem *item = lista[i].at(0);
        int ieq = item->data(Qt::DisplayRole).toInt() - 1;
        item = lista[i].at(1);
        char *lab  = item->data(Qt::DisplayRole).toString().toUtf8().data();
        for(int k=0;k<min(16, (int)strlen(lab)); k++)
            atml[ieq].lab[k] = lab[k];
        item = lista[i].at(2);
        QString spec = item->data(Qt::DisplayRole).toString();
        atml[ieq].ptab = FindSpec(spec);
        item = lista[i].at(3);
        atml[ieq].inte = item->data(Qt::DisplayRole).toFloat();
        for(int k=0; k<3;k++)
        {
            item = lista[i].at(4+k);
            atml[ieq].xc[k] = item->data(Qt::DisplayRole).toFloat();
        }
        item = lista[i].at(7);
        atml[ieq].biso = item->data(Qt::DisplayRole).toFloat();
        item = lista[i].at(8);
        atml[ieq].och = item->data(Qt::DisplayRole).toFloat();
    }
    return atml;
}

f_bond_type *Model::FromListToLegm(QList<QStandardItem *> *lista, int *nbonds)
{
    int nelems = *nbonds;
    f_bond_type *legs = new f_bond_type [nelems]();
    for(int i=0; i<nelems; i++)
    {
        QStandardItem *item = lista[i].at(0);
        int ieq = item->data(Qt::DisplayRole).toInt() - 1;
        legs[ieq].n1 = bonds[ieq].n1;
        legs[ieq].n2 = bonds[ieq].n2;
        legs[ieq].sigma = bonds[ieq].sigma;
        legs[ieq].ord = bonds[ieq].ordleg;
        item = lista[i].at(1);
        legs[ieq].dist = item->data(Qt::DisplayRole).toFloat();
    }
    return legs;
}

jav_anglebond_type *Model::FromListToAngs(QList<QStandardItem *> *lista, int *nangs)
{
    int nelems = *nangs;
    jav_anglebond_type *angs = new jav_anglebond_type [nelems]();
    for(int i=0; i<nelems; i++)
    {
        QStandardItem *item = lista[i].at(0);
        int ieq = item->data(Qt::DisplayRole).toInt() - 1;
        angs[ieq].n1 = anglebonds[ieq].n1;
        angs[ieq].n2 = anglebonds[ieq].n2;
        angs[ieq].n3 = anglebonds[ieq].n3;
        item = lista[i].at(1);
        angs[ieq].fv = item->data(Qt::DisplayRole).toFloat();
        angs[ieq].sigma = anglebonds[ieq].sigma;
    }
    return angs;
}

jav_torsion_type *Model::FromListToTors(QList<QStandardItem *> *lista, int *ntors)
{
    int nelems = *ntors;
    jav_torsion_type *angs= new jav_torsion_type [nelems]();
    for(int i=0; i<nelems; i++)
    {
        QStandardItem *item = lista[i].at(0);
        int ieq = item->data(Qt::DisplayRole).toInt() - 1;
        angs[ieq].n1 = torsions[ieq].n1;
        angs[ieq].n2 = torsions[ieq].n2;
        angs[ieq].n3 = torsions[ieq].n3;
        angs[ieq].n4 = torsions[ieq].n4;
        item = lista[i].at(1);
        angs[ieq].fv = item->data(Qt::DisplayRole).toFloat();
    }
    return angs;
}

atom_type *Model::FromJavatomToAtom()
{
    atom_type *atml = new atom_type [nats]();
    for(int i=0; i<nats_au; i++)
    {
        int ieq = Equivs[i].at(0);
        for(int j=0; j<3; j++)
            atml[ieq].xc[j] = atm[ieq].xyzc[j];
        atml[ieq].biso = atm[ieq].B;
        atml[ieq].och = atm[ieq].och;
        atml[ieq].ocry = atm[ieq].ocry;
        atml[ieq].nz = Species[atm[ieq].color];
        int klun = (int) atm[ieq].lab.length();
        for(int j=0; j<min(16,klun); j++)
            atml[ieq].lab[j] = atm[ieq].lab[j];
        for(int j=klun; j<16; j++)
            atml[ieq].lab[j] = ' ';
    }
    return atml;
}

f_bond_type *Model::FromJavbondToLegm()
{
    f_bond_type *legm = new f_bond_type [nconn_prot]();
    for(int i=0; i<nconn_prot; i++)
    {
        legm[i].n1 = bonds[i].n1;
        legm[i].n2 = bonds[i].n2;
        legm[i].dist = bonds[i].dist;
        legm[i].sigma = bonds[i].sigma;
        legm[i].ord = bonds[i].ordleg;
    }
    return legm;
}

QList<QStandardItem *> *Model::FillBondValence(float *bvs, int *cn)
{
    QList<QStandardItem *> *bvList = new QList<QStandardItem *> [nats_au]();
    for (int i = 0; i < nats_au; i++) {
         QStandardItem *item = new QStandardItem();
         item->setData(i+1, Qt::DisplayRole);
         bvList[i].append(item);
         item = new QStandardItem(QString(atm_au[i].lab.c_str()));
         bvList[i].append(item);
         if (atm_au[i].color >= 0)
             item = new QStandardItem(QString(LabSpec[atm_au[i].color]));
         else
             item = new QStandardItem("Q");
         bvList[i].append(item);
         item = new QStandardItem();
         item->setData(bvs[i],Qt::DisplayRole);
         bvList[i].append(item);
         item = new QStandardItem();
         item->setTextAlignment(Qt::AlignRight);
         item->setData(cn[i],Qt::DisplayRole);
         bvList[i].append(item);
    }
    return bvList;
}

QList<QStandardItem *> *Model::FillBondValenceInfo(int ninfo, bond_info_t *dinfo)
{
    QList<QStandardItem *> *infoList = new QList<QStandardItem *> [ninfo]();
    for (int i = 0; i < ninfo; i++) {
        QStandardItem *item = new QStandardItem();
        item->setData(i+1, Qt::DisplayRole);
        infoList[i].append(item);
        int n1 = dinfo[i].at2 - 1;
        item = new QStandardItem(QString(atm_au[n1].lab.c_str()));
        infoList[i].append(item);
        if (atm_au[n1].color >= 0)
            item = new QStandardItem(QString(LabSpec[atm_au[n1].color]));
        else
            item = new QStandardItem("Q");
        infoList[i].append(item);
        item = new QStandardItem();
        item->setData(dinfo[i].xat2[0],Qt::DisplayRole);
        infoList[i].append(item);
        item = new QStandardItem();
        item->setData(dinfo[i].xat2[1],Qt::DisplayRole);
        infoList[i].append(item);
        item = new QStandardItem();
        item->setData(dinfo[i].xat2[2],Qt::DisplayRole);
        infoList[i].append(item);
        int tra[3] = {dinfo[i].op2.tra[0], dinfo[i].op2.tra[1], dinfo[i].op2.tra[2]};
        QString label = QString(Properties::getStringSymm(&dinfo[i].op2.op)).trimmed();
        if(tra[0] + tra[1] + tra[2])
            label.append(QString("+(%1,%2,%3)").arg(tra[0]).arg(tra[1]).arg(tra[2]));
        item = new QStandardItem(label);
        infoList[i].append(item);
        item = new QStandardItem();
        item->setData(dinfo[i].dist,Qt::DisplayRole);
        infoList[i].append(item);
        item = new QStandardItem();
        item->setData(dinfo[i].bv,Qt::DisplayRole);
        infoList[i].append(item);
    }
    return infoList;
}

QList<QStandardItem *> *Model::FillBondValenceParameters(int n, bond_valence_t *bvec) const
{
    QList<QStandardItem *> *bvParList = new QList<QStandardItem *> [n]();
    for (int i=0; i < n; i++){
        QStandardItem *item = new QStandardItem(get_nth_element(bvec[i].z1-1));
        item->setFlags(item->flags() &= ~Qt::ItemIsEditable);
        bvParList[i].append(item);
        item = new QStandardItem();
        item->setTextAlignment(Qt::AlignRight);
        item->setData(bvec[i].val1, Qt::DisplayRole);
        bvParList[i].append(item);
        item = new QStandardItem(get_nth_element(bvec[i].z2-1));
        item->setFlags(item->flags() &= ~Qt::ItemIsEditable);
        bvParList[i].append(item);
        item = new QStandardItem();
        item->setTextAlignment(Qt::AlignRight);
        item->setData(bvec[i].val2, Qt::DisplayRole);
        bvParList[i].append(item);
        item = new QStandardItem();
        item->setData(bvec[i].ro,Qt::DisplayRole);
        bvParList[i].append(item);
        item = new QStandardItem();
        item->setData(bvec[i].b,Qt::DisplayRole);
        bvParList[i].append(item);
        item = new QStandardItem();
        item->setData(bvec[i].rmin,Qt::DisplayRole);
        bvParList[i].append(item);
        item = new QStandardItem();
        item->setData(bvec[i].rmax,Qt::DisplayRole);
        bvParList[i].append(item);
    }

    return bvParList;
}
